use strict;
use warnings;
use utf8;
use File::Basename qw(dirname);
use File::Slurp qw(edit_file);
use File::Path qw(make_path);
use LWP::Simple qw($ua getstore);

my @js =
(
  "loader.js",
  "config.js",
  "js/api/jak.js", 
  "js/api/smap.js",
  "js/api/poi.js",
  "js/api/pano.js"
);

my @img =
(
  "img/api/logo.png",
  "img/api/plane.png",
  
  # ikonky map
  "img/api/preview-letecka.png",
  "img/api/preview-zakladni.png",
  "img/api/preview-turisticka.png",
  "img/api/preview-letecka0203.png",
  "img/api/preview-letecka0406.png",
  "img/api/preview-historicka.png",
  "img/api/preview-zimni.png",
  "img/api/preview-letni.png",
  "img/api/preview-zemepisna.png",

  # orientace
  "img/api/cw.png",
  "img/api/cw-smooth.png",
  "img/api/cw-smooth-hover.png",    
  "img/api/ccw.png",

  # směrová růžice  
  "img/api/rosette.png",

  # layer-switch
  "img/api/layer-switch/button.png",
  "img/api/layer-switch/l-arr.png",
  "img/api/layer-switch/r-arr.png",
  
  # compass
  "img/api/compass/compass-n.png",
  "img/api/compass/compass-e.png",
  "img/api/compass/compass-s.png",
  "img/api/compass/compass-w.png",
  "img/api/compass/compass-n-ie8.png",
  
  # card
  "img/api/card/close.png",
  "img/api/card/tail.png",

  # marker
  "img/api/marker/drop-red.png",
  "img/api/marker/drop-blue.png",
  "img/api/marker/drop-yellow.png",
  "img/api/marker/drop-shadow.png",
  "img/api/marker/cluster.png",

  # pano
  "img/api/pano/arrow.png",
  "img/api/pano/arrow-hover.png",
  "img/api/pano/arrow-active.png",
  "img/api/pano/panorama-beta.png",
  "img/api/pano/circle.png",
  "img/api/pano/circle-active.png",
  "img/api/marker/eye.png",
  #"img/api/panorama3D.swf", 
  "img/api/empty.gif",
  
  # poi
  "img/api/marker/empty-poi.png",
  "img/api/marker/advertHighlight.png",
  "img/api/bg_foto_stin.png",
  "img/api/marker/balloon-1.png",
  "img/api/marker/balloon-2.png",
  "img/api/marker/balloon-3.png",
  "img/api/traffic/traffic1.png",
  "img/api/traffic/traffic2.png",
  "img/api/traffic/traffic3.png",
  "img/api/traffic/traffic4.png",
  "img/api/traffic/traffic5.png"
);  

my @css =
(
  "css/api/api.css",
  "css/api/card.css",
  "css/api/poi.css",
  "css/api/pano.css"
);

my @font =
(
  "font/api/mapy.eot",
  "font/api/mapy.svg",
  "font/api/mapy.woff",
  "font/api/mapy.ttf"
);

# stažení všeho
download("http://api4.mapy.cz/$_", $_) foreach (@js, @img, @css, @font);

# fix image and font paths in CSS files
foreach (@css)
{
  edit_file { s|(?<!\.\.)\((/img/api)|(../..$1|g } $_;
  edit_file { s|(?<!\.\.)(/font/api)|(./../../..$1|g } $_;
}

# fix IE8 GeoSetter WinXP problem
edit_file { s/(1e32|1.0E32)/9e18/ig } "js/api/smap.js";

################################################################################

# comment unused stuff in smap.js
# -- doesn't work correctly, because some lines don't start with SMap.Coords
#edit_file { s|^(SMap\.Coords\..*)$|//$1|mg } "js/api/smap.js";

# úprava loader.js (v současnosti nepoužíváno)
# my $basepath = "\" + window.basePath + \"Seznam";
# my $put_inside = "[\"$basepath/js/api/jak.js\", \"$basepath/js/api/smap.js\", \"$basepath/config.js\"]";  # použití relativních linků v single mode
# edit_file { s/this._loadList\(list\);/this._loadList($put_inside);/ } "loader.js";
# edit_file { s/(\}\)\(\);$)/$1\n\nLoader.load();/ } "loader.js";
# edit_file { s/mode: \"single\"/mode: \"multi\"/ } "loader.js";                              # mód, kde se použijí nezkomprimované js soubory

################################################################################

# download (url, filename)
sub download
{
  my $url = shift;
  my $filename = shift;
  
  make_path(dirname($filename));
  print "$url\n" unless -e $filename;   

  $ua->agent('Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/42.0.2311.15 Safari/537.36');
  getstore($url, $filename) unless -e $filename;    
}