/**
 * @class Coordinate Grid
 * @group map-layers
 * @augments SMap.Layer
 */

SMap.Layer.Grid = JAK.ClassMaker.makeClass({
	NAME: "SMap.Layer.Grid",
	VERSION: "1.0",
	EXTEND: SMap.Layer
});

SMap.Layer.Grid.prototype.$constructor = function(id, options)
{
	this._ec = [];
	this.$super(id);

	this._options = {};
	for (var p in options) { this._options[p] = options[p]; }

  // vytvoen grid tabulek  
  this._gridTables();
  
  // vytvoen konstant
  this.VLEVO = 0;
  this.NAHORE = 1;
  this.VPRAVO = 2;
  this.DOLE = 3;    
  this.MENSI_SOURADNICE = 0;
  this.VETSI_SOURADNICE = 1;
  
  // vytvoen tabulky s polohou popisk na zklad orientace a toho, zda jde o lat i lon
  // 0, 1, 2, 3 ... orientace mapy
  // popisky ... zda budou popisky vlevo, nahoe, atd.
  // uPopisku ... zda je souadnice u popisku men nebo vt ne na opan stran obrazovky
  
  this.polohaPopisku =
  { 
    lat:
    {
      0: {popisky: this.VLEVO,   uPopisku: this.MENSI_SOURADNICE},
      1: {popisky: this.NAHORE, uPopisku: this.MENSI_SOURADNICE},   
      2: {popisky: this.VLEVO,   uPopisku: this.VETSI_SOURADNICE},   
      3: {popisky: this.NAHORE, uPopisku: this.VETSI_SOURADNICE}
    },
    
    lon:
    {
      0: {popisky: this.NAHORE, uPopisku: this.VETSI_SOURADNICE},
      1: {popisky: this.VLEVO,   uPopisku: this.MENSI_SOURADNICE},
      2: {popisky: this.NAHORE, uPopisku: this.MENSI_SOURADNICE},
      3: {popisky: this.VLEVO,   uPopisku: this.VETSI_SOURADNICE}
    }
  }

  // CSS pro ukotven popisk
  this.anchors = {};
  this.anchors[this.VLEVO] = {left: 0, top: 15};
  this.anchors[this.NAHORE] = {left: -15, top: 0};
  this.anchors[this.VPRAVO] = {left: 100, top: 15};
  this.anchors[this.DOLE] = {left: 15, top: 0};
  
  if (JAK.Browser.client == "ie" && JAK.Browser.version <= 8)    /* u IE8 pesunujeme bublinu a ne marker, je teba odlinch hodnot */
  {
    this.anchors[this.NAHORE] = {left: 30, top: 0};
    this.anchors[this.VPRAVO] = {left: 100, top: 25};
    this.anchors[this.DOLE] = {left: 25, top: 100};  
  } 
}

/**
* Vygenerovn tabulek s intervaly nebo souadnicemi k zakreslen
*/

// defaultn options pro line style na dku 370 zde:
// http://jak.seznam.cz/doc/symbols/src/tmp_szn-simpleweb-jak_www_simpleweb_jak_tmp_util_vector.js.html

SMap.Layer.Grid.prototype._gridTables = function()
{
  // tabulka podle kter se bude generovat s souadnic
  var deg = 1, min = 1/60, sec = 1/3600;
  
  this.gridTable =
  [         /* type: typ kter se pevede pomoc SMap.Coords.format, step: krok, color: barva ry, width: ka ry */
            /* #EE82EE = violet (IE8 fix), #00FF00 = lime */
  /* 00 */ {type: "deg", step: deg * 180,style: {color: "#EE82EE",width: 8, outlineWidth: 1}}, /* 0, 180 (Greenwich, Date Line, rovnk) */
  /* 01 */ {type: "deg", step: deg * 20, style: {color: "#00FF00",width: 6, outlineWidth: 1}}, /* 0, 20, 40, ... */  
  /* 02 */ {type: "deg", step: deg * 10, style: {color: "#00FF00",width: 6, outlineWidth: 1}}, /* 0, 10, 20, ... */  
  /* 03 */ {type: "deg", step: deg * 5,  style: {color: "#00FF00",width: 4, outlineWidth: 1}}, /* 0, 5, 10, ... */
  /* 04 */ {type: "deg", step: deg,      style: {color: "#00FF00",width: 4, outlineWidth: 1}}, /* 0, 1, 2, ... */
  /* 05 */ {type: "min", step: min * 30, style: {color: "red",   width: 3, outlineWidth: 0}},  /* 00', 30' */ 
  /* 06 */ {type: "min", step: min * 10, style: {color: "red",   width: 3, outlineWidth: 0}},  /* 00', 10', 20', ... */
  /* 07 */ {type: "min", step: min * 5,  style: {color: "red",   width: 2, outlineWidth: 0}},  /* 00', 05', 10', 15' ... */ 
  /* 08 */ {type: "min", step: min,      style: {color: "red",   width: 2, outlineWidth: 0}},  /* 00', 01', 02', 03' ... */ 
  /* 09 */ {type: "sec", step: sec * 30, style: {color: "blue",  width: 1, outlineWidth: 0}},  /* 00'', 30'' */ 
  /* 10 */ {type: "sec", step: sec * 10, style: {color: "blue",  width: 1, outlineWidth: 0}},  /* 00'', 10'', 20'', ... */
  /* 11 */ {type: "sec", step: sec * 5,  style: {color: "blue",  width: 1, outlineWidth: 0}},  /* 00'', 05'', 10'', 15'' ... */ 
  /* 12 */ {type: "sec", step: sec,      style: {color: "blue",  width: 1, outlineWidth: 0}}   /* 00'', 01'', 02'', 03'' ... */ 
  ];
  
  var commonStyleOptions = {outlineColor: "black"};   /* bude mt efekt jen na deg, protoe jinde je vude outlineWidth = 0 */
  
  for (var i = 0; i < this.gridTable.length; i++)
  {
    var options = this.gridTable[i];
    SMap.Util.mergeObject(commonStyleOptions, options.style);        /* k options pipojme dal nae options (id je pro CSS) */
    SMap.Util.mergeObject({id: i}, options);
  }  

  // ===============================================================
  // znm rovnobky: obratnky Raka a Kozoroha, severn a jin polrn kruh
  
  function createCoord(deg, min, sec) { return (deg + min / 60 + sec / 3600); };  /* pomocn funkce pro tvorbu souadnice z minut, stup a sekund */

  this.rovnobezkyOptions = {id: "rovnobezky", type: "sec", style: {color: "#9D6BC6",  width: 4, outlineWidth: 0, style: JAK.Vector.STYLE_DASH}};

  this.rovnobezky =
  [
    createCoord(23, 26, 14.675),   /* obratnk raka */,
    -createCoord(23, 26, 14.440),  /* obratnk kozoroha */
    createCoord(66, 33, 45.7),     /* severn polrn kruh */
    -createCoord(66, 33, 45.7)     /* jin polrn kruh */    
  ]
  
  // ===============================================================
  // znm polednky: Ferro
  this.polednikyOptions = {id: "poledniky", type: "sec", style: {color: "orange",  width: 1, outlineWidth: 0, style: JAK.Vector.STYLE_SOLID}};  

  this.poledniky = 
  [
  //  -createCoord(17, 39, 46)     /* Ferro */
  ];
}

SMap.Layer.Grid.prototype.enable = function()
{
  // pokud nen vrstva aktivn, tak ji enablujeme
  if (!this.isActive())
  {
    // vrstva pro souadnicovou s
    this._gridGeomLayer = new SMap.Layer.Geometry();
    this.getMap().addLayer(this._gridGeomLayer);
    this._gridGeomLayer.enable();

    // vrstva pro popisky    
    this._markerLayer = new SMap.Layer.Marker();
    this.getMap().addLayer(this._markerLayer);
    this._markerLayer.enable();  
  }
  
  return (this.$super());
}


SMap.Layer.Grid.prototype.disable = function()
{
  // pokud je vrstva aktivn, tak ji disablujeme
  if (this.isActive())
  {
    this._gridGeomLayer.disable();
    this.getMap().removeLayer(this._gridGeomLayer);
    
    this._markerLayer.disable();
    this.getMap().removeLayer(this._markerLayer);
  }
  
  return (this.$super());
}

SMap.Layer.Grid.prototype.clear = function()
{
  // pokud je vrstva aktivn, tak ji vyistme
  if (this.isActive())
  {
    this._gridGeomLayer.removeAll();
    this._markerLayer.removeAll();
    
    // vytvome promnnou, kam se budou ukldat souadnice jednotlivch vykreslench ar
    // -- slou k zabrnn vykreslen njak u existujc ry nebo popisku pro stejnou ru
    this.addedMarkerCoords = [];    
  }
  
  return (this.$super());
}

/**
* pomocn funkce, kter nm zkonstruuje souadnice, piem posledn parametr bude zda je prvn parametr Lat nebo Lon
* @param {float} coord1 Prvn parametr (latitude nebo longitude)
* @param {float} coord2 Druh parametr (longitude nebo latitude)
* @param {boolean} latFirst Zda je prvn parametr latitude nebo longitude
  @returns {SMap.Coords} souadnice Vsledn souadnice
*/

SMap.Layer.Grid.prototype._constructCoords = function(coord1, coord2, latFirst)
{
  return (SMap.Coords.fromWGS.apply(this, latFirst ? [coord1, coord2] : [coord2, coord1]));  
}

/**
* Vykreslen ry samotn
* @param {float} minCoord1 Zaten souadnice (lat nebo lon)
* @param {float} maxCoord1 Koncov souadnice (lat nebo lon)
* @param {float} Coord2 Druh souadnice, kter bude konstantn (lon nebo lat)
* @param {boolean} isLat Zda je konstruovan ra latitude nebo longitude
* @param {Object} options Nastaven vzhledu ry
*/

SMap.Layer.Grid.prototype._drawCoordinateLine2 = function(minCoord1, maxCoord1, Coord2, isLat, options)
{
  var p1 = this._constructCoords(Coord2, minCoord1, isLat);
  var p2 = this._constructCoords(Coord2, maxCoord1, isLat);

  var minMaxDiff = maxCoord1 - minCoord1;         /* rozmez souadnice kde budeme vykreslovat ru */

  // vykreslme rovnobku, resp. polednk
  // -- a to tak, e si ru rozdlme na hodn mench dlk
  // -- tm se zajist, e i zakiven kivky se zakresl vrn
  // -- tak se tm zajist, e i kdy jsou okraje nedefinovan, vykresl se aspo st (viz ni mtka S-JTSK)
  var body = [];                   /* pole bod */
  var step = minMaxDiff / 50;      /* krok */
  
  for (var i = minCoord1; i < maxCoord1; i += step)
  {
    body.push(this._constructCoords(Coord2, i, isLat));
  }
    
  body.push(p2);                   /* pidme koncov bod */
  
  var polyline = new SMap.Geometry(SMap.GEOMETRY_POLYLINE, null, body, options.style);
  this._gridGeomLayer.addGeometry(polyline);
}

/**
* Formtovn souadnic pro bublinu na kraji obrazovky
* @param {SMap.Coords} cooods Souadnice popisku
* @param {boolean} isLat Zda chceme tisknout latitude nebo longitude
* @param {object} options Monosti tisku
* @returns {String} Vytisknut etzec
*/

SMap.Layer.Grid.prototype._vytiskniSouradnici = function(coords, isLat, options)
{
  return coords.format("WGS84", options.type, 0, {latOnly: isLat, lonOnly: !isLat /* zda tisknout Lat nebo Lon */});
}

/**
* Vykreslen souadnicov ry a popisku na n
* @param {float} minCoord1 Zaten souadnice (lat nebo lon)
* @param {float} maxCoord1 Koncov souadnice (lat nebo lon)
* @param {float} Coord2 Druh souadnice, kter bude konstantn (lon nebo lat)
* @param {boolean} isLat Zda je konstruovan ra latitude nebo longitude
* @param {Object} options Nastaven vzhledu ry
*/

SMap.Layer.Grid.prototype._drawCoordinateLine = function(minCoord1, maxCoord1, Coord2, isLat, options)
{
  var map = this.getMap();
  var o = map.getOrientation();
  var size = map.getSize(); /* rozmry mapy v pixelech */

  var p1 = this._constructCoords(Coord2, minCoord1, isLat);      /* koncov body ry */
  var p2 = this._constructCoords(Coord2, maxCoord1, isLat);

  var minMaxDiff = maxCoord1 - minCoord1;                        /* rozmez souadnice kde budeme vykreslovat ru */

  // ===============================================================
  // projdeme u jednou pidan ry a pokud jsme u tuto pidali (v menm mtku), pak ji nepidvme znovu
  for (var i = 0; i < this.addedMarkerCoords.length; i++)
  {
    if (this.addedMarkerCoords[i].isLat === isLat                     /* pokud se jedn znovu o lat nebo lon */
      && this.addedMarkerCoords[i].coords.distance(p1) < 0.001)   /* a pokud jsou souadnice stejn */
      return;
  }
  
  // pidme aktuln ru do pole, kter ve kontrolujeme
  this.addedMarkerCoords.push({coords: p1, isLat: isLat});

  // ===============================================================
  // vykreslen samotn ry
  this._drawCoordinateLine2.apply(this, arguments);

  // ===============================================================
  // v zvislosti na orientaci a tom, zda hledme lat i lon nkdy musme prohodit p1 a p2, abychom marker umstili na opanou stranu
  // -- tm doshneme toho, e bubliny budou msto dole nahoe atd.

  this.popisky = this.polohaPopisku[isLat ? "lat" : "lon"][o].popisky;       // vlevo, nahoe, ...
  this.uPopisku = this.polohaPopisku[isLat ? "lat" : "lon"][o].uPopisku;     // u popisku je men souadnice, vt souadnice
  
  // pokud nechceme dlat bublinu u prvnho bodu (protoe chceme nap. bublinu nahoe a ne dole), tak body prohodme
  if (this.uPopisku == this.VETSI_SOURADNICE)
  {
    var tmp = p1;
    p1 = p2;
    p2 = tmp;
  }

  // vytvome marker (ke ktermu pak pipojme bublinu)
  // -- ten ukotvme podle orientace a toho, zda hledme lat i lon
  var coords = p1;
  var marker = new SMap.Marker(coords, null, {url: JAK.mel("div"), anchor: this.anchors[this.popisky]});
  
  // ===============================================================  
  // vytvome bublinu
  var bubble = new SMap.Bubble({alwaysShow: true, className: "gridBubble " + options.type + " id_" + options.id});
  marker.decorate(SMap.Marker.Feature.Bubble, bubble);

  html = "<span>" + this._vytiskniSouradnici(coords, isLat, options) + "</span>";
  bubble.setHTML(html);

  // pidme marker s bublinou do vrstvy marker
  this._markerLayer.addMarker(marker);

  // ===============================================================
  // bublinu orotujeme, aby byla ve smru souadnicov st (tam, kde nejsou okraje mapy a souadnicov s rovnobn)
  // -- jako hel vezmeme azimut teny
  // -- druh bod je blzko tomu prvnmu, na stejn rovnobce / polednku
  // -- ale ne moc blzko, jinak by byl pixelov rozdl moc mal a pi operacch s pixely vznikaj nepesnosti
  var azimuth = p1.pixelAzimuth(map, this._constructCoords(Coord2, minCoord1 + minMaxDiff / 10, isLat));
  azimuth += (this.popisky == this.VPRAVO) ? 180 : 0;     /* aby nebyla bublina vzhru nohana */
  
  if (JAK.Browser.client == "ie" && JAK.Browser.version <= 8)           /* v IE8 je teba orotovat bublinu, protoe rotace markeru nefunguje */
    rotateElement(bubble.getDom(), azimuth);                            /* -- nen to ale ideln, je teba pak trochu jin hodnoty v this.anchors */
  else
    rotateElement(marker.getContainer()[SMap.LAYER_MARKER], azimuth);   /* vude jinde posunujeme marker */

  // ===============================================================
  // bubliny nemus bt hezky pod sebou, ale mou vyboovat, a to i mimo mapu (tam, kde nejsou okraje mapy a souadnicov s rovnobn)
  // -- je teba kadou bublinu posunout tak, aby byla konstantn vzdlenost od okraje mapy 
  // -- vyuijeme pevodu pmky spojujc oba konce mapy do parametrickho tvaru X = A + t * (B-A)
  // -- nsledn za t dosadme vzdlenost, o kterou chceme bublinu posunou doprava a X jsou nae vsledn souadnice
  var A = p1.toPixel(map);                /* poten bod A */
  var BA = p2.toPixel(map).minus(A);      /* vektor B-A */
  BA.scale(1 / BA.norm());                /* vydlen normou, aby ml velikost 1 */

  var xy = (this.popisky == this.VLEVO || this.popisky == this.VPRAVO) ? "x" : "y"; /* pokud jsou popisky vlevo nebo vpravo, bereme x, jinak y */
  var minus = (this.popisky == this.DOLE || this.popisky == this.VPRAVO) ? -1 : 1;  /* odlin chovn pro popisky vpravo a dole */
    
  var odsazeni = A[xy] + minus * size[xy] / 2;              /* vzdlenost (v pixelech) markeru s bublinou od levho okraje mapy */
  BA.scale(-odsazeni);                                      /* vektor B-A vynsobme pixelovou vzdlenost o kterou chceme A posunout */
  var X = A.clone()[minus == -1 ? "minus" : "plus"](BA);    /* X = A + t * (B-A), v ppad popisk vpravo i dole pak X = A - t * (B-A) */
  var newCoords = X.toCoords(map);
  
  // nkdy pevod na souadnice vyhod null
  // -- stv se to v ptam pohledu, na okraji oblasti, pi zapnutch zempisnch souadnicch
  // -- sele funkce _absolutePixelToCoords, vrac null
  if (newCoords === null)
  {
    this._markerLayer.removeMarker(marker);  
    return;
  }
  
  marker.setCoords(newCoords);      /* nastavme nov souadnice */
  
  // ===============================================================
  // pokud je bublina ve smru svislm pl blzko okraje, me se pekrvat s bublinou ve smru vodorovnm
  // -- proto vechny svisl bubliny pli blzko kraje odstranme 

  if (this.popisky == this.NAHORE || this.popisky == this.DOLE)
  {
    if (Math.abs(Math.abs(X.x) - size.x/2) < 100) /* vzdlenost od levho a pravho okraje nesm bt men ne konstanta */
      this._markerLayer.removeMarker(marker);    
  }
}

/**
* Zsk bounding box roh mapy (dvme sem, abychom tdu mohli pepsat pro jinou projekc)
* @returns {Object} bBox Bounding Box
*/

SMap.Layer.Grid.prototype._getBoundingBox = function()
{
  return SMap.Coords.mapCorners(this.getMap());
}


SMap.Layer.Grid.prototype.redraw = function(full)
{
	if (!this.isActive())
    return;

  // vyistme minule nakreslen ry
  this.clear();

  var map = this.getMap();
  var o = map.getOrientation();

  // zskme rozmry a rohy mapy
  var size = map.getSize();
  var b = this._getBoundingBox();

  // projdeme tabulku, podle kter se bude generovat s souadnic
  // -- musme jt v tabulce od 0, protoe duplicity se odstrauj (viz promnn this.addedMarkerCoords)
  for (var i = 0; i < this.gridTable.length; i++)
  {
    var options = this.gridTable[i];
    var step = options.step;                                        /* krok, nap. po 1 minut */

    // zjistme, zda by nebylo ar v tomto rozlien pli mnoho (pokud ano, tak nevykreslujeme)
    var numLat = (b.maxLat - b.minLat) / step;                      /* poet ar ek a dlek */
    var numLon = (b.maxLon - b.minLon) / step;
    
    var pixDist1 = size[(o == 0 || o == 2) ? "y" : "x"] / numLat;   /* poet pixel pipadajc na vzdlenost mezi dvma kami nebo dlkami */
    var pixDist2 = size[(o == 0 || o == 2) ? "x" : "y"] / numLon;   /* -- kter rozmr mapy dlme potem ek i dlek zvis na orientaci */

    if (Math.min(pixDist1, pixDist2) < 70 /*px*/)                   /* pokud je pixelov rozestup men ne nmi zvolen konstanta, tak konme */
      continue;
    
    // draw latitude lines
    for (var j = Math.round(b.minLat / step) * step; j <= b.maxLat; j += step)
      this._drawCoordinateLine(b.minLon, b.maxLon, j, true /*isLat*/, options);
    
    // draw longitude lines
    for (var j = Math.round(b.minLon / step) * step; j <= b.maxLon; j += step)
      this._drawCoordinateLine(b.minLat, b.maxLat, j, false /*isLat*/, options);      
  }
  
  // ===============================================================
  // *** vykreslme znm rovnobky: obratnky raka a kozoroha, severn a jin polrn kruh ***
  for (var i = 0; i < this.rovnobezky.length; i++)
  {
    if (b.minLat <= this.rovnobezky[i] && b.maxLat >= this.rovnobezky[i])
      this._drawCoordinateLine(b.minLon, b.maxLon, this.rovnobezky[i], true /*isLat*/, this.rovnobezkyOptions);
  };
  
  // ===============================================================
  // *** vykreslme znm polednky: Ferro ***
  for (var i = 0; i < this.poledniky.length; i++)
  {
    if (b.minLon <= this.poledniky[i] && b.maxLon >= this.poledniky[i])
      this._drawCoordinateLine(b.minLat, b.maxLat, this.poledniky[i], false /*isLat*/, this.polednikyOptions); 
  };
  
  return;
}

/**
 * @class Coordinate Grid (S-JTSK)
 * @group map-layers
 * @augments SMap.Layer.Grid
 */

SMap.Layer.Grid.SJTSK = JAK.ClassMaker.makeClass({
	NAME: "SMap.Layer.Grid.SJTSK",
	VERSION: "1.0",
	EXTEND: SMap.Layer.Grid
});


SMap.Layer.Grid.SJTSK.prototype.$constructor = function(id, options)
{
	this.$super(id, options);
  
  this.polohaPopisku =
  { 
    lat:
    {
      0: {popisky: this.VLEVO,   uPopisku: this.VETSI_SOURADNICE},   
      1: {popisky: this.NAHORE, uPopisku: this.VETSI_SOURADNICE},   
      2: {popisky: this.VLEVO,   uPopisku: this.MENSI_SOURADNICE},   
      3: {popisky: this.NAHORE, uPopisku: this.MENSI_SOURADNICE}
    },
        
    lon:
    {
      0: {popisky: this.NAHORE, uPopisku: this.MENSI_SOURADNICE},
      1: {popisky: this.VLEVO,   uPopisku: this.VETSI_SOURADNICE},
      2: {popisky: this.NAHORE, uPopisku: this.VETSI_SOURADNICE},
      3: {popisky: this.VLEVO,   uPopisku: this.MENSI_SOURADNICE}
    }
  }
}

SMap.Layer.Grid.SJTSK.prototype._gridTables = function()
{
  // tabulka podle kter se bude generovat s souadnic
  this.gridTable =
  [
  /* 00 */ {step: 2e6, style: {color: "yellow",  width: 5, outlineWidth: 1, outlineColor: "black"}},  
  /* 01 */ {step: 1e6, style: {color: "yellow",  width: 5, outlineWidth: 1, outlineColor: "black"}}, 
  /* 02 */ {step: 2e5, style: {color: "#FFD9FE", width: 5, outlineWidth: 1, outlineColor: "black"}},    
  /* 03 */ {step: 1e5, style: {color: "#FFD9FE", width: 5, outlineWidth: 1, outlineColor: "black"}},
  /* 04 */ {step: 2e4, style: {color: "#EFFF9E", width: 4, outlineWidth: 1, outlineColor: "black"}},    
  /* 05 */ {step: 1e4, style: {color: "#EFFF9E", width: 4, outlineWidth: 1, outlineColor: "black"}},
  /* 06 */ {step: 2e3, style: {color: "#C7FFE8", width: 3, outlineWidth: 1, outlineColor: "black"}},    
  /* 07 */ {step: 1e3, style: {color: "#C7FFE8", width: 3, outlineWidth: 1, outlineColor: "black"}},
  /* 08 */ {step: 2e2, style: {color: "#FCD9D4", width: 2, outlineWidth: 1, outlineColor: "black"}},    
  /* 09 */ {step: 1e2, style: {color: "#FCD9D4", width: 2, outlineWidth: 1, outlineColor: "black"}},
  /* 10 */ {step: 2e1, style: {color: "#D2CAFC", width: 1, outlineWidth: 0, outlineColor: "black"}},    
  /* 11 */ {step: 1e1, style: {color: "#D2CAFC", width: 1, outlineWidth: 0, outlineColor: "black"}}        
  ];
  
  var commonStyleOptions = {};
  
  for (var i = 0; i < this.gridTable.length; i++)
  {
    var options = this.gridTable[i];
    SMap.Util.mergeObject(commonStyleOptions, options.style);    /* k options pipojme dal nae options (id je pro CSS) */
    SMap.Util.mergeObject({id: i + "_sjtsk"}, options);
  }
  
  // dleit rovnobky a polednky
  this.rovnobezky = [];
  this.poledniky = [];  
}

SMap.Layer.Grid.SJTSK.prototype._getBoundingBox = function()
{
  var c = SMap.Coords.mapCorners(this.getMap());
  var c1 = c.topLeft.toSJTSK();
  var c2 = c.topRight.toSJTSK();
  var c3 = c.bottomLeft.toSJTSK();
  var c4 = c.bottomRight.toSJTSK();
  
  // zjistme minima a maxima X a Y
  var minX = Math.min(c1.x, c2.x, c3.x, c4.x);
  var minY = Math.min(c1.y, c2.y, c3.y, c4.y);
  var maxX = Math.max(c1.x, c2.x, c3.x, c4.x);
  var maxY = Math.max(c1.y, c2.y, c3.y, c4.y);

  // vsledn objekt
  var bBox =
  {
    minLat: minX, minLon: minY, maxLat: maxX, maxLon: maxY
  };
  
  return bBox;
}

SMap.Layer.Grid.SJTSK.prototype._constructCoords = function(coord1, coord2, latFirst)
{
  // pouijeme SJTSK
  return (SMap.Coords.fromSJTSK.apply(this, latFirst ? [coord2, coord1] : [coord1, coord2]));  
}

/**
* Formtovn souadnic pro bublinu na kraji obrazovky
*/

SMap.Layer.Grid.SJTSK.prototype._vytiskniSouradnici = function(coords, isLat, options)
{
  var xy = coords.toSJTSK();

  var coord = Math.round(isLat ? xy.y : xy.x);
  coord = coord.toString().replace(/(\d)(?=(\d{3})+$)/g, '$1 ')    /* dme nulu po kad tet slici odzadu */  
 
  var ret = (isLat ? "X" : "Y") + " " + coord;
  return ret;
}

SMap.Layer.Grid.SJTSK.prototype._drawCoordinateLine = function(minCoord1, maxCoord1, Coord2, isLat, options)
{
  // prohodme isLat
  this.$super(minCoord1, maxCoord1, Coord2, !isLat, options);
  return;
}


//--------------------------------------------------

function load_coordinate_grid(unload)
{
  if (!unload /*= load*/)
  {
    window.gridLayer = new SMap.Layer.Grid("gridLayer");
    m.addLayer(gridLayer);
    gridLayer.enable();
  }
  
  else if (unload && window.gridLayer)
  {
    gridLayer.disable();
    m.removeLayer(gridLayer);
  }
}

function load_coordinate_grid_sjtsk(unload)
{
  if (!unload /*= load*/)
  {
    window.gridSJTSKLayer = new SMap.Layer.Grid.SJTSK("gridSJTSKLayer");
    m.addLayer(gridSJTSKLayer);
    gridSJTSKLayer.enable();    
  }
  
  else if (unload && window.gridSJTSKLayer)
  {
    gridSJTSKLayer.disable();
    m.removeLayer(gridSJTSKLayer);    
  }
}

window.coordinate_grid = true;
window.coordinate_grid_sjtsk = true;