/*
   EventUtil is cross browser event handling object
   source is taken from book "Pro JavaScript for Web Developers, 3rd Edition"
*/ 
var EventUtil = 
{
  DOMReady: function(f) 
  {
    if (document.addEventListener) 
      document.addEventListener("DOMContentLoaded", f, false);
    else 
      window.setTimeout(f, 0);
  },

  addHandler: function(element, type, handler, capture) 
  {
    // element is null/undefined? return
    if (!element)
      return;    

    // encapsulate handler
    var handlerFn = function(e)
    {
      var e = EventUtil.getEvent(e);
      var target = EventUtil.getTarget(e);
      return (handler(e, target));
    };

    // add listener
    if (element.addEventListener) 
      element.addEventListener(type, handlerFn, !!capture);
    else if (element.attachEvent && !capture) 
      element.attachEvent('on' + type, handlerFn);
    else if (!capture) 
      element['on' + type] = handlerFn;
  
    return [element, type, handlerFn, !!capture];
  },
  
  removeHandler: function(element, type, handler, capture)
  {
    if (arguments.length == 1)
        return(EventUtil.removeHandler.apply(this, element));

    if (element.removeEventListener)
      element.removeEventListener(type, handler, !!capture);
    else if (element.detachEvent)
      element.detachEvent('on' + type, handler);
    else
      element['on' + type] = null;
  },  
 
  getEvent: function(e) {return e || window.event;},
  getTarget : function(e) {return e.target || e.srcElement;},
  preventDefault: function(e) {e.preventDefault ? e.preventDefault() : e.returnValue = false;},
  stopPropagation: function(e) {e.stopPropagation ? e.stopPropagation() : e.cancelBubble = true;},
  
  // mouseover / mouseout
  getRelatedTarget: function(e){var ret = e.relatedTarget || e.toElement || e.fromElement; return (ret ? ret : null);},

  addMouseHandler: function(element, type, handler, capture)
  {
    // info: http://www.quirksmode.org/js/events_mouse.html
    // info: http://blog.stchur.com/2007/03/15/mouseenter-and-mouseleave-events-for-firefox-and-other-non-ie-browsers/
    if (type == "mouseenter" || type == "mouseleave")     // nefunguje v Maxthonu 3 (safari), starších verzích Chromu
    {
      var mouseEnterSupport = (typeof document.documentElement.onmouseleave !== 'undefined');

      if (!mouseEnterSupport)
        type = (type == "mouseenter") ? "mouseover" : "mouseout";

      var handlerFn = function(e)
      {
        var e = EventUtil.getEvent(e);
        var target = EventUtil.getTarget(e);
        
        // mouseover/mouseout - exit if we don't have the right element
        if (!mouseEnterSupport && EventUtil.isSubnode(EventUtil.getRelatedTarget(e), element))
          return;

        return (handler(e, target));
      };
    }
  
    return(EventUtil.addHandler(element, type, handlerFn || handler, capture));
  },   

  // keypress    
  getCharCode: function(e) 
  {
    //log(e.key);
    if (e.key)
      return e.key.charCodeAt(0);
        
    return ((typeof e.charCode == "number") ? e.charCode : e.keyCode)
  },
  
  // helpful function
  isSubnode: function(node, parent)
  {
    while (node.parentNode !== null && node !== parent)
      node = node.parentNode;
    
    return (node === parent);
  },
  
  // get wheel delta
  getWheelDelta: function(e)
  {
    if (e.wheelDelta)
      return ((JAK.Browser.client == "opera" && JAK.Browser.version < 9.5) ? -e.wheelDelta : e.wheelDelta);
    else
      return (-e.detail * 40);
  },
  
  // deselect all selected text
  deselect: function()
  {
    var selection = ('getSelection' in window) ? window.getSelection() : (('selection' in document) ? document.selection : null);
    
    if ('removeAllRanges' in selection)
    {
      selection.removeAllRanges();
    }
    else if ('empty' in selection)
    {
      // IE8 (document.selection.empty) vyhodí chybu, pokud není vybraný text visible
      try { selection.empty(); } catch(ex) {};
    }
  }    
};

// dynamically load .css file
function loadCSS(url, element)
{//log(url);
  var link = document.createElement("link");
  link.rel = "stylesheet";
  link.type = "text/css";
  link.href = url;
  var head = element || document.getElementsByTagName("head")[0];
  head.appendChild(link);
}

// dynamically load .js file
function loadJS(url, customFn)
{//log(url);
  var head = document.head || document.getElementsByTagName("head")[0] || document.documentElement;
  var script = document.createElement("script");
  script.async = "async";
  script.src = url;
  
  // from jQuery
  var done = false;

  script.onload = script.onreadystatechange = function()
  {
    if (!done && (!this.readyState || /loaded|complete/.test(script.readyState))) 
    {
      done = true;
      customFn && customFn();
      
      // je možno navěsit další funkce během spouštění
      if (Array.isArray(window["js_fns_" + url]))
      {
        window["js_fns_" + url].forEach(function(fn) { fn() });
      }

      // handle memory leak in IE
      script.onload = script.onreadystatechange = null;
      head && script.parentNode && head.removeChild(script);
    }
  };    

  head.appendChild(script);
}

// http://stackoverflow.com/questions/4059147/check-if-a-variable-is-a-string
function isString(o)
{
  return (Object.prototype.toString.call(o) === '[object String]');
}

// loadne řetězec JS souborů, kde každý další se načte až po tom předchozím
window._loaded = {};    // už loadnuté JS/CSS

function require(chains)
{
  var STARTED_LOADING = 1;
  var FINISHED_LOADING = 2;
  var loaded = window._loaded;

  // umožníme volat i s argumenty
  if (arguments.length != 1 || !Array.isArray(chains))
    chains = Array.from(arguments);

  // pokud je pole prázdné, došli jsme na konec řetězce a ukončíme
  if (chains.length == 0)
    return;

  // vezmeme první prvek z pole řetětězců
  var chain = chains.shift();

  // nemusíme jako parametr posílat vždy pole, pak ale prvek převedeme na pole o jednom prvku  
  if (!Array.isArray(chain))
    chain = [chain];
    
  // odstraníme z řetězce všechny JS/CSS, které už byly loadnuty
  chain = chain.filter(function(str)
  {
    if (!isString(str))                    // není řetězec - OK
      return true;
      
    if (loaded[str] == FINISHED_LOADING)   // je CSS/JS a bylo loadnuto - BAD
      return false;
    else                                   // je CSS/JS a nebylo ještě loadnuto - OK
      return true;
  });
  
  // počet .js souborů v řetězci
  var jsFiles = chain.filter(function(obj) 
  {
    return (isString(obj) && obj.indexOf(".js") != -1)
  }).length;

  var jsFilesLoaded = 0;
  
  // projde celý řetězec
  chain.forEach(function(obj)
  {
    // funkce spustí
    if (typeof obj === 'function')
    {
      obj();
      //log(obj);
    }
    
    // CSS soubory loadne  
    else if (isString(obj) && obj.indexOf(".css") != -1)
    {
      loaded[obj] = FINISHED_LOADING; // umístíme soubor nově do pole loadnutých
      loadCSS(obj);
      //log(obj);
    }
    
    // JS soubory loadne a pokud jsou všechny, tak pustí tuto funkci znovu se zbylými řetězci  
    else if (isString(obj) && obj.indexOf(".js") != -1)
    {
      var jsHandler = function()
      {
        //log(obj + ": FINISHED_LOADING");
        loaded[obj] = FINISHED_LOADING; // umístíme soubor nově do pole loadnutých
        jsFilesLoaded++;

        // pokud byly všechny JS soubory v řetězci loadnuty, tak spustíme další řetězec
        if (jsFiles == jsFilesLoaded)
        {
          require(chains);  
        }
      }
    
      // standardní stav - loadování JS ještě právě neběží
      if (loaded[obj] != STARTED_LOADING)
      {
        //log(obj + ": STARTED_LOADING"); 
        loaded[obj] = STARTED_LOADING;
        loadJS(obj, jsHandler);
      }
      
      // speciální případ - stejný JS je právě loadován
      else
      {
        var prop = "js_fns_" + obj;
        window[prop] = window[prop] || [];
        window[prop].push(jsHandler);
      }
    }
  });
  
  // pokud jsme neloadovali žádné JS, je třeba další level spustit zde
  if (jsFiles == 0)
    require(chains);
}


 // detect javascript errors
//   window.onerror=function(msg, url, linenumber)
//   {
//     alert('Error message: '+msg+'\nURL: '+url+'\nLine Number: '+linenumber)
//     return true
//   }
  
  // suppress javascript errors
//   window.onerror = function(){return true;};  

  

// *** cookie handling ***
function supportLocalStorage()
{
  if (!window.localStorage)
    return false;
  
  // FireFox 7.x a starší vrací po restartu a volání getItem vždy null (při použití protokolu file://)   
  if (JAK.Browser.client == "gecko" && JAK.Browser.version < 8 && window.location.protocol == "file:")
    return false;
    
  return true;
}


// https://developer.mozilla.org/en-US/docs/DOM/Storage
function write_cookie(cookiename, value)
{
  // localStorage (doesn't work locally in IE, necessary for Webkit)
  if (supportLocalStorage())   
  {
    // don't set if set already to the same value
    if (localStorage.getItem(cookiename) != value)
    {
      localStorage.removeItem(cookiename);
      localStorage.setItem(cookiename, value);
    }
  }
    
  // cookie
  else
  {
    document.cookie = escape(cookiename) + "=" + escape(value) + "; expires=Tue, 19 Jan 2038 03:14:07 GMT; path=/";
  }
}

function clear_cookie(cookiename)
{
  if (supportLocalStorage())
  {
    localStorage.removeItem(cookiename);
  }
  
  else
  {
    document.cookie = escape(cookiename) + "=; expires=Thu, 01 Jan 1970 00:00:00 GMT; path=/";
  }
};

function read_cookie(cookiename)
{
  // local storage   
  if (supportLocalStorage())
    return(localStorage.getItem(cookiename));

  // test if cookie exists
  if ((new RegExp("(?:^|;\\s*)" + escape(cookiename).replace(/[\-\.\+\*]/g, "\\$&") + "\\s*\\=")).test(document.cookie) == false)
    return null;
  
  // get cookie
  var cookie = unescape(document.cookie.replace(new RegExp("(?:^|.*;\\s*)" + escape(cookiename).replace(/[\-\.\+\*]/g, "\\$&") + "\\s*\\=\\s*((?:[^;](?!;))*[^;]?).*"), "$1"));

  return(cookie);
}

// logger
function log(string)
{
  if (typeof testing !== 'undefined' && testing === false)
    return;
    
  // cross-origin policy for local files:
  // IE: unrestricted (likely - as of IE10)
  // Firefox: same directory or subdirectory (https://developer.mozilla.org/en-US/docs/Same-origin_policy_for_file:_URIs)
  // Chrome: banned (run with flag "--allow-file-access-from-files")
  
  // write to logdiv (accessing top frame only available in IE)
  var logdiv = document.getElementById("idlogger");

  if (!logdiv && navigator.appName == 'Microsoft Internet Explorer')
  {
    try {logdiv = window.top.document.getElementById("idlogger")} catch(ex){};
  }
   
  if (logdiv)
  {
    if (typeof logdiv.insertAdjacentHTML !== 'undefined')
      logdiv.insertAdjacentHTML("afterbegin", string + "<br>")
    else
      logdiv.innerHTML = string + "<br>" + logdiv.innerHTML;
  }
  
  // write to console      
  if (window.console && window.console.log)
  {
    try {console.log(string)} catch(ex){};
  }
  
  if (window.opera && window.opera.postError)
  {
    opera.postError(string);
  }
}

// print all object properties/values
function log_object(obj)
{
  for (var prop in obj)
  {
    try
    {
      log(prop + ": " + (/*obj.hasOwnProperty(prop)*/1 ? obj[prop] : "N/A"));
    }
    catch(ex)
    {
      log("<span style='color: red'>" + prop + ": reading property failed!</span>");
    }      
  }
}

// timer
function timer(name)
{
  if (typeof timers === 'undefined')
    window.timers = {};
    
  name = name || "timer";

  // end timer  
  if (name in timers)
  {
    log(name + ": " + (new Date().getTime() - timers[name]) / 1000 + ' s');
    delete timers[name];
  }
  
  // start timer
  else
  {
    timers[name] = new Date().getTime();  
  }
}  

// *** add/remove/swap classname (used as flags for CSS) ***
function setClassList()
{
  class_list = (typeof document.body.classList !== 'undefined');
}

function get_elements (obj)
{
  // string --> querySelectorAll, object -> return as is
  return ((typeof obj === 'string') ? document.querySelectorAll(obj) : [obj]);
}

function contains_class (object, class_name)
{
  if (!class_name)
    return false;  

  if (typeof class_list === 'undefined') setClassList();             
  var elements = get_elements(object);
  var re = new RegExp('\\b' + class_name + '(?!\\w)', '');
  
  for (var i = 0; i < elements.length; i++)
    if (class_list ? (elements[i].classList.contains(class_name)) : (elements[i].className.search(re) != -1)) return true;

  return false;
}  

function add_class (object, class_name)
{
  if (!class_name)
    return;

  if (typeof class_list === 'undefined') setClassList();           
  var elements = get_elements(object);
    
  for (var i = 0; i < elements.length; i++)
    class_list ? (elements[i].classList.add(class_name)) : (elements[i].className += ' ' + class_name);
}

function remove_class (object, class_name)
{
  if (!class_name)
    return;  

  if (typeof class_list === 'undefined') setClassList();           
  var elements = get_elements(object);
  var re = new RegExp(' ?\\b' + class_name + '(?!\\w)', 'g');

  for (var i = 0; i < elements.length; i++)
    class_list ? (elements[i].classList.remove(class_name)) : (elements[i].className = elements[i].className.replace(re, ''));
}
  
function toggle_class (object, class_name)
{
  if (!class_name)
    return;  

  if (typeof class_list === 'undefined') setClassList();           
  var elements = get_elements(object);
  
  for (var i = 0; i < elements.length; i++)
  {
    if (class_list)
      elements[i].classList.toggle(class_name);
    else
    {  
      if (contains_class(elements[i], class_name))
        remove_class(elements[i], class_name);
      else
        add_class(elements[i], class_name);
    }
  }
}

// Array Remove - By John Resig (MIT Licensed)
Array.prototype.remove = function(from, to)
{
  var rest = this.slice((to || from) + 1 || this.length);
  this.length = from < 0 ? this.length + from : from;
  return this.push.apply(this, rest);
};


//alert(navigator.appVersion);
function fixEditBoxesInIE()
{
  // fix edit boxes (UTF-8 not working inside of GSAK)
  // do verze 10 fungovalo toto: navigator.appName == 'Microsoft Internet Explorer'
  if (window.isGSAK && JAK.Browser.embedded)
  {    
    window.setTimeout(function()
    {
      loadJS("JS/ie8/setSelectionRange.js");

      if (typeof fixGSAKKeyPress === 'undefined')
      {
        loadJS((window.basePath || "../Macros/HTML_Keskomapa/") + "JS/ie8/setSelectionRange.js",
          function()
          {
            fixGSAKKeyPress(document.documentElement, true);
            
            // fix edit boxes (IE7-8, where capture mode doesn't work)
            if (typeof document.addEventListener === 'undefined')
            {
              /* SMap.Control.Search */
              /* context menu search form */
              /* GSAK google maps find box */
              var matches = document.querySelectorAll("input");
              
              for (var i = 0; i < matches.length; i++)
                fixGSAKKeyPress(matches[i], false);

              //fixGSAKKeyPress(document.getElementById("findBox"), false);   /* context menu search form */
              //fixGSAKKeyPress(document.getElementById("address"), false);   /* GSAK google maps find box */
            }            
          });
      }
    }, 500);
  }
  
  // prevent possible double run
  //window.fixEditBoxesInIE = function(){};
}

// when run with GSAK original map, run it here, otherwise run it after opening the context menu
if (document.getElementById("address"))
  fixEditBoxesInIE();  

function cloneObject(obj)
{
  return JSON.parse(JSON.stringify(obj));
}

// load UI custom element
function ui(fn, options)
{
  require(
  [
    basePath + "ui.css",
    basePath + "ui.js",
    function() { window[fn](options) }
  ]);  
}

// file dialog
function initFileLoad(fn)
{
  require(
  [
    basePath + "formats.js",
    basePath + "filereader.js",
    fn
  ]);
}

function loadFile(elementName, clickedButton)
{
  initFileLoad(function()
  {
    document.getElementById(elementName || "fileInput").click();
  });
}

// isArray polyfill (pro prohlížeče starší než je Chrome 5, Firefox 4.0, IE 9, Opera 10.5 and Safari 5)
if (typeof Array.isArray === 'undefined')
{
  Array.isArray = function (vArg) 
  {
    return Object.prototype.toString.call(vArg) === "[object Array]";
  };
}  

// indexOf, insertAt, insertArrayAt
[].indexOf || (Array.prototype.indexOf = function(a, b, c)
{ 
  for(c = this.length, b = (c + ~~b) % c; b < c && (!(b in this) || this[b] !== a); b++);
  return b ^ c ? b : -1;
})

function insertArrayAt(array, index, arrayToInsert)
{
  Array.prototype.splice.apply(array, [index, 0].concat(arrayToInsert));
  return array;
}  

String.prototype.repeat = function(num)
{
  return new Array(num + 1).join(this);
}

Array.prototype.clone = function()
{
  return this.slice(0);
};

Array.prototype.compare = function (array) {
    // if the other array is a falsy value, return
    if (!array)
        return false;

    // compare lengths - can save a lot of time
    if (this.length != array.length)
        return false;

    for (var i = 0; i < this.length; i++) {
        // Check if we have nested arrays
        if (this[i] instanceof Array && array[i] instanceof Array) {
            // recurse into the nested arrays
            if (!this[i].compare(array[i]))
                return false;
        }
        else if (this[i] != array[i]) {
            // Warning - two different object instances will never be equal: {x:20} != {x:20}
            return false;
        }
    }
    return true;
}

// getComputedStyle - doesn't work in IE8 and older
if (!window.getComputedStyle)
{
    window.getComputedStyle = function(el, pseudo) {
        this.el = el;
        this.getPropertyValue = function(prop) {
            var re = /(\-([a-z]){1})/g;
            if (prop == 'float') prop = 'styleFloat';
            if (re.test(prop)) {
                prop = prop.replace(re, function () {
                    return arguments[2].toUpperCase();
                });
            }
            return el.currentStyle[prop] ? el.currentStyle[prop] : null;
        }
        return this;
    }
}      

function rotateElement(elm, deg)
{
  deg -= 90;

  var transform = cssProperty("transform");
  
  if (transform)
  {
    elm.style[transform] = "rotate(" + deg + "deg)";
    return;
  }
  
  /* IE7-8 */
  if (JAK.Browser.client == "ie" && JAK.Browser.version <= 8)
  {
    var rad = deg * Math.PI * 2 / 360;
    costheta = Math.cos(rad);
    sintheta = Math.sin(rad);
  
    var a = parseFloat(costheta).toFixed(8);
    var c = parseFloat(-sintheta).toFixed(8);
    var b = parseFloat(sintheta).toFixed(8);
    var d = parseFloat(costheta).toFixed(8);
  
    elm.style.filter = "progid:DXImageTransform.Microsoft.Matrix(M11=" + a + ", M12=" + c + ", M21=" + b + ", M22=" + d + ",sizingMethod='auto expand')";
  }
}

// makes element non-draggable
function disableDragging(elm)
{
  /* IE10, modern browsers */
  elm.draggable = false;
  
  /* IE7-9, older browsers */
  EventUtil.addHandler(elm, "dragstart", function(e, target)  
  {
    EventUtil.preventDefault(e);
  });
}

// Full Screen API

/*!
* screenfull
* v2.0.0 - 2014-12-22
* (c) Sindre Sorhus; MIT License
*/
!function(){"use strict";var a="undefined"!=typeof module&&module.exports,b="undefined"!=typeof Element&&"ALLOW_KEYBOARD_INPUT"in Element,c=function(){for(var a,b,c=[["requestFullscreen","exitFullscreen","fullscreenElement","fullscreenEnabled","fullscreenchange","fullscreenerror"],["webkitRequestFullscreen","webkitExitFullscreen","webkitFullscreenElement","webkitFullscreenEnabled","webkitfullscreenchange","webkitfullscreenerror"],["webkitRequestFullScreen","webkitCancelFullScreen","webkitCurrentFullScreenElement","webkitCancelFullScreen","webkitfullscreenchange","webkitfullscreenerror"],["mozRequestFullScreen","mozCancelFullScreen","mozFullScreenElement","mozFullScreenEnabled","mozfullscreenchange","mozfullscreenerror"],["msRequestFullscreen","msExitFullscreen","msFullscreenElement","msFullscreenEnabled","MSFullscreenChange","MSFullscreenError"]],d=0,e=c.length,f={};e>d;d++)if(a=c[d],a&&a[1]in document){for(d=0,b=a.length;b>d;d++)f[c[0][d]]=a[d];return f}return!1}(),d={request:function(a){var d=c.requestFullscreen;a=a||document.documentElement,/5\.1[\.\d]* Safari/.test(navigator.userAgent)?a[d]():a[d](b&&Element.ALLOW_KEYBOARD_INPUT)},exit:function(){document[c.exitFullscreen]()},toggle:function(a){this.isFullscreen?this.exit():this.request(a)},raw:c};return c?(Object.defineProperties(d,{isFullscreen:{get:function(){return!!document[c.fullscreenElement]}},element:{enumerable:!0,get:function(){return document[c.fullscreenElement]}},enabled:{enumerable:!0,get:function(){return!!document[c.fullscreenEnabled]}}}),void(a?module.exports=d:window.screenfull=d)):void(a?module.exports=!1:window.screenfull=!1)}();

// zjistí, zda je podporována konkrétní CSS vlastnost
function capitalizeFirstLetter(string)
{
  if (!string)      /* kontrola "" */
    return string;
    
  return string.charAt(0).toUpperCase() + string.slice(1);
}

function cssProperty(property)
{
  var prefixes = ["", "ms", "Moz", "Webkit", "O"];
  
  for (var i = 0; i < prefixes.length; i++)
  {
    var prop = prefixes[i] + (prefixes[i] ? capitalizeFirstLetter(property) : property);  // "" --> transform, "ms" --> msTransform

    if (prop in document.documentElement.style)
      return prop;  
  }
  
  return "";
}

// encapsulate original function
function encapsulate (object, variable, fn)
{
  object[variable + "Orig"] = object[variable];
  object[variable] = fn;
}

String.prototype.replaceAll = function (find, replace) {
    var str = this;
    return str.replace(new RegExp(find.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&'), 'g'), replace);
};

function setOpacity (elm, opacity)
{
  if ("opacity" in elm.style)
    elm.style.opacity = opacity;
  
  else if ("filter" in elm.style)   // IE7-8
    elm.style.filter = "alpha(opacity=" + (opacity * 100) + ")";  
}

/**
* @license MIT, GPL, do whatever you want
* @requires polyfill: Array.prototype.slice fix {@link https://gist.github.com/brettz9/6093105}
*/
// umí např převést arguments na pole: var arr = Array.from(arguments)
if (!Array.from) {
    Array.from = function (object) {
        'use strict';
        return [].slice.call(object);
    };
}

function isObject(obj)
{
  return (obj !== null && typeof obj === 'object')
}

function mergeOptions(obj1, obj2)                     /* přepíše vlastnosti jednoho objektu druhým */
{
  obj2 = obj2 || {};
  
  for (var i in obj2)
  {
    if (!(i in obj1))
      obj1[i] = obj2[i];                              /* pokud nemáme, tak přidáme */
    else if (isObject(obj1[i]) && isObject(obj2)[i])  /* pokud máme dva objekty, tak voláme rekurzivně */
      obj1[i] = replaceInObject(obj1[i], obj2[i]);
    else
      obj1[i] = obj2[i];                              /* jinak přepíšeme */
  }

  return obj1;
}

// from: http://stackoverflow.com/questions/4376431/javascript-heredoc
function hereDoc(func) 
{
 return func.toString().match(/\/\*\s*([\s\S]*?)\s*\*\//m)[1];
}
