SMap.Util.mergeParams = function(from, to)
{
  for (var p in from)
  {
    if (!(p in to))
      to[p] = from[p];
  }
}

//============================================================================
//                                  SMap
//============================================================================

                    
// ************
// !!! nemazat !!!
// function setCenterZoom has no business calling setProjection
// (fixes reseting projection to Mercator where zoom 21 was set)
// - our method may have issues, so replace the calling function instead
// encapsulate(SMap.prototype, "setProjection", function(projection)
// {
//   if(arguments.callee.caller != this.setCenterZoom)
//     this.setProjectionOrig(projection);
//   // zoom may not be updated properly after leaving zoom 21, so fix it
//   else
//     this.redraw();
// });

// zabráníme aby setCenterZoom rušilo při změně zoomu orientaci (opraveno v API v4.11.7)
// -- také zabráníme změně orientace při přechodu z 21 --> 20
// -- při odstranění této funkce je třeba odkomentovat kód výše
encapsulate(SMap.prototype, "setCenterZoom", function(c, z)
{
	if (!c.isValid()) { return; }
	c = this._adjustCenter(c, z);

	var zoom = this._newZoom(this._zoom, z);

	if (zoom == this._zoom)
  {
		this.setCenter(c);
	}
  else
  {
		this._zoom = zoom;
		this._center = c;
		this._redraw(true);
	}
});

// v ptačím pohledu při přechodu z 49.191053, 16.607823 do centra Plzně (49.747474873,13.3775874) dochází k chybě
encapsulate(SMap.prototype, "setCenter", function(center)
{
  try
  {
    return(this.setCenterOrig.apply(this, arguments));
  }
  catch(ex)
  {
    if (this.getZoom() == 21)
    {
      this.setZoom(20, null, false /* vypnutí animace */);
    }
    else   // pokud nejsme v ptačím pohledu, může se jednat o jinou chybu, na kterou upozorníme
    {
      throw true;
    }
  };
});

// setProjection without redraw
SMap.prototype.setProjection2 = function(projection, redraw)
{
 	if (this._projection) { this._projection.setOwner(null); }
 	this._projection = projection;
 	this._projection.setOwner(this);
  
//   if (redraw)
//  	  this._redraw(true); /* po zmene orientace je nutne plne prekresleni */

 	return this;
}

encapsulate(SMap.prototype, "_redraw", function(full)
{
  // Seznam code
	/* nastavit vektorovemu canvasu pozici a rozmery */
	var cont = this._geometryCanvas.getContainer();
	cont.style.left = -(this._offset.x+this._sizeHalf.x) + "px";
	cont.style.top = -(this._offset.y+this._sizeHalf.y) + "px"; 
	
  // my code
  
  // pošleme event ještě před překreslením vrstev
  this._signals.makeEvent("map-redraw-before", this, {full:full});

  // překreslíme nejdříve hlavní vrstvu mapy, pak až ty další překryvné  
  var mainLayer= this.getLayer(this.CurrentMapId);
  
  if (mainLayer)
  {
    mainLayer.redraw(full);
  }
  
	this._layers.forEach(function(layer)
  {
    if (layer === mainLayer)
      return;
      
    layer.redraw(full);
  });
  
  // fix: when in zoom >= 20 and not Mapy.cz map, markers would sometimes move to incorrect position after zooming
  // -- affect markers, but possibly also other layers, so we redraw also Geometry, Canvas, Image
  if (!full && this.getMap().getZoom() >= 20)
  {
     	this._layers.forEach(function(layer)
      {  
        if (layer instanceof SMap.Layer.Marker 
        || layer instanceof SMap.Layer.Geometry 
        || layer instanceof SMap.Layer.Canvas
        || layer instanceof SMap.Layer.Image)
          layer.redraw(true);
      });  
  }  
  
  // Seznam code
	if (this._card) { this._card.anchorTo(this._card.getAnchor()); } /* pokud je vizitka, tak jeji ukotveni */
	this._signals.makeEvent("map-redraw", this, {full:full});
})

// getProjection -> přidání možnosti získat projekci zadané vrstvy
encapsulate(SMap.prototype, "getProjection", function(layerId)
{
  if (typeof layerId === 'undefined' || !layerId)
    return this.getProjectionOrig();
    
  else
  {
    var proj = SMap.CONFIG[this.desmartizeLayerId(layerId)].projection || SMap.projMercator;
    
    // pokud se ptáme na oblique vrstvu, tak vezmeme aktuální projekci
    // -- není to ideální, neměli bychom se ptát když nebude zapnutá! 
    if (layerId == SMap.DEF_OBLIQUE || layerId == SMap.DEF_OBLIQUE_HYBRID)
    {
      proj = this.getProjectionOrig();
      
      if (proj.getCode() != "oblique")
        throw "SMap.getProjection - asked for oblique layer projection when oblique layer not set!";
    }    
    
    return proj;
  }
});

// getZoomRange -> přidání možnosti získat rozsah zoomu zadané vrstvy
encapsulate(SMap.prototype, "getZoomRange", function(layerId)
{
  if (typeof layerId === 'undefined' || !layerId)
  {
    return this.getZoomRangeOrig();
  }
    
  else
  { 
    return SMap.CONFIG[this.desmartizeLayerId(layerId)].zoom || /* Mapy.cz transparent layers */ [5, 20];
  }
});

// odstranění minzoomu 5 (FIXME)
SMap.prototype.setZoomRange = function(min, max) {
	this._options.minZoom = min/*Math.max(min, 5)*/; /* FIXME zrusit po zverejneni mercatoru */
	this._options.maxZoom = max;
	var z = Math.max(min, this._zoom);
	z = Math.min(max, z);
	if (z != this._zoom) { this.setZoom(z); }
	this._signals.makeEvent("zoom-range-change", this);
}

/**
 * Spočítá střed a zoom pro množinu bodů
 * @author N (komplet přepsaná původní funkce API)
 * @param {SMap.Coords[]} points Pole souřadnic
 * @returns {array} Střed a zoom
 */
SMap.prototype.computeCenterZoom = function(points)
{
  // allow either [] or a list of arguments
  if (!Array.isArray(points))
    points = Array.prototype.slice.call(arguments); /* converts arguments object to an array */

  // krajní body a střed nejmenšího obdélníku obsahujícího všechny body
  var bBox = SMap.Coords.boundingBox(points);

  // načtení rozsahu zoomu
  var zoomRange = this.getMap().getZoomRange();
  var minZoom = zoomRange[0];
  var maxZoom = zoomRange[zoomRange.length - 1];

  for (var z = maxZoom; z >= minZoom; z--)
  {
    var proj = this.getMap().getProjection();

    // pokud máme oblique projekci, tak projekci překlopíme na Mercator    
    if (proj.getCode() == "oblique")
      proj = SMap.projMercator;

    // zkontrolujeme, zda všechny body hranice leží ve viewportu pro daný zoom
    // -- pokud ano, tak ukončíme; pokud ne, tak pokračujeme s dalším nižším zoomem        
    var areCornersInMap = [bBox.topLeft, bBox.topRight, bBox.bottomLeft, bBox.bottomRight].every(function(p)
    {
      return p.inMap(this, proj, bBox.center, z);
    }, this);

    // pokud jsme našli největší možný zoom, kam všechny hraniční body pasují, tak ukončíme
    if (areCornersInMap)
      return [bBox.center, z];
  }

  // pokud jsme nenašli vhodný zoom, tak vrátíme ten nejmenší
  return [bBox.center, minZoom];  
}

// ******* smooth out zoom level change (gets nearest zoom of another layer)
SMap.prototype.getNearestZoom = function(options)
{
  options = options || {};

  var optionsDefault =
  {
    layerFrom: null /* aktuální vrstva */,
    zoomFrom: this.getZoom(),
    orientationFrom: this.getOrientation(),
    layerTo: null, /* aktuální vrstva */
    lower: false,  /* nastaví cílový zoom o 1 nižší */
    higher: false  /* nastaví cílový zoom o 1 vyšší */
  };

  SMap.Util.mergeParams(optionsDefault, options);

  // získame rozsah zoomů cílové vrstvy
  var zoomRange = this.getZoomRange(options.layerTo);
  var minZoom = zoomRange[0];
  var maxZoom = zoomRange[zoomRange.length - 1];

  // najdeme nejbližší zoom v cílové vrstvě
  var scaleFrom = this.getScale({layer: options.layerFrom, zoom: options.zoomFrom, orientation: options.orientationFrom});
  var zoomBest;
  var distBest = Infinity;
  
  for (var z = minZoom; z <= maxZoom; z++)
  {
     var scaleTo = this.getScale({layer: options.layerTo, zoom: z, orientation: options.orientationFrom});
     
     // vezmeme poměr meřítek a pak logaritmus, protože log(a/b) je roven -log(b/a)
     var dist = Math.log(scaleFrom / scaleTo);
          
     if (Math.abs(dist) < Math.abs(distBest))
     {
      distBest = dist; /* zachováme znaménko, kvůli options.lower a options.higher */
      zoomBest = z;
     }
  }

  // u některých vrstev je třeba manuálně nastavit o úroveň vyšší či nižší zoom
  if (options.lower && zoomBest < maxZoom && distBest > 0.05)
    zoomBest--;

  if (options.higher && zoomBest > minZoom && distBest < -0.05)
    zoomBest++;

  return zoomBest;
}

// get scale (options can have params: layer, zoom, round)
SMap.prototype.getScale = function(options)
{
  options = options || {};
  
  var optionsDefault =
  {
    layer: null /* současná vrstva */,
    zoom: this.getZoom(),
    orientation: this.getOrientation(),
    round: false /* zaokrouhlit měřítko? */ 
  }
  
  SMap.Util.mergeParams(optionsDefault, options);

  // pokud byla zadána vrstva, tak si vezmeme její projekci (nebo UTM33 pokud žádnou u sebe nemá),
  // -- pokud nebyla zadána, tak vezmeme projekci aktuální mapy
  var proj = this.getProjection(options.layer);
  
  // spočteme vzdálenost pixelů (0,0) a (100,100)
  var px1 = new SMap.Pixel(0, 0);
  var px2 = new SMap.Pixel(100, 100);

  var p1 = proj.pixelToCoords(px1, this.getCenter(), options.zoom, options.orientation);
  var p2 = proj.pixelToCoords(px2, this.getCenter(), options.zoom, options.orientation);
  var scale = p1.distance(p2) / px2.norm(); /* vydělíme, jelikož chceme délku strany a ne úhlopříčky, navíc znormovanou */
  scale *= 95/160; /* poměr, abychom to dostali přibližně na seznamácké hodnoty */

  // zaokrouhlování
  if (options.round)
  {
    // měřítka v řádu tisíců zaokrouhlí na tisíce (např. 75248 --> 75000)
    if (scale > 0.9)
      scale = Math.round(scale * 10) * 1000;
    // v řádu set na stovky
    else if (scale > 0.09)
      scale = Math.round(scale * 100) * 100;
    // jinak zaokrouhlíme na jednotky
    else
      scale = Math.round(scale * 10000);  
  }
  else
    scale *= 10000;

  return scale;
}

// switches map (and updates layer switch)
SMap.prototype.switchMap = function(id)
{
  JAK.DOM.removeClass(document.querySelector(".layer-switch .active"), "active");
  this.getLayer(this.CurrentMapId).disable();
  var oldId = this.CurrentMapId;
  this.CurrentMapId = id;
  this.getLayer(this.CurrentMapId).enable();
  this.getSignals().makeEvent("map-switch", this, {current: id, old: oldId});
}

// ************
// addDefaultLayer - add custom build url
encapsulate(SMap.prototype, "addDefaultLayer", function(id)
{
  var conf = SMap.CONFIG[id];
  
  if (!conf)
    throw("addDefaultLayer: Layer with ID '" + id + "' doesn't exist!"); 

  // vrstva je WMTS
  if (conf.WMTS)
  {
    conf.options.tileSize = 256; // FIXME
    var layer = new SMap.Layer.WMTS(id, conf.url, conf.WMTS, conf.options);
    layer.setCopyright(conf.copyright);
    this.addLayer(layer);
  }
  
  // vrstva je WMS
  else if (conf.WMS)
  {
    var layer = new SMap.Layer.WMS(id, conf.url, conf.WMS, conf.options);
    layer.setCopyright(conf.copyright);
    this.addLayer(layer);
  }
  
  // vrstva je ArcGIS
  else if (conf.options && conf.options.ArcGIS)
  {
    var layer = new SMap.Layer.ArcGIS(id, conf.url, conf.options);
    layer.setCopyright(conf.copyright);
    this.addLayer(layer);
  }  

  // ostatní vrstvy
  else
  {
    var layer = this.addDefaultLayerOrig(id);
  }

  layer._buildURLOrig = layer._buildURL;
  layer._buildURL = conf.buildUrl || layer._buildURL;
  layer.redraw = conf.redraw || layer.redraw;  
  return layer;
});

/*** fix listener errors, when we double click on a marker ***/
encapsulate(SMap.prototype, "animate", function(c, options)
{
  if (JAK.Timekeeper.getInstance()._findListener(this) == -1)
    return (this.animateOrig(c, options));
});

encapsulate(SMap.prototype, "setZoom", function(z, fixedPoint, animate)
{
  if (JAK.Timekeeper.getInstance()._findListener(this) == -1)  
    return (this.setZoomOrig(z, fixedPoint, animate));
});

// převede id chytré vrstvy na id podřízené hlavní vrstvy, se kterou už můžeme pracovat
SMap.prototype.desmartizeLayerId = function(idIn)
{
  var idOut = idIn;
  if (idIn == SMap.DEF_SMART_BASE)   idOut = SMap.DEF_BASE;
  if (idIn == SMap.DEF_SMART_OPHOTO) idOut = SMap.DEF_OPHOTO;
  if (idIn == SMap.DEF_SMART_TURIST) idOut = SMap.DEF_TURIST;
  if (idIn == SMap.DEF_SMART_WINTER) idOut = SMap.DEF_TURIST_WINTER;
  if (idIn == SMap.DEF_SMART_SUMMER) idOut = SMap.DEF_SUMMER;

  return idOut;
}

SMap.prototype.isLayerSmart = function(id)
{
  id = id || this.CurrentMapId;
  return (this.desmartizeLayerId(id) != id)
}

/****** global disabling/enabling of marker dragging (fixes some bugs in free draw polygon mode) ******/
SMap.prototype.disableMarkerDragging = function()
{
  this.markerDraggingDisabled = true;
  
  this.markerDraggingFnBackup = SMap.Marker.Feature.Draggable.prototype._dragStart;
  SMap.Marker.Feature.Draggable.prototype._dragStart = function(){};
}

SMap.prototype.enableMarkerDragging = function()
{
  if (!this.markerDraggingFnBackup)
    return;

  this.markerDraggingDisabled = false;
  SMap.Marker.Feature.Draggable.prototype._dragStart = this.markerDraggingFnBackup;
}

//============================================================================
//                            SMap.Control.Mouse
//============================================================================

// stop moving map using mouse if alt or shift is pressed (to enable setting selection on any mouse/key combination)
encapsulate(SMap.Control.Mouse.prototype, "_mouseDown", function(e, elm)
{
  if (e.shiftKey || e.altKey)
    return;
    
  return(this._mouseDownOrig(e, elm));
});

//----------------------------------------------------------------------------------------
// allow zooming 3 levels on Shift/Alt + middle scroll

encapsulate(SMap.Control.Mouse.prototype, "_scroll", function(e, elm)
{
  // zabráníme tomu, aby ve Firefoxu přestalo fungovat CTRL+kolečko zoomu (jehož funkcí je změna velikosti stránky)
  if (e.ctrlKey)
    return;

  // Seznam code
	if (e.wheelDeltaX) { return; } /* posun po vodorovne ose */
	JAK.Events.cancelDef(e);
	if (!this._checkScrollFrequency()) { return; }

	var delta = e.wheelDelta || -e.detail;
	var x = e.clientX;
	var y = e.clientY;
  
  // my code
  // -- pokud držíme shift nebo alt, tak změníme zoom hned o 3 stupně
  var zoomChange = (delta > 0) ? 1 : -1;

  if (e.altKey || e.shiftKey /*|| e.button == JAK.Browser.mouse.right*/)
    zoomChange *= 3;  
  
  this._zoom(zoomChange.toString(), x, y);
});

// pokud je aktivní tažení mapy myší, pak je třeba odvěsit posluchač, jinak dojde k chybě v SMap.Control.Mouse.prototype._driftStep
// -- problém je i v SMap.prototype.$destructor, kde je control odebrána dříve než je volán destruktor (proto kontrolujeme existenci mapy)
// -- tento destructor je třeba volat před odebráním pomocí removeControl, aby this.getMap() nevracelo null, jinak this._stopDrift() skončí chybou
encapsulate(SMap.Control.Mouse.prototype, "$destructor", function()
{
  if (this.getMap() && this._drift.speed)
  {
    // kód níže je e this._stopDrift(), ale byla přidána kontrola existence this.getMap()
  	JAK.Timekeeper.getInstance().removeListener(this);
  	this._drift.speed = 0;
    
    if (this.getMap())
  	 this._resyncMap();  
  }

  this.$destructorOrig();
});

//============================================================================
//                            SMap.Control.Scale
//============================================================================

// change this function to add "layer-change-before" event
SMap.Control.Scale.prototype.setOwner = function(owner) {
	this.$super(owner);
	if (!owner) { return; }

	this._sc.push(this.getMap().getSignals().addListener(this, "map-redraw", "_redraw"));
  
  // ** my code
  this._sc.push(this.getMap().getSignals().addListener(this, "layer-change-before", "_redraw"));
  // **
  
	this._redraw();
}
 
// set distances and widths in scale
encapsulate(SMap.Control.Scale.prototype, "_redraw", function()
{
  var zoom = this.getMap().getZoom();
  
  // compute length of each rectangle
  var positionX = - this.getMap().getSize().x / 2 + this.getContainer().offsetLeft;
  var positionY = - this.getMap().getSize().y / 2 + (this.getContainer().offsetTop + this.getContainer().offsetHeight - 3 /* scale rectangle height */);
  
  var px1 = new SMap.Pixel(positionX, positionY);
  var px2 = px1.clone().plus(new SMap.Pixel(50, 0));
  
  var p1 = this.getMap().getProjection().pixelToCoords(px1, this.getMap().getCenter(), zoom, this.getMap().getOrientation());
  var p2 = this.getMap().getProjection().pixelToCoords(px2, this.getMap().getCenter(), zoom, this.getMap().getOrientation());

  var distance = Math.round(p1.distance(p2));

  // set distance and units
  var m_or_km = (distance < 1500) ? 1 : 1000;

  if (m_or_km == 1000)  /* granularize kilometers */
  {
    if (distance < 10000)
      distance = Math.round(distance / 10) * 10;
    else if (distance < 100000)
      distance = Math.round(distance / 100) * 100;
    else
      distance = Math.round(distance / 1000) * 1000;
  }

  this._steps[zoom] = [distance, m_or_km];

  // *** original stuff
	this._zoom = zoom;

	var def = this._steps[zoom];
	this._dom.container.style.display = "";
	var width = Math.round(def[0] * Math.pow(2, zoom - 15));
  
  var totalWidth = 0;
  var totalLength = 0;

  // set all widths to correct numbers
	this._dom.numbers.forEach(function(number, index)
  {
    number.style.left = (1 + totalWidth) + "px";
    number.innerHTML = totalLength / def[1];

		if (index < this._dom.parts.length) /* dílek */
			this._dom.parts[index].style.width = "50px";
    else   /* jednotky */
 			this._dom.unit.style.left = totalWidth + "px";
      
    totalWidth += 50;
    totalLength += def[0];

	}, this);
  
  this._dom.unit.innerHTML = this._units[def[1]]; 
});

//============================================================================
//                         SMap.Control.Copyright
//============================================================================

// přepíše funkci na výpočet měřítka
SMap.Control.Copyright.prototype._computeScale = function()
{
  return ("1 : " + this.getMap().getScale({round: true}));
}

// přidává zoom, měřítko a orientaci do textu dole
encapsulate(SMap.Control.Copyright.prototype, "_redraw", function()
{
  this._redrawOrig();
  var text = this.getMap().getZoom();
  
  if (this.getMap().getOrientation() != SMap.NORTH)
    text += {0: "", 1: " (WEST)", 2: " (SOUTH)", 3: " (EAST)"}[this.getMap().getOrientation()];

  text += " / " + this._computeScale();
  
  this._dom.container.innerHTML = text + " " + this._dom.container.innerHTML;
});

//============================================================================
//                          SMap.Layer.Geometry
//============================================================================

// support contexmenu event for "geometry-click"
// -- z eventu je pak možno zjistit, že to bylo contextmenu a ne click
encapsulate(SMap.Layer.Geometry.prototype, "_build", function()
{
  var ret = this._buildOrig.apply(this, arguments);
  var group = this._dom.container[SMap.LAYER_GEOMETRY];
  this._ec.push(JAK.Events.addListener(group, "contextmenu", this, "_click"));
  return ret;  
});

//============================================================================
//                            SMap.Layer.Marker
//============================================================================


SMap.Marker.prototype.click = function(e, elm) {
  //log(e.type);
	this.getMap().getSignals().makeEvent("marker-click", this, {event:e});
}

/****** add marker dblclick ******/
encapsulate(SMap.Layer.Marker.prototype, "_build", function()
{
  this._buildOrig();
  this._ec.push(JAK.Events.addListener(this._dom.container[SMap.LAYER_ACTIVE], "dblclick", this, "_dblclick"));
  this._ec.push(JAK.Events.addListener(this._dom.container[SMap.LAYER_MARKER], "dblclick", this, "_dblclick"));
});

SMap.Layer.Marker.prototype._dblclick = function(e, elm) 
{//log_object(e)
 	var t = JAK.Events.getTarget(e);
 	var tops = [];
 	for (var p in this._dom.container) { tops.push(this._dom.container[p]); }
 	
 	for (var id in this._markers) {
 		var m = this._markers[id].marker;
 		var node = t;
 		var a = m.getActive();
 		do {
 			if (node == a) 
       {
 				m.dblclick(e, elm, "x"); // my only change
 				return;
 			}
 			node = node.parentNode;
 		} while (tops.indexOf(node) == -1);
 	}
}

// viz ještě v SMap funkce disableMarkerDragging a  enableMarkerDragging

//============================================================================
//                              SMap.Marker
//============================================================================

SMap.Marker.prototype.dblclick = function(e, elm)
{log(e.type)
  this.getMap().getSignals().makeEvent("marker-dblclick", this, {event:e});
};

/****** set dragging switch ******/

// when dragging with CTRL or SHIFT, old IE would select all text on the page --> prevent that
SMap.Marker.prototype._ie8DragFix = function()
{
  // when dragging with CTRL or SHIFT, old IE would select all text on the page --> prevent that
  if (JAK.Browser.client == "ie" && JAK.Browser.version <= 9)
    EventUtil.deselect();
}

SMap.Marker.prototype.verifyEventTarget = function(e, elm)
{
  var target = EventUtil.getTarget(e);
  
  if (e.type == "marker-drag-start" || e.type == "marker-drag-move" || e.type == "marker-drag-stop")
    return (EventUtil.getTarget(e) === this);
  
  // click, mouseover, ...
  else
  {
    if (target === this.getContainer()[SMap.LAYER_MARKER])
      return true;
      
    if (EventUtil.isSubnode(target, this.getContainer()[SMap.LAYER_MARKER]))
      return true;  
  }
  
  return false;
}

// set flag when drag starts (sumple click also turns this on temporarily!!!)
SMap.Marker.prototype._toggleDragOn = function(e, elm)
{
  if (!this.verifyEventTarget(e, elm))
    return;

  add_class(this.getContainer()[SMap.LAYER_MARKER], "isBeingDragged"); 
  this._isBeingDragged = true;
  this._ie8DragFix();
}

// set flag when drag ends
SMap.Marker.prototype._toggleDragOff = function(e, elm)
{
  if (!this.verifyEventTarget(e, elm))
    return;

  remove_class(this.getContainer()[SMap.LAYER_MARKER], "isBeingDragged"); 
  this._isBeingDragged = false;
  this._ie8DragFix();
}

// note: in webkit, marker title can't coexist with draggable markers (displaying card then often fails)
// (fixed by displaying div on hover instead)

//============================================================================
//                              SMap.Bubble
//============================================================================

// -- funguje jen tehdy, pokud má marker container div (pokud je container img, tak ne)
SMap.Bubble = JAK.ClassMaker.makeClass
({
  NAME: "SMap.Bubble",
  VERSION: "2.0",   /* 6.3.2015 - rewritten, hiding/showing handled by CSS */
  IMPLEMENT: [SMap.IOwned],
});
 
SMap.Bubble.prototype.$constructor = function(options, css)
{
  this.id = JAK.idGenerator();
  this.cssName = "markerBubble";
  
  // set options
  this.options = 
  {
    html: "",
    className: "",
    alwaysShow: false  
  }
  
  for (var param in (options || {}))
    this.options[param] = options[param];
  
  // create bubble div
  this.dom = JAK.mel("div", {className: this.options.className + " " + this.cssName}, css || {});

  this.setHTML(this.options.html);
  
  if (this.options.alwaysShow)
    this.show();

	this._ec = [];   /* listeners created by JAK.Events.addListener */
	this._sc = [];   /* listeners created by this.getMap().getSignals().addListener */
}

SMap.Bubble.prototype.$destructor = function()
{
	JAK.Events.removeListeners(this._ec);

	if (this.getMap())
    this.getMap().getSignals().removeListeners(this._sc);  
}

/* set bubble HTML */
SMap.Bubble.prototype.setHTML = function(html)
{
  this.options.html = html || "";
  this.dom.innerHTML = this.options.html;
}

/* force show */
SMap.Bubble.prototype.show = function()
{
  this.dom.style.display = "block";
}

/* force hide */
SMap.Bubble.prototype.hide = function()
{
  this.dom.style.display = "none";
}

/* vrátí div prvek s bublinou */
SMap.Bubble.prototype.getDom = function()
{
  return this.dom;
}

/* nastaví třídu bubliny (parametr je buď pole tříd nebo řetězec s třídami) */
SMap.Bubble.prototype.setClasses = function(classes)
{
  if (Array.isArray(classes))
    classes = classes.join(" ");
  
  this.options.className = classes;  
  this.getDom().className = this.cssName + " " + this.options.className;
}

SMap.Marker.Feature.Bubble = JAK.ClassMaker.makeSingleton(
{
  NAME: "SMap.Marker.Feature.Bubble",
  VERSION: "1.0",
  EXTEND: JAK.AbstractDecorator
});
 
SMap.Marker.Feature.Bubble.prototype.decorate = function(marker, bubble)
{
  this.$super(marker);
                 
  marker.bubble = marker.bubble || [];
  marker.bubble.push(bubble);

  // add bubble div to marker
  var markerDiv = marker.getDom();
  markerDiv.appendChild(bubble.dom);
  add_class(markerDiv, "markerWithBubble"); 

  // install drag handlers
  // -- can't do now, because we need this.getMap()
  // -- also handle click on marker which prevents drag-stop from firing
  marker._ec[bubble.id] = JAK.Events.addListener(markerDiv, "mouseover", marker, function()
  {
    JAK.Events.removeListener(this._ec[bubble.id]);
    bubble._sc.push(this.getMap().getSignals().addListener(this, "marker-drag-start marker-drag-move", "_toggleDragOn"));
    bubble._sc.push(this.getMap().getSignals().addListener(this, "marker-drag-stop", "_toggleDragOff"));
    this._ec[bubble.id] = JAK.Events.addListener(markerDiv, "click", this, "_toggleDragOff");
  });

  return marker;
}


SMap.Marker.prototype.getBubble = function()
{
  if (!this.bubble)              // bubble není --> null
    return null;
    
  if (this.bubble.length == 1)
    return this.bubble[0];       // bubble je jedna --> vrátíme ji
    
  else
    return this.bubble;          // bubble je víc --> vrátíme pole se všemi bubble
}

SMap.Marker.prototype.removeBubble = function(bubble)
{
  // odstranění z pole bublin
  this.bubble.splice(this.bubble.indexOf(bubble), 1);
  
  // odstranění handleru
  JAK.Events.removeListener(this._ec[bubble.id]);
  delete this._ec[bubble.id];
  
  // odstranění domu
  bubble.dom.parentNode.removeChild(bubble.dom);
}



//============================================================================
//                            SMap.Control.Layer
//============================================================================

// ************
// get current layer + automatically switch projection + let us know before turning on the layer
encapsulate(SMap.Control.Layer.prototype, "_itemClick", function(e, elm)
{
  for (var id in this._layers) 
  {
  	if (this._layers[id] == elm)
    {
      var map = this.getMap();

      // set nearest zoom level
      map._zoom = map.getNearestZoom({layerTo: id});

      if (id != map.CurrentMapId)
        this.getMap().getSignals().makeEvent("layer-switch-click-before");

      // set new ID
      map.CurrentMapId = id;
      
      // set new projection (needed for correct copyright get scale - otherwise would use old projection)
      var conf = SMap.CONFIG[id];
      map.setProjection2(/* projection in CONFIG settings */ conf.projection || /* default projection */ SMap.projMercator);
    }
  }

  this._itemClickOrig(e, elm);
  
  this.getMap().getSignals().makeEvent("layer-switch-click-after");
});

// fix Control.Layer switch (remove javascript, use CSS instead)
SMap.Control.Layer.prototype._menuClose = function(){};
SMap.Control.Layer.prototype._closeCancel = function(){};
SMap.Control.Layer.prototype._mouseOut = function(){};

// tyhle dvě funkce musíme částečně zachovat, aby se přepínač při startu přepnul na správnou stránku 
SMap.Control.Layer.prototype._mouseOver = function(e, elm)
{
	this._open();
};

SMap.Control.Layer.prototype._open = function()
{
	if (this._opened) { return; }
	this._opened = true;
	
	this._switchActivePage();
};

// manuální přidání více vrstev
// -- šetří asi 30 ms při 30 vrstvách
// -- také opravuje bug v Opeře, kdy při startu mapy a myší nad tlačítkem se přepínač přesunul na špatnou stránku
// -- ještě opravuje aby preview linky fungovaly i když je api stahováno z netu 
SMap.Control.Layer.prototype.addMultipleDefaultLayers = function(arr)
{
  var that = this;
  
  arr.forEach(function(id)
  {
    var def = SMap.CONFIG[id];
    var preview = (def.preview.indexOf("/") == -1) ? (SMap.CONFIG.img + "/" + def.preview) : def.preview; 
    that._layers[id] = that._buildItem(def.label, preview, def.title);
  });
  
  this._setPage(0);    /* tohle je zde třeba, jinak se na začátku objeví na obou stranách šipky */
}   

// předělání šipek, aby nejen posunovaly vpřed a vzad, ale aby rotovaly (tedy poslední -> první atd.)
// encapsulate(SMap.Control.Layer.prototype, "_setPage", function()
// {
//   var ret = this._setPageOrig.apply(this, arguments);
//   
//   if (this._maxPage() != 0)
//   {
// 	 this._dom.prev.style.display = "";
// 	 this._dom.next.style.display = "";
//   }
// });
// 
// encapsulate(SMap.Control.Layer.prototype, "_prev", function()
// {
//   this._setPage((this._page == 0) ? this._maxPage() : (this._page - 1));
// });
// 
// encapsulate(SMap.Control.Layer.prototype, "_next", function()
// {
//   this._setPage((this._page == this._maxPage()) ? 0 : (this._page + 1));
// });

// na poslední stránce posuneme přepínač, aby v něm byl plný počet prvků
encapsulate(SMap.Control.Layer.prototype, "_pageToPixel", function(page)
{
  var ret = this._pageToPixelOrig.apply(this, arguments);
  
  if (page > 0 && page == this._maxPage())
  {
    var itemsTotal = 0;
    
    for (var layer in this._layers)
      itemsTotal++;
    
    var itemsPerPage = this._options.items;
    var itemsLastPage = itemsTotal % itemsPerPage
    
    if (!itemsLastPage)
      itemsLastPage = itemsPerPage;
      
    ret += (itemsPerPage - itemsLastPage) * this._options.width;
  }

  return ret;  
});

//============================================================================
//                             SMap.Layer
//============================================================================

encapsulate(SMap.Layer.prototype, "enable", function()
{
  if (SMap.CONFIG[this.getId()] && SMap.CONFIG[this.getId()].preview)
  {
    // no ID set yet? then set it (will be updated in _itemClick)
    this.getMap().CurrentMapId = this.getMap().CurrentMapId || this.getId();

    // set zoom range for the new map
    var zoomRange = SMap.CONFIG[this.getId()].zoom;
    
    if (zoomRange && zoomRange.length)  
      this.getMap().setZoomRange.apply(this.getMap(), zoomRange);
   
    // change layer
    if (this.getId() == this.getMap().CurrentMapId)
    {
      // set new projection again (in case this is our first run)
      var conf = SMap.CONFIG[this.getId()];
      this.getMap().setProjection2(/* projection in CONFIG settings */ conf.projection || /* default projection */ SMap.projMercator);

      this.getMap().getSignals().makeEvent("layer-change-before", this.getId());
      
      // redraw all marker and geometry layers (so that they are not off when projection changes)
      // - taken from SMap.prototype._redraw
      var map = this.getMap();
     	var cont = map._geometryCanvas.getContainer();
     	cont.style.left = -(map._offset.x+map._sizeHalf.x) + "px";
     	cont.style.top = -(map._offset.y+map._sizeHalf.y) + "px"; 

     	map._layers.forEach(function(layer)
      {
        if (!(layer.getId() in SMap.CONFIG)) /* skip map layers */
          layer.redraw(true);
      });
      
     	if (map._card) { map._card.anchorTo(map._card.getAnchor()); } /* pokud je vizitka, tak jeji ukotveni */
    }
  }
  
  this.enableOrig(this);
});

//============================================================================
//                       SMap.Layer.Tile / SMap.TileSet
//============================================================================

function matMultiply(m1, m2)
{
  // pokud je argumentů více než 2, tak máme víc matic a všechny je postupně odleva vynásobíme
  if (arguments.length > 2)
  {
    var result = null;
    
    for (var i = 0; i < arguments.length - 1; i++)
      result = matMultiply(result || arguments[i], arguments[i+1]);

    return result;
  }
  // pokud máme jen jednu matici, tak ji vrátíme
  else if (arguments.length == 1)
  {
    return m1;
  }

  var m1_height = m1.length;
  var m1_width = m1[0].length;
  var m2_height = m2.length;
  var m2_width = m2[0].length;
  
  // kontrola rozměrů vstupních matic
  if (m1_width != m2_height)
    throw "Matrix width/height incorrect!";

  // vytvoření nové prázdné matice
  var ans = new Array(m1_height);

  for (var i = 0; i < ans.length; i++)
    ans[i] = new Array(m2_width);  

  // sloupce výsledné matice
  for (var h = 0; h < m1_height; h++)
  {
    // řádky výsledné matice
    for (var w = 0; w < m2_width; w++)
    {
      // skalární součin sloupce a řádku
      var sum = 0;
      
      for (var i = 0; i < m1_width; i++)
        sum += m1[h][i] * m2[i][w];
        
      ans[h][w] = sum; 
    }
  }

  return ans;
}

// function testLinearTransformation
// {
//   var deg = 180 / Math.PI;
// 
//   var blp = new SMap.Pixel(70, 50);
//   var brp = new SMap.Pixel(100, 50);
//   var tlp = new SMap.Pixel(70, 100);
//   var trp = new SMap.Pixel(100, 100);
//   
//   var skewX = 50;
//   var skewY = 0;
//   var translateX = 200;
//   var translateY = 1000;
//   var scaleX = 2.75;
//   var scaleY = 7.75;
//   var rotAngle = 50;
//     
//   var skewMatrix = [[1,Math.tan(skewX / deg),0], [Math.tan(skewY / deg),1,0], [0,0,1]];
//   var translationMatrix = [[1,0,translateX], [0,1,translateY], [0,0,1]];
//   var scaleMatrix = [[scaleX,0,0], [0,scaleY,0], [0,0,1]];
//   var rotationMatrix = [[Math.cos(rotAngle/deg),-Math.sin(rotAngle/deg),0], [Math.sin(rotAngle/deg),Math.cos(rotAngle/deg),0], [0,0,1]];
//   
//   var matrix = matMultiply(skewMatrix, rotationMatrix, scaleMatrix, translationMatrix);
// 
//   var blp2 = vec2px(matMultiply(matrix, px2vec(blp)));
//   var brp2 = vec2px(matMultiply(matrix, px2vec(brp)));
//   var tlp2 = vec2px(matMultiply(matrix, px2vec(tlp)));
//   var trp2 = vec2px(matMultiply(matrix, px2vec(trp)));
// 
//   getLinearTransform(blp, brp, tlp, trp, blp2, brp2, tlp2, trp2);
// };

// zdroje:
// http://www.useragentman.com/blog/2011/01/07/css3-matrix-transform-for-the-mathematically-challenged/
// http://franklinta.com/2014/09/08/computing-css-matrix3d-transforms/
// https://gist.github.com/fta2012/bd63f7fd9f385870efc0

function getLinearTransform(tlp, trp, brp, blp, tlp2, trp2, brp2, blp2)
{
  // všechny 4 vektory dáme do matice, abychom je mohli násobit naráz
  var vectors = [[blp2.x, brp2.x, tlp2.x, trp2.x], [blp2.y, brp2.y, tlp2.y, trp2.y], [1,1,1,1]];

  function updateVectors(matrix)   /* po nalezení parametrů reverzujeme transformaci */
  {
    vectors = matMultiply(matrix, vectors);
   
    blp2.x = vectors[0][0], blp2.y = vectors[1][0];
    brp2.x = vectors[0][1], brp2.y = vectors[1][1];
    tlp2.x = vectors[0][2], tlp2.y = vectors[1][2];
    trp2.x = vectors[0][3], trp2.y = vectors[1][3];
  };
  
  // výpočty
  var deg = 180 / Math.PI;
  
  var a = Math.atan2(trp2.y - tlp2.y, trp2.x - tlp2.x);  /* sklon TopRight-TopLeft */
  var b = Math.atan2(brp2.y - blp2.y, brp2.x - blp2.x);  /* sklon BottomRight-BottomLeft */
  var c = Math.atan2(trp2.y - brp2.y, trp2.x - brp2.x);  /* sklon TopRight-BottomRight */
  var d = Math.atan2(tlp2.y - blp2.y, tlp2.x - blp2.x);  /* sklon TopLeft-BottomLeft */  

  var avgHorizontalAngle = (a + b) / 2;
  var avgVerticalAngle = (c + d) / 2;

  var matrices = [];
  var css = "";

  //--------------------------------------------
  // ** Skew **
  var skewX = avgHorizontalAngle - (avgVerticalAngle - 90/deg);
  var skewY = 0;
  
  if (Math.abs(skewX * deg) > 90)    /* pro orientace WEST a EAST je třeba úprav */
  {
    skewY = -skewX;
    skewX = 0;
  }

  matrices.push([[1,Math.tan(skewX),0], [Math.tan(skewY),1,0], [0,0,1]]);
  updateVectors([[1,-Math.tan(skewX),0], [-Math.tan(skewY),1,0], [0,0,1]]);
  css += "skewX(" + (skewX * deg) + "deg) skewY(" + (skewY * deg) + "deg) ";
  //css += "skew(" + (skewX * deg) + "deg, " + (skewY * deg) + "deg) ";
  
  // Firefox:
  // x-6 -- nesnese skew slovně ani v matici, nesnese ani jakoukoli jinou matici
  // 7-8 -- nesnese skew slovně ani v matici
  // 9-28 -- nesnese skew slovně, ale v matici ano
  // 29+ -- OK
  if (JAK.Browser.client == "gecko" && JAK.Browser.version < 9)
  {
    updateVectors(matrices.pop());
    css = css.replace(/skew[XY]?\(.*?\)/g, "");
  }

  //--------------------------------------------
  // ** Rotation **
  var angle = avgHorizontalAngle;

  matrices.push([[Math.cos(angle),-Math.sin(angle),0], [Math.sin(angle),Math.cos(angle),0], [0,0,1]]);
  updateVectors([[Math.cos(angle),Math.sin(angle),0], [-Math.sin(angle),Math.cos(angle),0], [0,0,1]]);
  css += "rotate(" + (angle * deg) + "deg) ";
 
  //--------------------------------------------
  // ** Scale **
  var scaleX = (tlp2.distance(trp2) / tlp.distance(trp) + blp2.distance(brp2) / blp.distance(brp)) / 2;
  var scaleY = (trp2.distance(brp2) / trp.distance(brp) + tlp2.distance(blp2) / tlp.distance(blp)) / 2;

  matrices.push([[scaleX,0,0], [0,scaleY,0], [0,0,1]]);
  updateVectors([[1/scaleX,0,0], [0,1/scaleY,0], [0,0,1]]);
  css += "scale(" + scaleX + "," + scaleY + ") ";

  //--------------------------------------------
  // ** Translation **
  var translateX = ((tlp2.x - tlp.x) + (brp2.x - brp.x) + (trp2.x - trp.x) + (blp2.x - blp.x)) / 4;
  var translateY = ((tlp2.y - tlp.y) + (brp2.y - brp.y) + (trp2.y - trp.y) + (blp2.y - blp.y)) / 4;
  
  matrices.push([[1,0,translateX], [0,1,translateY], [0,0,1]]);
  updateVectors([[1,0,-translateX], [0,1,-translateY], [0,0,1]]);
  
  //css += "translate(" + translateX + "px, " + translateY + "px, 0px) ";
  css += "translateX(" + translateX + "px) ";
  css += "translateY(" + translateY + "px) ";
  //css += (JAK.Browser.client != "opera") ? "translateZ(0px) " : "";

  //--------------------------------------------
  // Opera a a starší verze Firefoxu (min. po verzi 18) nemají rádi skew :-(
  // -- takže raději použijeme matici

  if (JAK.Browser.client == "opera" 
   || (JAK.Browser.client == "gecko" && JAK.Browser.version >= 9 && JAK.Browser.version < 29))
  {
    var matrix = matMultiply.apply(this, matrices);
    
    // Opera opravdu nemá v CSS ráda čisla s hodně desetinnými místy :(
    if (JAK.Browser.client == "opera")
    {
      for (var i = 0; i < matrix.length; i++)
        for (var j = 0; j < matrix[i].length; j++)
          matrix[i][j] = matrix[i][j].toFixed(8);
    }

    css = "matrix(" + [matrix[0][0],matrix[1][0],matrix[0][1],matrix[1][1],matrix[0][2],matrix[1][2]] + ")";
  }

  return css;  
}

// **** reproject tiles when layers have incompatible projection ****

SMap.Layer.Tile.prototype.tileTransform = function(tile, node, projection, zoom)
{
  // transform origin musíme nastavit zde při vzniku dílku, později už by byl posunutý   
  if (!node || node.style[SMap.TRANSFORM + "Origin"])
    return;

  // get source corners
  var map = this.getMap();
  var c = tile.tileCorners(map);

  var tlp = c.topLeft.toPixel(map);
  var trp = c.topRight.toPixel(map);
  var blp = c.bottomLeft.toPixel(map);
  var brp = c.bottomRight.toPixel(map);

  // get target corners
  function getTargetCorner(p)
  {
    return projection.coordsToPixel(p, map.getCenter(), zoom, map.getOrientation());
  };

  var tlp2 = getTargetCorner.apply(this, [c.topLeft]);
  var brp2 = getTargetCorner.apply(this, [c.bottomRight]);
  var trp2 = getTargetCorner.apply(this, [c.topRight]);
  var blp2 = getTargetCorner.apply(this, [c.bottomLeft]);

  // nastavení CSS
  var origin = {0: blp, 1: tlp, 2: trp, 3: brp}[map.getOrientation()];
  node.style[SMap.TRANSFORM + "Origin"] = (-origin.x) + "px " + (-origin.y) + "px";
  node.style[SMap.TRANSFORM] = getLinearTransform(tlp, trp, brp, blp, tlp2, trp2, brp2, blp2);
}

// pokud chceme transformovat dílky, je třeba přenastavit zoom, projekci atd.
SMap.Layer.Tile.prototype.enterTileTransform = function(vals)
{
  this.getMap()._zoom = this.getMap().getNearestZoom({
    layerTo: this.getId(),
    higher: this._options.reprojectionZoomHigher,  /* sometimes choose only higher or lower zoom for rendering nicety reasons */
    lower: this._options.reprojectionZoomLower});

  this.getMap()._projection = vals.proj; 

  // zvětšíme průhled o 1 dílek na každé straně
  // -- abychom vyřešili to, že všechny dílky mohou být pootočeny a nezakrývat tak celý průhled
  // -- hodnotu násobíme dvakrát, protože je třeba přidal velikost nahoři i dole, resp. vlevo i vpravo
  var addPixels = 2 * this._options.tileSize;
  this.getMap()._size.plus(new SMap.Pixel(addPixels,  addPixels));
}

SMap.Layer.Tile.prototype.exitTileTransform = function(vals)
{
  // vrátíme projekce a zoom, a ukončíme
  this.getMap()._projection = vals.projOld; 
  this.getMap()._zoom = vals.zoom;
  this.getMap()._size = vals.size;
}

// SMap.TileSet.prototype.setOpacity = function(opacity)
// {
//   this._container.style.opacity = opacity;
// 
//   // doesn't work (will probably work in the future, when seznam.cz adds the ability to change opacity)
//   if (JAK.Browser.client == "ie" && JAK.Browser.version <= 8)
//     this._container.style.filter = "alpha(opacity=" + (opacity * 100) + ")";
// }

encapsulate(SMap.Layer.Tile.prototype, "redraw", function(full)
{
  if (!this._active)
    return;

  // nastavíme transparentní vrstvě novou projekci    
  layerUpdateProjection(this.getMap().getProjection(), this.getId());

  var vals =
  {
    proj: this.getMap().getProjection(this.getId()),       /* projekce, ve které bude aktuální vrstva */
    projOld: this.getMap().getProjection(),                /* projekce mapy */
    zoom: this.getMap().getZoom(),
    size: this.getMap().getSize().clone()
  }

  //--------------------------------------------------------------------
  // rozhodneme zda reprojektovat
  var reproject = false;
  
  // ANO, pokud jsou rozdílné projekce
  if (!vals.projOld.equals(vals.proj))
    reproject = true;
  
  // ANO, pokud je požadovaný zoom vrstvy mimo její aktuální rozsah
  var zoomRange = SMap.CONFIG[this.getId()].zoom;   /* can't use getZoomRange, would break oblique */

  if (Array.isArray(zoomRange) && (vals.zoom < zoomRange[0] || vals.zoom > zoomRange[1]))
    reproject = true;
  
  // NE, pokud není podporováno CSS Transform
  if (!SMap.TRANSFORM)
    reproject = false;

  if (reproject)
    this.enterTileTransform(vals);

  //--------------------------------------------------------------------
  // u hlavní vrstvy načteme víc než jen rozměry obrazovky, abychom při tažení myší viděli hned o kousek dál
  if (this.getId() == this.getMap().CurrentMapId)
  {
    var prefetch = 500;      /* pixelů navíc na každé ze 4 stran */
    var prefetchPx = new SMap.Pixel(2 * prefetch, 2 * prefetch);
    this.getMap().getSize().plus(prefetchPx);
  }
  
  // volání původní funkce
  this.redrawOrig.apply(this, arguments);
  
  // obnovení rozměru viewportu
  if (this.getId() == this.getMap().CurrentMapId)
  {
    this.getMap().getSize().minus(prefetchPx);
  }  

  //--------------------------------------------------------------------
  // nastavení opacity, nastavení transformací dílků, skrytí dílků při chybě

  var tileSet = this._tileSets[0]; /* index 0 odpovídá aktuálnímu zoomu */
  var tiles = tileSet._tiles;

  // nastavení opacity (vypnuto, prováděno u jednotlivých dílků)
  //if (this._options.opacity)
  //  tileSet.setOpacity(this._options.opacity);

  // projití všech dílků
  for (var i in tiles)
  {
    var t = tiles[i].tile;
    var node = tiles[i].node;

    if (!node)
      continue;
    
    // transformace dílků
    if (reproject)
      this.tileTransform(t, node, vals.projOld, vals.zoom);
    
    // pokud dojde k chybě, tak dílek skryjeme 
    node.onerror = function() { node.style.display = "none"; };
  }

  //--------------------------------------------------------------------
  // obnovení nastavení
  if (reproject)
    this.exitTileTransform(vals);
});

// pokud je vrstva průhledná, tak po změně zoomu nebo otočení starou verzi ihned zrušíme
encapsulate(SMap.Layer.Tile.prototype, "_createTileSet", function()
{
  var tileSet = this._createTileSetOrig.apply(this, arguments);
  
	if (this._options.alpha)
  {
    tileSet.onload = null;
    this._tileSetLoaded();
  }
  
  // pro nastavení opacity SMap.Tileset
  tileSet._options.opacity = this._options.opacity || null;
  
  if ("mercatorOpacity" in this._options)
  {
    tileSet._options.mercatorOpacity = this._options.mercatorOpacity;
    tileSet._options.projection = SMap.CONFIG[this.getId()].projection;
  }

  return tileSet;  
});

// seřadíme dlaždice od středu
// -- zde možno hooknout i _createTiles, ale tam pak nemáme přístup k this._tiles a seřazovali bychom i jiné než nové dílky
encapsulate(SMap.TileSet.prototype, "addTiles", function(tiles)
{
  // tiles je object, kde jsou klíčemi SMap.Tile převedené na řetězce (x-y-zoom)
  // -- takže ho musíme převést na pole
  // -- původní kód z Mapy API
	var newTiles = [];
  
	for (var id in tiles)
  {
		if (id in this._tiles) { continue; }
		newTiles.push(tiles[id]);
	}
  
  // dlaždice seřadíme od středu
  var map = this._map;

	newTiles.sort(function(a, b)
  {
		return a.tile.toPixel(map).norm() - b.tile.toPixel(map).norm();
	});
  
  // dáme je zpět do původního objektu
  // -- pořadí by pak ve (for var id in tiles) měla většina prohlížečů dodržet
  // viz: http://stackoverflow.com/questions/5525795/does-javascript-guarantee-object-property-order
  tiles = {};
  
  for (var i = 0; i < newTiles.length; i++)
    tiles[newTiles[i].tile.toString()] = newTiles[i];

  // volání původní funkce
  return this.addTilesOrig(tiles);
});

var _setOpacity = function(node, options, map)
{
  var opacity = options.opacity || 1;

  // některé WMS vrstvy od ČÚZK nefungují, pokud je nastaven Mercator
  if ("mercatorOpacity" in options && options.projection.getCode() == "EPSG:4326")
  {
    opacity = options.mercatorOpacity;
  }

  //--------------
  if ("opacity" in node.style)
    node.style.opacity = opacity;

  // doesn't work (will probably work in the future, when seznam.cz adds the ability to change opacity)
  if (JAK.Browser.client == "ie" && JAK.Browser.version <= 8 && opacity < 1)
    node.style.filter = "alpha(opacity=" + (opacity * 100) + ")";
}

// 1) v IE8 vrací "node.onload = function(e)" parametr e undefined
// -- chyba Seznamu, zatím neopraveno :(

// 2) opacity je třeba nastavit pro každý jednotlivý dílek, jinak nebude fungovat v IE <= 9 a možná i jinde

encapsulate(SMap.TileSet.prototype, "_createTileNode", function()
{
  var node = this._createTileNodeOrig.apply(this, arguments);
  var oldOnload = node.onload;
  
	node.onload = function(e) 
  {
    oldOnload.call(this, {target: node})
    _setOpacity(node, this._options, this._map);

	}.bind(this);
  
  // v IE8 po zapnutí podvrstvy, jejím vypnutí a znovu zapnutí bude filter odstraněn (a tedy opacity bude 1) 
  if (JAK.Browser.client == "ie" && JAK.Browser.version <= 8)
    _setOpacity(node, this._options, this._map);

  return node;  
});

// tato funkce je hlavně pro IE8 a WMS vrstvy, kde lze žádat různou projekci
// -- nastaví transparentní vrstvě novou defaultní projekci (pokud může mít více projekcí) v závislosti na aktuální projekci
// -- pokud funguje CSS transform, tak necháme původní projekci 
var layerUpdateProjection = function(currentProjection, id)
{
  // no change if transform is supported  
  if (SMap.TRANSFORM)
    return;
    
  var conf = SMap.CONFIG[id];
  
  // no change if we no compatible projections set in options  
  if (!conf.options || !conf.options.compatibleProjections)
    return;
    
  var compatible = conf.options.compatibleProjections;
    
  for (var i = 0; i < compatible.length; i++)
  {
    if (currentProjection.equals(compatible[i]))
    {
      conf.projection = compatible[i];
      return; 
    }
  }
}

// zjistíme, zda jsou vrstvy pro reprojekci kompatibilní
var layersAreCompatible = function(id1, id2)
{
  var proj1 = (SMap.CONFIG[id1].options || {}).compatibleProjections || [SMap.CONFIG[id1].projection];
  var proj2 = (SMap.CONFIG[id2].options || {}).compatibleProjections || [SMap.CONFIG[id2].projection];
  
  for (var i = 0; i < proj1.length; i++)
  {
    for (var j = 0; j < proj2.length; j++)
    {
      if (proj1[i].equals(proj2[j]))
        return true;
    }
  } 
  
  return false;
}

//============================================================================
//                              SMap.Tile
//============================================================================

// získání rohů Tile (bez rotace, takže topLeft může být na mapě i třeba bottomRight)
SMap.Tile.prototype.tileCorners = function(map, projection, zoom)
{
	var proj = projection || map.getProjection();
  var zoom = zoom || map.getZoom();
  
  // topLeft
  var coef = proj._screenCoefficients;
  var x = this.tileSize * (this.x + (coef.x == -1 ? 1 : 0));
  var y = this.tileSize * (this.y + (coef.y == 1 ? 1 : 0));
  var tl = proj._absolutePixelToCoords(new SMap.Pixel(x, y), zoom);

  // topRight, bottomRight, bottomLeft
  var x = this.tileSize * coef.x;
  var y = this.tileSize * coef.y * -1;
  var tr = proj.shiftCoords(tl, zoom, new SMap.Pixel(x, 0));
  var br = proj.shiftCoords(tl, zoom, new SMap.Pixel(x, y));
  var bl = proj.shiftCoords(tl, zoom, new SMap.Pixel(0, y));

  return {topLeft: tl, topRight: tr, bottomRight: br, bottomLeft: bl};
}

// rohy orotovaného Tile
SMap.Tile.prototype.tileCornersRotated = function(map, projection, zoom)
{
  var c = this.tileCorners.apply(this, arguments);

  switch (map.getOrientation())
  {
    case SMap.NORTH: return c;
    case SMap.WEST:  return {topLeft: c.bottomLeft,  topRight: c.topLeft,     bottomRight: c.topRight,   bottomLeft: c.bottomRight};
    case SMap.SOUTH: return {topLeft: c.bottomRight, topRight: c.bottomLeft,  bottomRight: c.topLeft,    bottomLeft: c.topRight};
    case SMap.EAST:  return {topLeft: c.topRight,    topRight: c.bottomRight, bottomRight: c.bottomLeft, bottomLeft: c.topLeft};
  }
}

SMap.Tile.prototype.getBBOX = function(map, version)
{
  version = version || "1.3.0";
  var proj = map.getProjection();
  var c = this.tileCorners(map, proj);
  var tl = c.topLeft;
  var br = c.bottomRight;

  // UTM33
  if (proj.getCode() == "EPSG:32633")
  {
    tl = tl.toUTM33(); /* [lon, lat] */
    br = br.toUTM33();
    return [tl[0], tl[1], br[0], br[1]].join();
  }  

  // WGS84
  if (proj.getCode() == "EPSG:4326")
  {
    // 1.1.1 requires Lon,Lat; new versions require Lat,Lon
    if (version == "1.1.1")
      return [tl.lon(), tl.lat(), br.lon(), br.lat()].join();
    else
      return [tl.lat(), tl.lon(), br.lat(), br.lon()].join();
  }  
    
  // S-JTSK
  else if (proj.getCode() == "EPSG:102067")
  {
    tl = tl.toSJTSK();
    br = br.toSJTSK();

    return [-tl.x, -tl.y, -br.x, -br.y].join();
  }
  
  return "";
}

//============================================================================
//                             SMap.Layer.WMS
//============================================================================

/* původní verze SMap.Layer.WMS */
/* -- potřebujeme ji, abychom mohli reprojektovat dílky, nastavit jejich velikost, aby byla podporována změna orientace atd. */

// vše o WMS: http://docs.geoserver.org/latest/en/user/services/wms/basics.html

// Seznam code
SMap.Layer.WMS = JAK.ClassMaker.makeClass(
{
	NAME: "SMap.Layer.WMS",
	EXTEND: SMap.Layer.Tile
});

// Seznam code
SMap.Layer.WMS.prototype.$constructor = function(id, url, params, options)
{
	this._params = 
  {
    request: "GetMap",
		format: "image/jpeg",
		service: "WMS",
		version: "1.3.0",
		styles: ""
	};
  
	for (var p in params) { this._params[p] = params[p]; }
	this.$super(id, url, options);
}

SMap.Layer.WMS.prototype._buildURL = function(tile)
{
	var data = 
  {
		width: this._options.tileSize,
		height: this._options.tileSize,
		crs: this.getMap().getProjection().getCode()
	};
  
  for (var p in this._params) { data[p] = this._params[p]; }
  
  // u 1.1.1 je použit parametr SRS místo CRS
  if (data.version == "1.1.1")
  {
    data.srs = data.srs /*pokud bylo uživatelsky vloženo, neměníme*/ || data.crs;
    delete data.crs;
  }
  
  // získání bBoxu
  data.bbox = tile.getBBOX(this.getMap(), data.version);

  // Seznam code
	var params = [];
  
	for (var p in data) 
  {
		params.push(encodeURIComponent(p) + "=" + encodeURIComponent(data[p]));
	}

	return this._url + "?" + params.join("&");
}


//============================================================================
//                             SMap.Layer.ArcGIS
//============================================================================

SMap.Layer.ArcGIS = JAK.ClassMaker.makeClass(
{
	NAME: "SMap.Layer.ArcGIS",
	EXTEND: SMap.Layer.Tile
});

// SMap.Layer.ArcGIS.prototype.$constructor = function(id, url,  options)
// {
// 	this.$super.apply(this, arguments);
// }

SMap.Layer.ArcGIS.prototype._buildURL = function(tile)
{
  return (this._url + tile.zoom + "/" + tile.y + "/" + tile.x); /* {zoom}/{y}/{x} */
}

//============================================================================
//                          SMap.Projection
//============================================================================

// funkce, která zjistí, zda jsou projekce kompatibilní
SMap.Projection.prototype.equals = function(projection)
{
  return this.getCode() == projection.getCode();
}

// v ptačím pohledu (zoom 21), na okraji oblasti někdy dojde při zoomování k chybě
// -- chyba je: "Uncaught exception: TypeError: 'this.getMap' is not a function"
encapsulate(SMap.Projection.Oblique.prototype, "_response", function(proj)
{
  try
  {
    return(this._responseOrig(proj));
  }
  catch(ex) {};
});

//============================================================================
//                        SMap.POIServer.XML/FRPC
//============================================================================

function fixPoi()
{
  // toto asi není používáno (?), ale stejně to sem dáme
  encapsulate(SMap.POIServer.XML.prototype, "_buildLookup", function()
  {
    this._options.zoomCorrection = this.getMap().getNearestZoom({layerTo: SMap.DEF_OPHOTO}) - this.getMap().getZoom();
    return(this._buildLookupOrig.apply(this, arguments));
  });

  encapsulate(SMap.POIServer.FRPC.prototype, "_buildLookupArguments", function()
  {
    this._options.zoomCorrection = this.getMap().getNearestZoom({layerTo: SMap.DEF_OPHOTO}) - this.getMap().getZoom();
    return(this._buildLookupArgumentsOrig.apply(this, arguments));
  });
  
  // abychom zabránili opakovanému spouštění
  window.fixPoi = function() {};
}

//============================================================================
//                                   Misc
//============================================================================

// create default projections (so that we don't have to create them each time we change a map)
SMap.projUTM33 = new SMap.Projection.UTM33();
SMap.projMercator = new SMap.Projection.Mercator();
SMap.projKrovak = new SMap.Projection.Krovak();
 
// this functions sometimes fails in IE7 and maybe IE8 - attempt to fix it
encapsulate(JAK.VML.prototype, "setCenterRadius", function(element, center, radius)
{
  try
  {
    return(this.setCenterRadiusOrig(element, center, radius));
  }
  catch(ex){/*log("JAK.VML setCenterRadius error");*/};
});

// ************
// fix local XHR (for oblique layer), not needed for IE11+ (but keep it turned off for now)
if (JAK.Browser.client == "ie" /*&& JAK.Browser.version <= 10*/ && window.location.protocol == "file:")   
{
  // remove layer 21 for IE8-
  // also doesn't work in IE9 when using MOTW
  // (http://msdn.microsoft.com/en-us/library/ms537628(v=vs.85).aspx)
  if (JAK.Browser.version <= 8)
     SMap.CONFIG.obliqueExtents = [];

  // in IE9+, change XDomainRequest to XMLHttpRequest (otherwise calls will be denied)  
  else
    XDomainRequest = XMLHttpRequest;
}

// detekce verze Firefoxu (JAK.Browser umí detekovat jen po verzi 7)
encapsulate(JAK.Browser, "_get_gecko_ver", function()
{
  var version = JAK.Browser._get_gecko_verOrig();
    
  if (version == 7)
  {
    try
    {  
      // např. Mozilla/5.0 (Windows NT 6.1; rv:36.0) Gecko/20100101 Firefox/36.0
      version = JAK.Browser.agent.match(/Firefox\/([\d\.]+)/)[1];
      version = version.split(".");
      version = version.shift() + "." + version.join("");
      version = parseFloat(version);

      if (version > 7)
        return version;
    }
    catch(ex) { return 7; }
  }
  
  return version;
});

if (JAK.Browser.client == "gecko" && JAK.Browser.version == 7)
{
  JAK.Browser.version = JAK.Browser._get_gecko_ver();
}

// pokud je prohlížeč Maxthon 3, tak se nezíská správná verze
encapsulate(JAK.Browser, "_getClient", function()
{
  var client = JAK.Browser._getClientOrig();
  
  if (client == "oth" && this._agent.indexOf("Safari") != -1)
    client = "safari";
    
  return client;
});

if (JAK.Browser.client == "oth")
{
  JAK.Browser.client = JAK.Browser._getClient();
}

//============================================================================
SMap.Marker.prototype.getDom = function()
{
	return this._dom.container[SMap.LAYER_MARKER];
}

//*****************************************************

SMap.Control.Button = JAK.ClassMaker.makeClass(
{
	NAME: "SMap.Control.Button",
	VERSION: "1.0",
	EXTEND: SMap.Control.Visible
});

SMap.Control.Button.prototype.$constructor = function(options)
{
	this.$super();
  
	this._options = 
  {
		label: "",
    id: "",
    className: "button",
    styles: {},
    handler: function() {}
	}

	for (var p in options) { this._options[p] = options[p]; }
  this._build();
  
}

SMap.Control.Button.prototype._build = function()
{
	this._dom.container = JAK.mel("div", 
    {id: this._options.id, className: this._options.className, innerHTML: this._options.label},
    this._options.styles);

  this._ec.push(JAK.Events.addListener(this._dom.container, "click", this, this._options.handler));
}






// vypneme některé (v tuto chvíli) zbytečnosti

// SMap.Layer.Marker.prototype._build = function() {
// 	//this._dom.container[SMap.LAYER_SHADOW] = JAK.mel("div", {id:this._id + "-" + SMap.LAYER_SHADOW});
// 	this._dom.container[SMap.LAYER_MARKER] = JAK.mel("div", {id:this._id + "-" + SMap.LAYER_MARKER});
// 	//this._dom.container[SMap.LAYER_ACTIVE] = JAK.mel("div", {id:this._id + "-" + SMap.LAYER_ACTIVE});
// 	
// 	//this._ec.push(JAK.Events.addListener(this._dom.container[SMap.LAYER_ACTIVE], "click", this, "_click"));
// 	this._ec.push(JAK.Events.addListener(this._dom.container[SMap.LAYER_MARKER], "click", this, "_click"));
// 	//this._ec.push(JAK.Events.addListener(this._dom.container[SMap.LAYER_ACTIVE], "mousedown", this, "_mouseDown"));
// 	this._ec.push(JAK.Events.addListener(this._dom.container[SMap.LAYER_MARKER], "mousedown", this, "_mouseDown"));
//   
//   //this._dom.container[SMap.LAYER_SHADOW] = {style: {}};
//   //this._dom.container[SMap.LAYER_ACTIVE] = {style: {}};
// }