/* custom map definitions */
var default_preview_path = window.basePath + "Images/Maps/";

SMap.CONFIG.bBox_CZ = {latMin: 48, latMax: 52, lonMin: 11, lonMax: 20};
SMap.CONFIG.bBox_MapyCZ = {latMin: 38, latMax: 57, lonMin: 5, lonMax: 25};     /* former UTM33 bBox */ 
SMap.CONFIG.bBox_Mercator = {latMin: -75, latMax: 85, lonMin: -SMap.Coords.maxLon, lonMax: SMap.Coords.maxLon};

// vytvoříme z existující vrstvy novou vrstvu, která bude průhledná (tedy ji můžeme dát jako podřízenou vrstvu bez různých problémů)
function createSubLayer(id, options)
{
  var conf = {};
  options = options || {};
  
  // vytvoříme novou vrstvu překopírováním parametrů z původní vrstvy
  for (var prop in SMap.CONFIG[id])
    conf[prop] = SMap.CONFIG[id][prop];
  
  // smažeme vlastnosti, které mají netransparentní vrstvy
  delete conf.label;
  delete conf.title;
  delete conf.preview;      /* tohle tu nesmí být, SMap.Layer.prototype.enable by to považovalo za hlavní vrstvu */
  
  // defaultně přidáme následující options
  conf.options = {fadeTime: 0, alpha: true, opacity: 0.3};

  // překopírujeme options z původní vrstvy
  if (SMap.CONFIG[id].options)
  {
    for (var prop in SMap.CONFIG[id].options)
    {
      if (!(prop in conf.options))
        conf.options[prop] = SMap.CONFIG[id].options[prop];
    }
  }
  
  // překopírujeme do options to co jsme dodali v parametru options
  for (var prop in options)
    conf.options[prop] = options[prop];
  
  // smažeme zoom, pokud se jedná o WMS vrstvu  
  if (conf.WMS && conf.zoom)
    delete conf.zoom;

  return conf;  
}

//======================================================================================
//                                   mapy ČÚZK
//======================================================================================
SMap.CONFIG["ZM10_ID"] = 
{
  zoom: [6, 19],
  label: "ZM10",
  title: "Základní mapa 10k",
  preview: default_preview_path + "ZM10.jpg",
  url: "",
  options: {tileSize: 256},
  WMS: {layers: "", format: "image/png8", transparent: "FALSE"},
  copyright: {"-": "ČÚZK"},
  projection: SMap.projMercator,

  buildUrl: function(t /*tile*/)
  {
    if      (t.zoom >= 14) this._params.layers = "GR_ZM10";
    else if (t.zoom >= 11) this._params.layers = "GR_ZM50";
    else if (t.zoom >= 0)  this._params.layers = "GR_CR4";
    
    if      (t.zoom >= 14) var mapDir = "WMS_ZM10_PUB";
    else if (t.zoom >= 11) var mapDir = "WMS_ZM50_PUB";
    else if (t.zoom >= 0)  var mapDir = "WMS_PREHLEDKY";
    
    // ZM10 má složitější link
    var zm_link = (t.zoom >= 14) ? ("-zm" + Math.floor(Math.random() * 4)) : "";
    
    this._url = "http://geoportal" + zm_link + ".cuzk.cz" + "/" + mapDir + "/service.svc/get";
    
    return this._buildURLOrig(t);
  },
  bBox: SMap.CONFIG.bBox_CZ 
};

SMap.CONFIG["ZM10_SJTSK_ID"] = 
{
  zoom: [0, 10],
  label: "ZM10 Turbo",
  title: "Základní mapa 10k (S-JTSK verze - rychlejší)",
  preview: default_preview_path + "ZM10.jpg",
  options: {tileSize: 256, ArcGIS: true},
  url: "http://ags.cuzk.cz/ArcGIS/rest/services/zm/MapServer/tile/",  /* někdy je třeba prozkoumat Web Mercator verzi zmwm */
  projection: new SMap.Projection.Krovak({origin: {x: -33699800, y: 33699800}, resolution: 1024.1300482601}),
  copyright: {"-": ["Český úřad zeměměřický a katastrální"]},
  bBox: SMap.CONFIG.bBox_CZ  
};

// SMap.CONFIG["myin"] = 
// {
//   zoom: [0, 6],
//   label: "ZM10 Turbo",
//   title: "Základní mapa 10k (S-JTSK verze - rychlejší)",
//   preview: default_preview_path + "ZM10.jpg",
//   options: {tileSize: 2048, ArcGIS: true, alpha: true},
//   buildUrl: function(t /*tile*/)
//   {
//     var path = basePath + "tiles/zoom_" + t.zoom + "/" + t.y + "_" + t.x + ".jpg";
//     return path;
//   },
//   url: basePath + "tiles/zoom_{zoom}/{y}_{x}.jpg",
//   projection: new SMap.Projection.Krovak({origin: {x: -704461.4800093155, y: -1098899.9691483772}, resolution: 0.25*51.0209619776 /*51.021547808*/}),
//   copyright: {"-": ["Český úřad zeměměřický a katastrální"]},
//   bBox: SMap.CONFIG.bBox_CZ  
// };

SMap.CONFIG["Orto_CZ_ID"] =
{
  zoom: [0, 12],
  label: "Ortofoto ČR",
  title: "Ortofoto ČR",
  preview: default_preview_path + "Orto_CZ.png",
  options: {tileSize: 256, ArcGIS: true},
  url: "http://ags.cuzk.cz/ArcGIS/rest/services/ortofoto/MapServer/tile/",
  projection: new SMap.Projection.Krovak({origin: {x: -33699800, y: 33699800}, resolution: 1024.1300482601}),
  copyright: {"-": ["Český úřad zeměměřický a katastrální"]},
  bBox: SMap.CONFIG.bBox_CZ  
};

SMap.CONFIG["ZABAGED_ID"] = 
{
  zoom: [0, 14],
  label: "ZABAGED",
  title: "Základní báze geografických dat (ZABAGED)",
  preview: default_preview_path + "ZABAGED.png",
  url: "http://geoportal.cuzk.cz/WMTS_ZABAGED/WMTService.aspx",
  options: {tileSize: 256},
  WMTS: {layer: "zbg", tileMatrixSet: "jtsk:epsg:5514", style: "default"},
  projection: new SMap.Projection.Krovak({origin: {x: -925000, y: -920000}}),
  copyright: {"-": ["Český úřad zeměměřický a katastrální"]},
  bBox: SMap.CONFIG.bBox_CZ  
};

// historické ortofoto od ČÚZK
// 1998,1999,2000,2001,2002 (vrstvy, kromě 2002, nejdou překrývat :( )
// 2003,2004,2005
// 2006,2007,2008
// 2009,2010,2011
// 2012, (2013 není zatím dostupné)
 
SMap.CONFIG["Orto_1998_2002_ID"] = 
{
  zoom: [7, 21],
  label: "Ortofoto 00-02",
  title: "Ortofoto ČÚZK 2000-2002",
  preview: default_preview_path + "Orto_CZ_Historicke.png",
  url: "http://geoportal.cuzk.cz/WMS_ORTOFOTO_ARCHIV/WMService.aspx",
  options: {tileSize: 512},
  WMS: {layers: "2000,2002" /*1998,1999,2000,2001,2002*/, version: "1.3.0", transparent: "false", format: "image/jpeg"},
  projection: SMap.projMercator,
  copyright: {"-": "Český úřad zeměměřický a katastrální"},
  bBox: SMap.CONFIG.bBox_CZ    
};

SMap.CONFIG["Orto_2003_2005_ID"] = 
{
  zoom: SMap.CONFIG["Orto_1998_2002_ID"].zoom,
  label: "Ortofoto 03-05",
  title: "Ortofoto ČÚZK 2003-2005",
  preview: SMap.CONFIG["Orto_1998_2002_ID"].preview,
  url: SMap.CONFIG["Orto_1998_2002_ID"].url,
  options: SMap.CONFIG["Orto_1998_2002_ID"].options,
  WMS: {layers: "2003,2004,2005", version: "1.3.0", transparent: "false", format: "image/jpeg"},
  projection: SMap.CONFIG["Orto_1998_2002_ID"].projection,
  copyright: SMap.CONFIG["Orto_1998_2002_ID"].copyright,
  bBox: SMap.CONFIG["Orto_1998_2002_ID"].bBox
};

SMap.CONFIG["Orto_2006_2008_ID"] = 
{
  zoom: SMap.CONFIG["Orto_1998_2002_ID"].zoom,
  label: "Ortofoto 06-08",
  title: "Ortofoto ČÚZK 2006-2008",
  preview: SMap.CONFIG["Orto_1998_2002_ID"].preview,
  url: SMap.CONFIG["Orto_1998_2002_ID"].url,
  options: SMap.CONFIG["Orto_1998_2002_ID"].options,
  WMS: {layers: "2006,2007,2008", version: "1.3.0", transparent: "false", format: "image/jpeg"},
  projection: SMap.CONFIG["Orto_1998_2002_ID"].projection,
  copyright: SMap.CONFIG["Orto_1998_2002_ID"].copyright,
  bBox: SMap.CONFIG["Orto_1998_2002_ID"].bBox
};

SMap.CONFIG["Orto_2009_2011_ID"] = 
{
  zoom: SMap.CONFIG["Orto_1998_2002_ID"].zoom,
  label: "Ortofoto 09-11",
  title: "Ortofoto ČÚZK 2009-2011",
  preview: SMap.CONFIG["Orto_1998_2002_ID"].preview,
  url: SMap.CONFIG["Orto_1998_2002_ID"].url,
  options: SMap.CONFIG["Orto_1998_2002_ID"].options,
  WMS: {layers: "2009,2010,2011", version: "1.3.0", transparent: "false", format: "image/jpeg"},
  projection: SMap.CONFIG["Orto_1998_2002_ID"].projection,
  copyright: SMap.CONFIG["Orto_1998_2002_ID"].copyright,
  bBox: SMap.CONFIG["Orto_1998_2002_ID"].bBox  
};

SMap.CONFIG["Orto_2012_2013_ID"] = 
{
  zoom: SMap.CONFIG["Orto_1998_2002_ID"].zoom,
  label: "Ortofoto 12-13",
  title: "Ortofoto ČÚZK 2012-2013",
  preview: SMap.CONFIG["Orto_1998_2002_ID"].preview,
  url: SMap.CONFIG["Orto_1998_2002_ID"].url,
  options: SMap.CONFIG["Orto_1998_2002_ID"].options,
  WMS: {layers: "2012,2013", version: "1.3.0", transparent: "false", format: "image/jpeg"},
  projection: SMap.CONFIG["Orto_1998_2002_ID"].projection,
  copyright: SMap.CONFIG["Orto_1998_2002_ID"].copyright,
  bBox: SMap.CONFIG["Orto_1998_2002_ID"].bBox  
};

// Katastrální mapy
// info: http://www.cuzk.cz/Dokument.aspx?PRARESKOD=998&MENUID=0&AKCE=DOC:10-WMS_PRO_KM
// DEF_BUDOVY, DEF_PARCELY --> červené/modré nápisy (čitelnější pro nižší rozlišení)
// KN - katastr černý, KN_I - katastr bílý
// hranice_parcel_barevne - digitalizovaný katastr
// -- Mercator nevypadá hezky
// -- také možno použít SMap.projUTM33
SMap.CONFIG["Katastr"] = 
{
  url: "http://services.cuzk.cz/wms/wms.asp",
  options: {fadeTime: 0, alpha: true, tileSize: 512, compatibleProjections: [SMap.projKrovak, SMap.projMercator, SMap.projUTM33]},
  WMS: {layers: "KN_I", version: "1.3.0", transparent: "true", format: "image/png"},
  copyright: {"-": "Český úřad zeměměřický a katastrální"},
  projection: SMap.projKrovak 
};

SMap.CONFIG["DefinicniBodyBudov"] = 
{
  url: SMap.CONFIG["Katastr"].url,
  options: SMap.CONFIG["Katastr"].options,
  WMS: {layers: "DEF_BUDOVY", version: "1.3.0", transparent: "true", format: "image/png"},
  copyright: SMap.CONFIG["Katastr"].copyright,
  projection: SMap.CONFIG["Katastr"].projection
};

SMap.CONFIG["DefinicniBodyParcel"] = 
{
  url: SMap.CONFIG["Katastr"].url,
  options: SMap.CONFIG["Katastr"].options,
  WMS: {layers: "DEF_PARCELY,hranice_parcel_barevne", version: "1.3.0", transparent: "true", format: "image/png"},
  copyright: SMap.CONFIG["Katastr"].copyright,
  projection: SMap.CONFIG["Katastr"].projection
};

SMap.CONFIG["GeomorfologickeCelky_ID"] = 
{
  zoom: [6, 14],
  label: "Geomorfologické celky",
  title: "Geomorfologicke celky ČR",
  preview: default_preview_path + "GeomorfologickeCelky.png",
  url: "http://ags.cuzk.cz/arcgis/services/GeomorfologickeJednotky/MapServer/WmsServer",
  options: {tileSize: 256},
  WMS: {layers: "0", format: "image/jpeg", transparent: "TRUE"},
  copyright: {"-": "Český úřad zeměměřický a katastrální"},
  projection: SMap.projMercator,
  bBox: SMap.CONFIG.bBox_CZ 
};

SMap.CONFIG["GeomorfologickeCelkySub_ID"] = createSubLayer("GeomorfologickeCelky_ID",
  {opacity: 0.6, compatibleProjections: [SMap.projKrovak, SMap.projMercator, SMap.projUTM33]});

// Vrstevnice
// info: http://directory.spatineo.com/service/3806/
// Vrstevnice základní (GL_CA010)	
// Vrstevnice zdůrazněná (GL_CA011)	
// Vrstevnice doplňková (GL_CA012)       
SMap.CONFIG["Vrstevnice_ID"] = 
{
  url: "http://geoportal.cuzk.cz/WMS_ZABAGED_PUB/service.svc/get",
  options: {tileSize: 256, alpha: true, opacity: 0.4, mercatorOpacity: 0.4,
    compatibleProjections: [SMap.projKrovak, SMap.projMercator, SMap.projUTM33]},
  WMS: {layers: "GL_CA010,GL_CA011,GL_CA012", format: "image/png", transparent: "TRUE"},
  copyright: {"-": "GeoPortal.gov / ČÚZK"},
  buildUrl: function(t /*tile*/)
  {
    // higher opacity for OSM maps (forests there decrease contour visibility)
    if (this.getMap().CurrentMapId == "OSM_ID")
      this._options.opacity = 0.7;

    return this._buildURLOrig(t);
  },
  // for Mercator we need resolution 1200x420 (which is currently impossible) to prevent white background in WGS84 mode  
  projection: SMap.projKrovak /* Mercator je neprůhledný :( */
};

// Bodová pole:
// info: http://geoportal.cuzk.cz/(S(fgnv4m5hz0pfm22evqtqn0kf))/Default.aspx?mode=TextMeta&side=wms.verejne&metadataID=CZ-CUZK-WMS-BODOVAPOLE-P&metadataXSL=metadata.sluzba&head_tab=sekce-03-gp&menu=3123

// Stanice_site_CZEPOS - Stanice sítě CZEPOS
// BP_CZEPOS - Přehled stanic sítě CZEPOS
// BP_POPIS_CZEPOS - Označení stanic sítě CZEPOS
// Polohove_bodove_pole - Polohové bodové pole
// BP_TB - Bod ZPBP
// BP_TB_ETRS - Bod ZPBP určený v ETRS
// BP_TB_RTK - Bod ZPBP určený v ETRS metodou RTK
// BP_ZPBP_ETRS_ZRUS - Zrušený bod ZPBP určený v ETRS
// BP_POPIS_ZPBP_ETRS_ZRUS - Čísla zrušených bodů ZPBP určených v ETRS
// BP_ZHB - ZhB
// BP_ZHB_ETRS - ZhB určený v ETRS
// BP_ZHB_RTK - ZhB určený v ETRS metodou RTK
// BP_POPIS_PBP - Čísla bodů ZPBP a ZhB
// BP_PRIDRUZ - Přidružený bod
// BP_PRIDRUZ_ETRS - Přidružený bod určený v ETRS
// BP_PRIDRUZ_RTK - Přidružený bod určený v ETRS metodou RTK
// BP_PPBP - Bod PPBP
// BP_POPIS_PPBP - Čísla bodů PPBP
// Vyskove_bodove_pole - Výškové bodové pole
// BP_ZVBP - Bod ZVBP
// BP_PVBP - Bod PVBP
// BP_POPIS_VBP - Čísla bodů VBP
// Tihove_bodove_pole - Tíhové bodové pole
// BP_ZTBP - Bod ZTBP
// BP_POPIS_ZTBP - Čísla bodů ZTBP

// hromadné kategorie: Stanice_site_CZEPOS, Polohove_bodove_pole, Vyskove_bodove_pole, Tihove_bodove_pole
// CZEPOS: Stanice_site_CZEPOS,BP_CZEPOS,BP_POPIS_CZEPOS
// Základní polohové bodové pole (ZPBP): BP_TB,BP_TB_ETRS,BP_TB_RTK,BP_ZPBP_ETRS_ZRUS,BP_POPIS_ZPBP_ETRS_ZRUS
// Zhušťovací body (ZhB): BP_ZHB,BP_ZHB_ETRS,BP_ZHB_RTK,BP_POPIS_PBP
// Přidružené body: BP_PRIDRUZ,BP_PRIDRUZ_ETRS,BP_PRIDRUZ_RTK
// Podrobné polohové bodové pole (PPBP): BP_PPBP,BP_POPIS_PPBP
// Výškové bodové pole (VBP: BP_ZVBP,BP_PVBP,BP_POPIS_VBP
// Tíhové bodové pole (TBP): BP_ZTBP,BP_POPIS_ZTBP

SMap.CONFIG["BodovaPole_ID"] = 
{
  url: "http://geoportal.cuzk.cz/WMS_BODPOLE/WMService.aspx",
  options: {tileSize: 512, alpha: true, mercatorOpacity: 0.5,
    compatibleProjections: [SMap.projKrovak, SMap.projMercator, SMap.projUTM33]},
  WMS: {layers: "Stanice_site_CZEPOS,Tihove_bodove_pole,Polohove_bodove_pole", format: "image/png", transparent: "true"},
  copyright: {"-": "Český úřad zeměměřický a katastrální"},
  projection: SMap.projKrovak /* Mercator je neprůhledný :( */
};

// Správní a katastrální hranice ČR
// info: http://geoportal.cuzk.cz/(S(fgnv4m5hz0pfm22evqtqn0kf))/Default.aspx?mode=TextMeta&side=wms.verejne&metadataID=CZ-CUZK-WMS-SPRAVHRAN-P&metadataXSL=metadata.sluzba&head_tab=sekce-03-gp&menu=3120
// -- k ničemu :(, nefunguje při přiblížení

// GP_SPH_STAT_G - státní hranice
// GP_SPH_OBLAST_G - oblasti
// GP_SPH_KRAJ_G - kraje
// GP_SPH_OKRES_G - okresy
// GP_SPH_ORP_G - obce s rozšířenou působností
// GP_SPH_OPU_G - pověřené obce
// GP_SPH_OBEC - obce
// GP_SPH_MC - městské části
// GP_SPH_SO - správní obvody
// GP_SPH_KU - katastrální území
// GP_SPH_ZSJ - ZSJ
// (co má příponu G je generální, funguje i bez přípony)

// SMap.CONFIG["SpravniHranice_ID"] = 
// {
//   url: "http://geoportal.cuzk.cz/WMS_SPH_PUB/WMService.aspx",
//   options: {tileSize: 512, alpha: true},
//   WMS: {layers: "GP_SPH_STAT_G", format: "image/png", transparent: "TRUE"},
//   copyright: {"-": "Český úřad zeměměřický a katastrální"},
//   projection: SMap.projKrovak
// };

//======================================================================================
//                              mapy z Geoportal.gov.cz
//======================================================================================

SMap.CONFIG["VojMapIII_ID"] = 
{
  zoom: [5, 14],
  label: "III. voj. mapování",
  title: "III. vojenské mapování (1876-1878)",
  preview: default_preview_path + "III. vojenske mapovani.png",
  url: "http://geoportal.gov.cz/ArcGIS/services/CENIA/cenia_rt_III_vojenske_mapovani/MapServer/WMSServer",
  options: {tileSize: 512},
  WMS: {layers: "0", format: "image/png8"},
  projection: SMap.projUTM33,
  copyright: {"-": ["3rd Military Survey, Austrian State Archive", "Datový podklad MŽP ČR", "Laboratoř geoinformatiky UJEP"]},
  bBox: SMap.CONFIG.bBox_CZ  
};

SMap.CONFIG[SMap.DEF_HISTORIC] =  /* větší zoom než SMap.DEF_HISTORIC */
{
	zoom: [5, 15],
	label: "II. voj. mapování",
	title: "II. vojenské mapování (1836–1852)",
	preview: "preview-historicka.png",
  url: "http://geoportal.gov.cz/ArcGIS/services/CENIA/cenia_rt_II_vojenske_mapovani/MapServer/WMSServer",
  options: {tileSize: 512},
  WMS: {layers: "0", format: "image/png8"},
  projection: SMap.projUTM33,
  copyright: {"-": ["2nd Military Survey, Austrian State Archive", "Datový podklad MŽP ČR", "Laboratoř geoinformatiky UJEP"]},
  bBox: SMap.CONFIG.bBox_CZ  
};

SMap.CONFIG["Vojenska_ID"] = 
{
  zoom: [0, 11],
  label: "Vojenská mapa",
  title: "Vojenská mapa",
  preview: default_preview_path + "Vojenska mapa.png",
  url: "http://geoportal.gov.cz/arcgis/rest/services/CENIA/cenia_rt_RETM/MapServer/WMTS",
  options: {tileSize: 256}, /* alpha: true by bylo dobré pro nízké zooomy */
  WMTS: {tileMatrixSet: "default028mm", style: "default"},
  projection: new SMap.Projection.Krovak({origin: {x: -920000, y: -920000}}),
  copyright: {"-": ["3rd Military Survey, Austrian State Archive", "Datový podklad MŽP ČR", "Laboratoř geoinformatiky UJEP"]},
  bBox: SMap.CONFIG.bBox_CZ  
};

SMap.CONFIG["DigitalniModelUzemi_ID"] = 
{
  zoom: [0, 12],
  label: "DMÚ 25",
  title: "Digitální model území DMÚ25",
  preview: default_preview_path + "DMU.png",
  url: "http://geoportal.gov.cz/arcgis/rest/services/CENIA/cenia_t_podklad/MapServer/WMTS",
  options: {tileSize: 256, alpha: true},
  WMTS: {tileMatrixSet: "default028mm", style: "default"},
  projection: new SMap.Projection.Krovak({origin: {x: -920000, y: -920000}}),
  copyright: {"-": ["Geografická služba AČR"]},
  bBox: SMap.CONFIG.bBox_CZ  
};

SMap.CONFIG["Automapa_ID"] = 
{
  zoom: [0, 8],
  label: "Automapa",
  title: "Automapa",
  preview: default_preview_path + "Automapa.png",
  url: "http://geoportal.gov.cz/arcgis/rest/services/CENIA/cenia_rt_automapy/MapServer/WMTS",
  options: {tileSize: 256},
  WMTS: {tileMatrixSet: "default028mm", style: "default"},
  projection: new SMap.Projection.Krovak({origin: {x: -3088633, y: 1519883}, resolution: 2048.260096520193 / 2}),
  copyright: {"-": ["PlanStudio"]},
  bBox: SMap.CONFIG.bBox_MapyCZ
};

// jiná verze reliéfu (vybledlá)
SMap.CONFIG["Relief2_ID"] = 
{
  url: "http://geoportal.gov.cz/ArcGIS/services/CENIA/cenia_stinovani/MapServer/WMSServer",
  options: {fadeTime: 0, alpha: true, opacity: 0.25},
  WMS: {layers: "0", format: "image/png", transparent: "TRUE"},
  copyright: {"-": "ČÚZK"},
  projection: SMap.projUTM33
};

//======================================================================================
//                               další mapy v S-JTSK
//======================================================================================

SMap.CONFIG["Muller_ID"] = 
{
  zoom: [2, 9],
  label: "Müllerova mapa",
  title: "Müllerova mapa Čech a Moravy (1720)",
  preview: default_preview_path + "Mullerova mapa.png",
  url: "http://mapy.vugtk.cz/cgi-bin/mapserv",
  options: {tileSize: 512},
  WMS: {layers: "muller,mullermorava", map: "/var/www/map_data/wms.map", srs: "epsg:2065" /*S-JTSK Ferro*/, version: "1.1.1", format: "image/jpeg"},
  projection: SMap.projKrovak,
  copyright: {"-": ["VÚGTK"]},
  bBox: SMap.CONFIG.bBox_CZ 
};

// IZS
// http://giscr.hzspk.cz/arcgis/rest/services/giscr/MapServer/tile/1/256/243
// http://giscr.hzspk.cz/arcgis/rest/services/giscr/MapServer/tile/0/128/121
SMap.CONFIG["IZS_ID"] = 
{
  zoom: [0, 9],
  label: "IZS",
  title: "Mapa IZS",
  preview: default_preview_path + "IZS.png",
  options: {tileSize: 512, ArcGIS: true},
  url: "http://giscr.hzspk.cz/arcgis/rest/services/giscr/MapServer/tile/",
  copyright: {"-": ["HZS ČR (giscr.hzspk.cz)", "ČÚZK", "OpenStreetMap"]},
  projection: new SMap.Projection.Krovak({origin: {x: -33699800, y: 33699800}, 
    resolution: 529.167725002117, scales: [2000000, 1000000, 500000, 200000, 100000, 50000, 20000, 10000, 5000, 2000]}),
  bBox: {latMin: 46, latMax: 55, lonMin: 6, lonMax: 24}
};

//======================================================================================
//                                        Slovensko
//======================================================================================

SMap.CONFIG["ZBGIS_SK_ID"] = 
{
  zoom: [0, 9],
  label: "ZBGIS Slovensko",
  title: "Mapa Slovenska ZBGIS",
  preview: default_preview_path + "ZBGIS Slovensko.png",
  options: {tileSize: 256, ArcGIS: true},
  url: "https://zbgis.skgeodesy.sk/tkgis/Proxy.ashx?http://img-pp.zbgis.local/ArcGIS/rest/services/ZBGIS_tk_sjtsk/MapServer/tile/",
  copyright: {"-": ["Úrad geodézie, kartografie a katastra SR", "Geodetický a kartografický ústav Bratislava"]},
  projection: new SMap.Projection.Krovak({origin: {x: -600000, y: -1120000},
    resolution: 529.167725002117, scales: [2000000, 1000000, 500000, 250000, 100000, 50000, 25000, 10000, 5000, 2000]}),  
  bBox: {latMin: 47, latMax: 50, lonMin: 16, lonMax: 23}
};

SMap.CONFIG["Ortofoto_SK_ID"] = 
{
  zoom: [0, 9],
  label: "Ortofoto Slovensko",
  title: "Ortofoto Mapa Slovenska",
  preview: default_preview_path + "Ortofoto Slovensko.png",
  options: SMap.CONFIG["ZBGIS_SK_ID"].options,  
  url: "https://zbgis.skgeodesy.sk/tkgis/Proxy.ashx?http://img-pp.zbgis.local/ArcGIS/rest/services/ORTO_tk_sjtsk/MapServer/tile/",
  copyright: SMap.CONFIG["ZBGIS_SK_ID"].copyright,    
  projection: SMap.CONFIG["ZBGIS_SK_ID"].projection,  
  bBox: SMap.CONFIG["ZBGIS_SK_ID"].bBox
};

SMap.CONFIG["ZM10_SK_ID"] = 
{
  zoom: [2, 12],
  label: "ZM10 Slovensko",
  title: "Základná mapa 1:10 000",
  preview: default_preview_path + "ZM10 Slovensko.png",
  url: "https://zbgis.skgeodesy.sk/tkgis/Proxy.ashx?http://ws-p-pp.web.pex.local/ArcGIS/rest/services/tkgis/ZMSR_tk_sjtsk/MapServer/export",
  options: {tileSize: 512},
  WMS: {layers: "show:3,7,11,15,19,23", format: "png8", dpi: "96", transparent: "true", size: "512,512", bboxSR: "102067", imageSR: "102067", f: "image"},
  copyright: SMap.CONFIG["ZBGIS_SK_ID"].copyright,   
  projection: SMap.projKrovak,
  bBox: SMap.CONFIG["ZBGIS_SK_ID"].bBox
};

//======================================================================================
//                                       Polsko
//======================================================================================

// info: http://geoportal.gov.pl/uslugi/usluga-przegladania-wms
// další linky: http://forum.openstreetmap.org/viewtopic.php?id=17082&p=1
SMap.CONFIG["Poland_Orto_ID"] = 
{
  zoom: [5, 21],
  label: "Ortofoto Polsko",
  title: "Ortofoto Polsko",
  preview: default_preview_path + "Poland Orto.jpg",
  url: "http://mapy.geoportal.gov.pl/wss/service/img/guest/ORTO/MapServer/WMSServer",
  options: {tileSize: 512, wmsReverseCoords: true},
  WMS: {layers: "Raster", version: "1.1.1", format: "image/jpeg"},
  projection: SMap.projMercator,
  copyright: {"-": ["GeoPortal.pl"]},
  bBox: {latMin: 48, latMax: 56, lonMin: 13, lonMax: 25}
};

SMap.CONFIG["Poland_Topo_ID"] = 
{               
  zoom: [5, 17],
  label: "Topomapa Polsko",
  title: "Topomapa Polsko",
  preview: default_preview_path + "Poland Topo.png",
  url: "http://mapy.geoportal.gov.pl/wss/service/img/guest/TOPO/MapServer/WMSServer",
  options: SMap.CONFIG["Poland_Orto_ID"].options,
  WMS: SMap.CONFIG["Poland_Orto_ID"].WMS,
  projection: SMap.CONFIG["Poland_Orto_ID"].projection,
  copyright: SMap.CONFIG["Poland_Orto_ID"].copyright,
  bBox: SMap.CONFIG["Poland_Orto_ID"].bBox
};

//======================================================================================
//                            Open Street Maps based
//======================================================================================

SMap.CONFIG["OSM_ID"] = 
{
  zoom: [2, 17],
  label: "OSM",
  title: "Open Street Maps",
  preview: default_preview_path + "OSM.png",
  url: "http://tile.openstreetmap.org/",
  options: {query: "{zoom}/{x}/{y}.png"},
  copyright: {"-": "Open Street Maps"},
  projection: SMap.projMercator,
  bBox: SMap.CONFIG.bBox_Mercator
};

SMap.CONFIG["OSMCyclo_ID"] = 
{
  zoom: [2, 17],
  label: "OSM Cyclo",
  title: "OpenCycleMap",
  preview: default_preview_path + "OSM Cyclo.png",
  url: "http://tile.opencyclemap.org/cycle/",
  options: {query: "{zoom}/{x}/{y}.png"},
  copyright: {"-": "Open Street Maps"},
  projection: SMap.projMercator,
  bBox: SMap.CONFIG.bBox_Mercator
};

// http není příliš stabilní, https je lepší, ale může vyhazovat dialog :(
// je skoro stejná jako MTBMap
SMap.CONFIG["OpenTopoMap_ID"] = 
{
  zoom: [2, 17],
  label: "Open TopoMap",
  title: "Open TopoMap",
  preview: default_preview_path + "OpenTopoMap.png",
  url: "https://opentopomap.org/",
  options: {query: "{zoom}/{x}/{y}.png"},
  copyright: {"-": ["OpenTopoMap", "OpenStreetMap"]},
  projection: SMap.projMercator,
  bBox: SMap.CONFIG.bBox_Mercator
};

SMap.CONFIG["MTBMap_ID"] = 
{
  zoom: [2, 16],
  label: "MTB Map",
  title: "MTB Map",
  preview: default_preview_path + "MTB Map.png",
  url: "http://tile.mtbmap.cz/mtbmap_tiles/", // www.mtbmap.cz or tile.mtbmap.cz
  options: {query: "{zoom}/{x}/{y}.png"},
  copyright: {"-": ["<a href=http://MTBMap.cz>MTBMap.cz</a>", "Martin Tesař", "OpenStreetMap"]},
  projection: SMap.projMercator,
  bBox: {latMin: 35, latMax: 60, lonMin: -10, lonMax: 45}
};

SMap.CONFIG["FreemapSK_ID"] = 
{
  zoom: [5, 16],
  label: "Freemap SK",
  title: "Freemap.sk",
  preview: default_preview_path + "FreemapSK.png",
  
  // a. --> can be a/b/c/d
  // T - turistická, A - autoatlas, C - cyklo, K - běžkařská
  // url for K: http://t4.freemap.sk/K/
  
  url: "http://a.freemap.sk/data/layers/presets/T/",  
  options: {query: "{zoom}/{x}/{y}.jpeg"},
  copyright: {"-": "Freemap.sk"},
  projection: SMap.projMercator,
  bBox: {latMin: 47, latMax: 57.5, lonMin: 0, lonMax: 37}
};

SMap.CONFIG["LandShading_ID"] = 
{
  zoom: [2, 15],
  url: "http://tiles.openpistemap.org/landshaded/",
  options: {query: "{zoom}/{x}/{y}.png", opacity: 1},
  copyright: {"-": "OpenPisteMap"},
  projection: SMap.projMercator
};

SMap.CONFIG["Boundaries_ID"] =   /* hranice krajů, zemí atd., u nás stejné jako vrstva "prehledky" od ČÚZK */
{
  zoom: [6, 18],
  url: "http://129.206.74.245:8007/tms_b.ashx?",
  options: {query: "x={x}&y={y}&z={zoom}", fadeTime: 0, alpha: true},
  copyright: {"-": "OpenStreetMap"},
  projection: SMap.projMercator
};

//======================================================================================
//                              Google Maps
//======================================================================================

SMap.CONFIG["GSatellite_ID"] = 
{
  zoom: [2, 20],
  label: "Google Satellite",
  title: "Google Satellite Maps",
  preview: default_preview_path + "Google Satellite.jpg",
  // note: version (v=???) sometimes changes, so fix it from time to time
  url: "http://khm1.google.com/kh/v=174&",
  options: {query: "x={x}\&y={y}\&z={zoom}", sublayerOpacity: 0.3},
  copyright: {"-": "Google"},
  projection: SMap.projMercator,
  bBox: SMap.CONFIG.bBox_Mercator
};

SMap.CONFIG["GSatelliteSub_ID"] = createSubLayer("GSatellite_ID", {opacity: 0.3});

SMap.CONFIG["GMaps_ID"] = 
{
  zoom: [2, 17],
  label: "Google Maps",
  title: "Google Maps",
  preview: default_preview_path + "Google Maps.jpg",
  url: "http://mt2.google.com/vt/hl=cs&",
  options: {query: "x={x}&y={y}&z={zoom}"},
  copyright: {"-": "Google"},
  projection: SMap.projMercator,
  bBox: SMap.CONFIG.bBox_Mercator
};

SMap.CONFIG["GHybrid_ID"] = 
{
  // use m instead of h to display satellite maps
  url: "http://mt2.google.com/vt/lyrs=h&hl=cs&",
  options: {query: "x={x}&y={y}&z={zoom}", fadeTime: 0, alpha: true},
  copyright: {"-": "Google"},
  projection: SMap.projMercator
};

SMap.CONFIG["GTerrain_ID"] =   /* musí být pod GTerrainSubLayer_ID, protože se na něj odkazuje */
{
  zoom: [2, 15],
  label: "Google Terrain",
  title: "Google Terrain",
  preview: default_preview_path + "Google Terrain.png",
  url: "http://mt2.google.com/vt/lyrs=t,r&hl=cs&",
  options: {query: "x={x}&y={y}&z={zoom}"},
  copyright: {"-": "Google"},
  projection: SMap.projMercator,
  bBox: SMap.CONFIG.bBox_Mercator
};

SMap.CONFIG["GStreetView_ID"] = 
{
  // lmc:photos_outside_streetview_134,svv --> street view + images
  // lmc:panoramio,svv --> street view + panoramio images
  // lmc:,svv --> street view only
  zoom: [2, 20],
  url: "http://mt0.google.com/mapslt?lyrs=lmc:,svv&w=256&h=256&hl=cs&style=40,18&",
  options: {query: "x={x}&y={y}&z={zoom}", fadeTime: 0, alpha: true},
  copyright: {"-": "Google"},
  projection: SMap.projMercator
};

//======================================================================================
//                                 other maps
//======================================================================================

// http://mc.bbbike.org/mc/?lon=15.418542&lat=50.398&zoom=9&num=1&mt0=mapnik&mt1=mapbox-terrain
// https://www.mapbox.com/signup/?plan=starter
SMap.CONFIG["MapBoxTerrain_ID"] = 
{
  zoom: [2, 17],
  label: "MapBox Terrain",
  title: "MapBox Terrain",
  preview: default_preview_path + "MapBox Terrain.png",
  url: "http://b.tiles.mapbox.com/v4/matt.72ef5189/",
  buildUrl: function(t /*tile*/)
  {
    return this._url + (t.zoom-1) + "/" + t.x + "/" + t.y + "@2x.png?access_token=pk.eyJ1IjoiZHVuY2FuZ3JhaGFtIiwiYSI6IlJJcWdFczQifQ.9HUpTV1es8IjaGAf_s64VQ";
  },
  options: {tileSize: 512},    
  copyright: {"-": "MapBox"},
  projection: SMap.projMercator,
  bBox: {latMin: -85, latMax: 85, lonMin: -180, lonMax: 180}
};

SMap.CONFIG["GeocachesMap_ID"] = 
{
  zoom: [2, 20],
  url: "http://tiles01.geocaching.com/map/map.png?",
  options: {query: "x={x}&y={y}&z={zoom}&ep=1", fadeTime: 0, alpha: true, reprojectionZoomLower: true},
  copyright: {"-": "Groundspeak.com"},
  projection: SMap.projMercator
};

//======================================================================================
//                             Mapy.cz maps + fixes
//======================================================================================

SMap.CONFIG["MapyCZ_OPHOTO_Sub"] = createSubLayer(SMap.DEF_OPHOTO, {opacity: 0.3});

// fill in missing zoom range
[SMap.DEF_TRAIL, SMap.DEF_BIKE, SMap.DEF_HYBRID].forEach(function(id)
{
  SMap.CONFIG[id].zoom = [5, 20];
});

// fill in missing projections
for (layer in SMap.CONFIG)
{
  SMap.CONFIG[layer].projection = SMap.CONFIG[layer].projection || SMap.projMercator;
}

// fill in missing bBox
[SMap.DEF_BASE, SMap.DEF_TURIST, SMap.DEF_OPHOTO, SMap.DEF_HISTORIC, SMap.DEF_OPHOTO0203, 
SMap.DEF_OPHOTO0406, SMap.DEF_SMART_BASE, SMap.DEF_SMART_OPHOTO, SMap.DEF_SMART_TURIST, 
SMap.DEF_TURIST_WINTER, SMap.DEF_SMART_WINTER, SMap.DEF_SUMMER, SMap.DEF_SMART_SUMMER,
SMap.DEF_GEOGRAPHY].forEach(function(mapId)
{
  SMap.CONFIG[mapId].bBox = SMap.CONFIG.bBox_Mercator;//SMap.CONFIG.bBox_MapyCZ;  
});

// funkce addDefaultLayer se při přidání SMap.DEF_PANO dožaduje addDefaultLayer, což nemusí být dostupné
// -- proto vrstvu zkopírujeme pod jiné ID
// -- také je třeba nastavit průhlednost, jinak se při změně zoomu staré dílky nemažou 
SMap.CONFIG[SMap.DEF_PANO].options.alpha = true;
SMap.CONFIG["pano_ID"] = SMap.CONFIG[SMap.DEF_PANO];

// oprava preview letní mapy
SMap.CONFIG[SMap.DEF_SUMMER].preview = default_preview_path + "MapyCZ Letni.png";

// reliéf použijeme starou vrstvu, dokud ta původní na Mapy.cz opět nefunguje
SMap.CONFIG[SMap.DEF_RELIEF].zoom = [3, 15];
SMap.CONFIG[SMap.DEF_RELIEF].options.query = "{zoom}_{ppx}_{ppy}";
SMap.CONFIG[SMap.DEF_RELIEF].url = SMap.CONFIG.protocol + "//m#.mapserver.mapy.cz/relief/";
SMap.CONFIG[SMap.DEF_RELIEF].projection = SMap.projUTM33;

window.otherMapsJS_Loaded = true;