translations["_total_length"] = {"English": "Len", "Czech": "Délka"};
translations["_polygon_area"] = {"English": "Area", "Czech": "Plocha"};
translations["_total_points"] = {"English": "Points", "Czech": "Bodů"};
translations["_poly_load_file"] = {"English": "File", "Czech": "Soubor"};
translations["_poly_import_from_text"] = {"English": "Import", "Czech": "Import"};
translations["_poly_help"] = {"English": "Help", "Czech": "Nápověda"};
translations["_import_text_button"] = {"English": "Import track/polygon", "Czech": "Importuj trasu/polygon"};
translations["_elevation"] =  {"English": "Elevation", "Czech": "Nadmořská výška"};
translations["Index"] =  {"Czech": "Index"};

translations["_delete_track"] =  
{
  "English": "Do you really want to delete the track? This action can't be reversed!",
  "Czech": "Chcete track opravdu smazat? Tato akce nelze vrátit zpět!"
};

translations["_delete_all_tracks"] =  
{
  "English": "Do you really want to delete all tracks? This action can't be reversed!",
  "Czech": "Chcete opravdu smazat všechny tracky? Tato akce nelze vrátit zpět!"
};

translations["_merge_all_tracks"] =  
{
  "English": "Do you really want to merge all tracks into one? This action can't be reversed!",
  "Czech": "Chcete opravdu sloučit všechny tracky do jednoho? Tato akce nelze vrátit zpět!"
};
              
/**********************************************************/
/*                   Polygon - main class                 */
/**********************************************************/
   
function Polygon(map, button, settings)
{
  this.polylines = [];
  this.activeId = -1;
  this.polygonModeOn = false;
  this.polylineCounter = 0; /* for name in select box */
  
  this.map = map;
  this.button = button;
  this.settings = settings;

  var that = this;
  
  // handle clicking on "Polygon" button
  EventUtil.addHandler(button, "click", function()
  {
    that.polygonModeOn ? that.exit() : that.init();
  });
  
  // create sidebar
  this.textArea = new TextArea(this);   

  // create default empty polylines
  for (var i = 0; i < 5; i++)
    this.addPolyline({name: "Poly" + ++this.polylineCounter, Formats: new Formats("")});

  this.activatePolyline(0);
  
}

Polygon.prototype.init = function()
{
  this.polygonModeOn = true;
  this.button.style.color = "red";
  var that = this;
  
  // nothing activated? then activate first polyline (if there are any)
  if (this.activeId == -1 && this.polylines.length)
    this.activeId = 0;  
  
  if (this.activeId != -1)
    this.activatePolyline(this.activeId);
  
//   // run help box on first run
//   if (!this.settings.polygon_first_run)
//   {
//     this.settings.polygon_first_run = true;
//     ui("copybox", {text: tr("_polygon_info"), width: 600, height: 200});
//   }

  // show text area
  this.textArea.show();

  /**** left click on polyline - change active polyline ****/
  this.onPolylineClick = this.map.getSignals().addListener(window, "geometry-click", function(e)
  {
    // CTRL + left click - change active track
    // alternative for Opera: ALT + right click
    if (!e.data.event.ctrlKey && (e.data.event.button != JAK.Browser.mouse.right || !e.data.event.altKey))
      return;

    // get segment + polyline
    var segmentId = e.target.segmentId;
    var polylineId;

    for (var i = 0; i < that.polylines.length; i++)
    {
      if (that.polylines[i] === e.target.that)
        polylineId = i;
    }
    
    // if we clicked on helper-line (activated by M), we get undefined here
    // -- so we don't do anything (M needs to be deactivated first, otherwise it's impossible to click the points)
    if (typeof polylineId === 'undefined')
      return;

    // activate polyline
    that.activatePolyline(polylineId);
    
    // send for further processing
    that.polylines[polylineId].onGeometryClick(e.data.event, segmentId);

    // cleanup
    EventUtil.preventDefault(e.data.event);
    EventUtil.stopPropagation(e.data.event);
    
    // old IE would select all text on the page --> prevent that
    if (JAK.Browser.client == "ie" && JAK.Browser.version <= 8)
      EventUtil.deselect();
  });

  /**** left click on map - add point / move-rotate selection ****/
  this.onMapClick = this.map.getSignals().addListener(window, "map-click", function(e)
  {
    if (window.contextMenuActive)
      return;
  
    e = e.data.event;

    // proceed only on left mouse button click (Chrome sends this event also on middle button click)
    // IE8 sends wrong JAK.Browser.mouse
    if (e.button != JAK.Browser.mouse.left && (JAK.Browser.client != "ie" || JAK.Browser.version > 8))
      return;

    // prevent CTRL - conflicts with track selection (CTRL + click on track/polygon)
    if (e.ctrlKey)
      return;

    // don't react when contextmenu is open
    if (that.activeId != -1 && !window.contextMenuActive)
      that.polylines[that.activeId].onMapClick(e);
  });

  /**** middle click on the map  ****/
  this.onMapMiddleClick = EventUtil.addHandler(this.map.getContainer().parentNode, "mouseup", function(e, target)
  {
    if (e.button != JAK.Browser.mouse.middle || that.activeId == -1 || window.contextMenuActive || JAK.Browser.client == "safari")
      return;

    that.polylines[that.activeId].onMapMiddleClick(e);
  });

  /**** map redraw - redraw active polyline ****/
  // makes minimap in Opera flicker on rectangle move
  this.onMapRedraw = this.map.getSignals().addListener(window, "map-redraw", function(e, elm)
  {
    if (that.activeId != -1)
      that.polylines[that.activeId].onMapRedraw(e);
  });

  /**** map lock/unlock - prevent redraw in this mode ****/
  this.onMapLock = m.getSignals().addListener(window, "map-lock", function(e)
  {
    if (that.activeId != -1)
      that.polylines[that.activeId].mapIsLocked = true;
  });
  
  this.onMapUnLock = m.getSignals().addListener(window, "map-unlock", function(e)
  {
    if (that.activeId != -1)
      that.polylines[that.activeId].mapIsLocked = false;
  })    
  
  /**** marker left click (conflicts with dblclick) ****/
  this.onMarkerLeftClick = this.map.getSignals().addListener(window, "marker-click", function(e)
  {
    if (e.data.event.button != JAK.Browser.mouse.left && (JAK.Browser.client != "ie" || JAK.Browser.version > 8))
      return;  

    if (that.activeId != -1)
      that.polylines[that.activeId].onMarkerLeftClick(e);
  });
  
  /**** marker double click (conflicts with click)  ****/
  this.onMarkerLeftDblClick = this.map.getSignals().addListener(window, "marker-dblclick", function(e)
  {
    if (e.data.event.button != JAK.Browser.mouse.left && (JAK.Browser.client != "ie" || JAK.Browser.version > 8))
      return;  
    
    if (that.activeId != -1)
      that.polylines[that.activeId].onMarkerLeftDblClick(e);
  });
  
  /**** marker drag move - create auxiliary lines to position ****/
  this.onMarkerDragMove = this.map.getSignals().addListener(window, "marker-drag-move", function(e)
  {
    if (that.activeId != -1)
      that.polylines[that.activeId].onMarkerDragMove(e);  
  });      
  
  /**** marker dragged - move it ****/
  this.onMarkerDragStop = this.map.getSignals().addListener(window, "marker-drag-stop", function(e)
  {
    if (that.activeId != -1)
      that.polylines[that.activeId].onMarkerDragStop(e);  
  });
  
  /**** handle keys (moving selection, history) ****/
  this.onKeyDown = EventUtil.addHandler(document.documentElement, "keydown", function (e, target)
  {
    if (contains_class(document.body, "lightbox_on"))
      return;

    var key = String.fromCharCode(e.keyCode).toUpperCase();

    // handle TAB (switch between tracks)
    if (e.keyCode == 9 && !e.shiftKey && !e.ctrlKey && !e.altKey)
    {
      that.lastTabIndex = that.lastTabIndex || that.activeId;
      var startIndex = that.lastTabIndex;
      
      do
      {
        that.lastTabIndex = (that.lastTabIndex + 1 >= that.polylines.length) ? 0 : ++that.lastTabIndex;
        
        // found next one
        if (that.polylines[that.lastTabIndex].points.length)
        {
          that.activatePolyline(that.lastTabIndex);
          that.map.setCenterZoom.apply(that.map, that.map.computeCenterZoom(that.polylines[that.lastTabIndex].points));
          break;
        }

      } while (that.lastTabIndex != startIndex);

      EventUtil.preventDefault(e);
      EventUtil.stopPropagation(e);
    }
    
    // SHIFT + delete --> delete active track
    else if (e.keyCode == 46 && e.shiftKey && !e.ctrlKey && !e.altKey && that.activeId != -1)
    {
      if (!(that.polylines[that.activeId].history.length > 1 /* some history */
      || that.polylines[that.activeId].points.length)        /* or some points present */
      || confirm(tr("_delete_track")))                       /* if so, then ask */
        that.deletePolyline(that.activeId);
    }
    
    // SHIFT + CTRL + ALT + delete --> delete all tracks
    else if (e.keyCode == 46 && e.shiftKey && e.ctrlKey && e.altKey)
    {
      if (confirm(tr("_delete_all_tracks")))
      {
        for (var i = that.polylines.length - 1; i >= 0; i--)
          that.deletePolyline(i);
      }    
    }
    
    // SHIFT + CTRL + ALT + J --> join all tracks (and delete old ones as well)
    else if (key == "J" && e.shiftKey && e.ctrlKey && e.altKey && that.polylines.length >= 1)
    {
      if (confirm(tr("_merge_all_tracks")))
      {
        // create new layer
        that.addPolyline({name: "Joined", Formats: new Formats("")});
        that.activatePolyline(that.polylines.length - 1);
        var joined = that.polylines[that.polylines.length - 1];        

        while(that.polylines.length > 1)
        {
          var poly = that.polylines[0];
          
          // if a layer is a polygon, set the mode of the joined layer to it
          if (poly._isPolygon())
            joined._setPolygon(true);
          
          // set first point as a segment start
          if (poly.points.length && joined.points.length)
            poly.points[0].custom.segmentStart = true;
          
          // concatenate points  
          joined.points = joined.points.concat(poly.points);
          
          // delete old layer
          that.deletePolyline(0);
        }
      }    
    }    
    
    // insert --> create a new track and switch to it
    else if (e.keyCode == 45 /*&& e.shiftKey*/ && !e.ctrlKey && !e.altKey)
    {
      that.addPolyline({name: "MyPoly" + ++that.polylineCounter, Formats: new Formats("")}); 
    }        
  
    else if (that.activeId != -1)
      that.polylines[that.activeId].onKeyDown(e, target);    
  });
  
  this.onKeyUp = EventUtil.addHandler(document.documentElement, "keyup", function (e, target)
  {
    if (that.activeId != -1 && !contains_class(document.body, "lightbox_on"))
      that.polylines[that.activeId].onKeyUp(e, target);    
  });

  this.onKeyPress = EventUtil.addHandler(document.documentElement, "keypress", function (e, target)
  {
    if (that.activeId != -1 && !contains_class(document.body, "lightbox_on"))
      that.polylines[that.activeId].onKeyPress(e, target);    
  });
  
  this.onBlur = EventUtil.addHandler(window, "blur", function (e, target)
  {
    if (that.activeId != -1)
      that.polylines[that.activeId].onBlur(e, target);    
  });
      
  this.onFocus = EventUtil.addHandler(window, "focus", function (e, target)
  {
    if (that.activeId != -1)
      that.polylines[that.activeId].onFocus(e, target);    
  });
  
  /**** track mouse move ****/
  this.onMouseMoveHandler = JAK.Events.addListener(this.map.getContent(), "mousemove", this, function(e, target)
  {
    if (that.activeId != -1)
      that.polylines[that.activeId].onMouseMove(e, target);  
  })
  
  /**** select points ****/
  var selOptionsDefault = {customHandler: encapsulateCall("onPolygonSelectionStop"), customFailDrag: encapsulateCall("onPolygonSelectionStopFail"),
    customRedraw: encapsulateCall("onPolygonSelection"), customStartDrag: encapsulateCall("onPolygonSelectionStart"), thickness: 4};

  this.selectionControls = [];
  var selControls = [];
  var selRectangleMouse = JAK.Browser.mouse.left;
  var selPolygonMouse = JAK.Browser.mouse.right;

  selControls.push(
  {
    mouseButton: selRectangleMouse,
    shift: true,
    alt: true,
    cursor: "cell",
    customId: "polygon_Rectangle_AddToSelection"
  });
  
  selControls.push(
  {
    mouseButton: selRectangleMouse,
    shift: true,
    cursor: "cell",
    customId: "polygon_Rectangle_Select"
  });

  selControls.push(
  {
    mouseButton: selRectangleMouse,
    alt: true,
    customId: "polygon_Rectangle_DeSelect"
  });
    
  selControls.push(
  {
    mouseButton: selRectangleMouse,
    shift: true,
    alt: true,
    ctrl: true,
    customId: "polygon_Rectangle_SelectXOR"
  });
  
  selControls.push(
  {
    mouseButton: selPolygonMouse,
    shift: true,
    alt: true,
    cursor: "cell",
    customId: "polygon_Polygon_AddToSelection"
  });
  
  selControls.push(
  {
    mouseButton: selPolygonMouse,
    shift: true,
    cursor: "cell",
    customId: "polygon_Polygon_Select"
  });
  
  selControls.push(
  {
    mouseButton: selPolygonMouse,
    alt: true,
    customId: "polygon_Polygon_DeSelect"
  });
    
  selControls.push(
  {
    mouseButton: selPolygonMouse,
    shift: true,
    alt: true,
    ctrl: true,
    customId: "polygon_Polygon_SelectXOR"
  });
     
  for (var i = 0; i < selControls.length; i++)
  {
    SMap.Util.mergeObject(selOptionsDefault, selControls[i]);
    this.selectionControls.push(new SMap.Control.Selection(selControls[i]));
    this.map.addControl(this.selectionControls[this.selectionControls.length - 1]);  
  }
     
  /**** move points (use selection for this) ****/
  function encapsulateCall(name)
  {
    return function()
    {
      if (that.activeId != -1)
        that.polylines[that.activeId][name](this);
    }    
  }
  
  this.selectionControls.push(new SMap.Control.Selection(
  {
    ctrl: true,
    mouseButton: JAK.Browser.mouse.left,
    customHandler: encapsulateCall("onCtrlDragStop"),
    customRedraw: encapsulateCall("onCtrlDrag"),
    customStartDrag: encapsulateCall("onCtrlDragStart")
  }));    

  this.map.addControl(this.selectionControls[this.selectionControls.length - 1]);
  
  /**** rotate points (use selection for this) ****/
  this.selectionControls.push(new SMap.Control.Selection(
  {
    ctrl: true, 
    shift: true, 
    mouseButton: JAK.Browser.mouse.left,
    customHandler: encapsulateCall("onCtrlShiftDragStop"), 
    customRedraw: encapsulateCall("onCtrlShiftDrag"), 
    customStartDrag: encapsulateCall("onCtrlShiftDragStart")
  }));    

  this.map.addControl(this.selectionControls[this.selectionControls.length - 1]);
  
  /**** scale points (use selection for this) ****/
  this.selectionControls.push(new SMap.Control.Selection(
  {
    ctrl: true,
    alt: true,
    mouseButton: JAK.Browser.mouse.left,
    customHandler: encapsulateCall("onCtrlAltDragStop"),
    customRedraw: encapsulateCall("onCtrlAltDrag"),
    customStartDrag: encapsulateCall("onCtrlAltDragStart")
  }));    

  this.map.addControl(this.selectionControls[this.selectionControls.length - 1]);
  
  /**** draw points just by dragging middle mouse button (allow version adding points before - ALT) ****/
  // turn it off for IE8 and older (causes problems)
  // disable for Safari based browsers - doesn't distinguish left and middle clicks
  // require CTRL pressed for Opera
  if ((JAK.Browser.client != "ie" || JAK.Browser.version > 8) && JAK.Browser.client != "safari")
  {
    for (var i = 0; i < 4; i++)
    {
      this.selectionControls.push(new SMap.Control.Selection(
      {
        alt: !!(i % 2),
        shift: (i < 2),
        ctrl: (JAK.Browser.client == "opera"),
        mouseButton: JAK.Browser.mouse.middle,
        customId: ("onDragDraw_" + ((i % 2) ? "Alt" : "NoAlt") + ((i < 2) ? "_Shift" : "_NoShift")),
        customHandler: encapsulateCall("onDragDrawStop"),
        customRedraw: encapsulateCall("onDragDraw"),
        customStartDrag: encapsulateCall("onDragDrawStart")
      }));
    
      this.map.addControl(this.selectionControls[this.selectionControls.length - 1]);
    }
  }

  // broadcast signal
  this.map.getSignals().makeEvent("polygon-mode-init", this);
}

Polygon.prototype.exit = function()
{
  this.polygonModeOn = false;
  this.button.style.color = "";

  // remove listeners/controls
  this.map.getSignals().removeListener(this.onPolylineClick);
  this.map.getSignals().removeListener(this.onMapClick);
  this.map.getSignals().removeListener(this.onMapRedraw);
  this.map.getSignals().removeListener(this.onMapLock);
  this.map.getSignals().removeListener(this.onMapUnLock);
  this.map.getSignals().removeListener(this.onMarkerLeftClick);
  this.map.getSignals().removeListener(this.onMarkerLeftDblClick);
  this.map.getSignals().removeListener(this.onMarkerDragMove);
  this.map.getSignals().removeListener(this.onMarkerDragStop);
  
  JAK.Events.removeListener(this.onMouseMoveHandler);

  EventUtil.removeHandler(this.onMapMiddleClick);
  EventUtil.removeHandler(this.onKeyDown);
  EventUtil.removeHandler(this.onKeyUp);
  EventUtil.removeHandler(this.onKeyPress);
  EventUtil.removeHandler(this.onBlur);
  EventUtil.removeHandler(this.onFocus);
  
  while (this.selectionControls.length)
  {
    var control = this.selectionControls.pop();
    
    // destructor doesn't get called for some reason, so call it manually
    control.$destructor();
    this.map.removeControl(control);
  }
  
  // deactivate polyline  
  this.deactivatePolyline(this.activeId);

  // hide sidebar
  this.textArea.hide();

  // broadcast signal
  this.map.getSignals().makeEvent("polygon-mode-exit", this);    
}

Polygon.prototype.deactivatePolyline = function(id)
{
  this.polylines[id].deactivate();
  
  this.textArea.updateSelectPolygonBox(this.polylines, -1);
}

Polygon.prototype.activatePolyline = function(id)
{
  // turn off old polyline
  if (this.activeId != -1 && this.polylines.length)
    this.deactivatePolyline(this.activeId);

  this.activeId = id;
  this.polylines[this.activeId].activate();
  
  this.textArea.updateSelectPolygonBox(this.polylines, this.activeId);
}

Polygon.prototype.deletePolyline = function(id)
{
  // deactivate and remove layer
  this.deactivatePolyline(id);
  
  this.polylines[id].destroy();
  this.polylines.remove(id);
  
  if (this.activeId >= this.polylines.length)
    this.activeId = 0;
  
  // too few tracks left? create a new empty one  
  if (this.polylines.length <= 0)
    this.addPolyline({name: "Poly" + ++this.polylineCounter, Formats: new Formats("")});  

  this.activatePolyline(this.activeId);
  
  if (this.polylines[this.activeId].points.length)
    this.map.setCenterZoom.apply(this.map, this.map.computeCenterZoom(this.polylines[this.activeId].points));
  
  this.textArea.updateSelectPolygonBox(this.polylines, this.activeId);
}

Polygon.prototype.addPolyline = function(options)
{
  // turn off old polyline
  if (this.polylines.length && this.activeId != -1)
    this.deactivatePolyline(this.activeId);

  var polyline = new Polyline(this, options || {});
  this.polylines.push(polyline);

  polyline.init();
  this.activatePolyline(this.polylines.length - 1);
  
  this.textArea.updateSelectPolygonBox(this.polylines, this.activeId);
}

/**********************************************************/
/*                       TextArea                         */
/**********************************************************/
                                       
function TextArea($super)
{
  this.$super = $super;
  this.map = this.$super.map;
  this.mapadiv = this.map.getContainer();

  this.format = window.testing ? "GSAK" : "none";
  var that = this;

  // ** main div **
  this.mainDiv = document.createElement("div");
  this.mainDiv.id = "PolygonDiv";
  this.mainDiv.innerHTML = "" 
    + "<div id='line_total_length'></div>" 
    + "<div id=polygon_options>" 
      + "<span>" + tr("Name") + "</span>"
      + "<select id=polygon_current_name></select>"
      + "<span>" + tr("Export") + "</span>"
      + "<select id=polygon_export_format>"
        + "<option value=none>------</option>"
        + "<option value=GSAK" + (window.testing ? " selected" : "") + ">GSAK</option>"
        + "<option value=GPX_Track>GPX (Track)</option>"
        + "<option value=GPX_Route>GPX (Route)</option>"
        + "<option value=GPX_Waypoints>GPX (Waypoints)</option>"
        + "<option value=PLT>PLT</option>"
        //+ "<option value=RTE>RTE</option>" 
        + "<option value=WPT>WPT</option>"
       // + "<option value=KML>KML</option>"
      + "</select>"
    + "</div>"
    + "<div id=polygon_buttons>"
      + "<span id=poly_file_import class=button>" + tr("_poly_load_file") + "</span>"
      + "<span id=poly_text_import class=button>" + tr("_poly_import_from_text") + "</span>"
      + "<span id=poly_help class=button>" + tr("_poly_help") + "</span>"
    +  "</div>"
    + "<div id=textarea_parent><textarea id='polygon_textarea'></textarea></div>" 
    + "<div id=copy_button class=button>" + tr((JAK.Browser.client == "ie") ? "Copy" : "Select") + "</div>";

  this.mapadiv.parentNode.insertBefore(this.mainDiv, this.mapadiv.nextSibling);
  this.textArea = document.getElementById("polygon_textarea");
  this.lineLengthDiv = document.getElementById("line_total_length");
  this.optionsDiv = document.getElementById("polygon_options");
  this.selectPolygonBox = document.getElementById("polygon_current_name");
  this.selectFormatBox = document.getElementById("polygon_export_format");
  
  this.importFileButton = document.getElementById("poly_file_import");
  this.importTextButton = document.getElementById("poly_text_import");
  this.helpButton = document.getElementById("poly_help");

  // set textarea height
  this.mapadiv.clientHeight; /* IE7-8: may fix possible invalid values on first access */
  this.textArea.parentNode.style.height = (this.mapadiv.clientHeight - (20 + 20 + 18)) + "px"; 

  // prevent any keystrokes within the text area to bubble
  function clickOnTextArea(e)
  {
    EventUtil.stopPropagation(e);  
  }

  EventUtil.addHandler(this.textArea, "keydown", clickOnTextArea);
  EventUtil.addHandler(this.textArea, "keyup", clickOnTextArea);
  EventUtil.addHandler(this.textArea, "keypress", clickOnTextArea);
   
  // ** copy to clipboard button **
  this.copyButton = document.getElementById("copy_button");
  
  EventUtil.addHandler(this.copyButton, "click", function(e)
  {
    that.textArea.select();
    
    if (JAK.Browser.client == "ie")
      window.clipboardData.setData("Text", that.textArea.value)
  });
  
  // change polyline on different option select
  EventUtil.addHandler(this.selectPolygonBox, "change", function(e, target)
  {
    that.$super.activatePolyline(target.selectedIndex);
        
    // center map on track (if there are any points)
    var points = that.$super.polylines[target.selectedIndex].points;
    
    if (points.length)
      that.map.setCenterZoom.apply(that.map, that.map.computeCenterZoom(points));
  });
  
  // get format
  EventUtil.addHandler(this.selectFormatBox, "change", function(e, target)
  {
    that.format = target.value;
    
    // refresh text area
    if (that.$super.activeId != -1)
      that.writePolyline(that.$super.polylines[that.$super.activeId]);
  });
  
  // handle buttons
  EventUtil.addHandler(this.importFileButton, "click", function(e, target)
  {
    loadFile();
  });

  EventUtil.addHandler(this.importTextButton, "click", function(e, target)
  {
    ui("copybox", {text: "", width: 600, height: 200, select: true, closeButtonText: tr("_import_text_button"),
      getText: function(text)
      {
        if (text.length)
        {
          var file = new Formats(text);
          
          if (file.type)
          {
            for (var i = 0; i < file.Formats.length; i++)
              file.Formats[i].sendToPolygon();          
          }
        }
      }
    });
  });
  
  EventUtil.addHandler(this.helpButton, "click", function(e, target)
  {
    require(basePath + "polygon_help.js", function() { ui("copybox", {text: tr("_polygon_info"), width: 600, height: 200}) });
  });
  
  // set textare height on window resize
  EventUtil.addHandler(window, "resize", function(e, target)
  {
    that.mapadiv.clientHeight; /* IE7-8: may fix possible invalid values on first access */
    that.textArea.parentNode.style.height = (that.mapadiv.clientHeight - (20 + 20 + 18)) + "px"; 
  });    
};

/* this resizes and moves text area in case we full-screen map within listing */
TextArea.prototype.syncViewPort = function()
{
  this.mapadiv.parentNode.insertBefore(this.mainDiv, this.mapadiv.nextSibling);
  
  // set textarea height
  this.mapadiv.clientHeight; /* IE7-8: may fix possible invalid values on first access */
  this.textArea.parentNode.style.height = (this.mapadiv.clientHeight - (20 + 20 + 18)) + "px";
}

TextArea.prototype.show = function()
{
  add_class(document.documentElement, "polygon_on");
  this.map.syncPort();
  window.createDefaultMapControls();
}

TextArea.prototype.hide = function()
{
  remove_class(document.documentElement, "polygon_on");
  this.map.syncPort();
  window.createDefaultMapControls();
}

TextArea.prototype.writePolyline = function(polyline)
{
  // write points
  var text = "";
  var segments = polyline.getSegments();
 
  // process format (PLT, GPX, etc.)
  var options = {isPolygon: polyline._isPolygon()};
  
  if (this.format != "none")
    text += polyline.Formats.exportFormat(this.format, segments, options);
  
  this.textAreaSet(text);
                             
  // write line length + number of points / segments
  var distance = polyline.getLength(segments);

  if (distance < 1000)
    var distString = distance.toFixed(2) + " m";
  else
    var distString = (distance / 1000).toFixed(3) + " km";

  // write area (if polygon)
  if (polyline._isPolygon())
  {
    // get area in ha
    var area = polyline.getArea() / (100 * 100);
    var unit = "ha";

    // convert to km2    
    if (area > 1000)
    {
      area /= 10 * 10;
      unit = "km2";
    }
    
    // convert to m2
    else if (area < 1)
    {
      area *= (100 * 100);
      unit = "m2";
    }
    
    area = area.toFixed(2);
  }
  
  this.lineLengthDiv.innerHTML = tr("_total_length") + ": " + distString
    + (polyline._isPolygon() ? (" (" + /*tr("_polygon_area") + ": "*/ + area + " " + unit + ")") : "") 
    + ", " + tr("_total_points") + ": " + polyline.points.length + (segments.length >= 2 ? ("/" + segments.length) : "");  

  // display it as title in case it overflows
  this.lineLengthDiv.title = this.lineLengthDiv.innerHTML;
}

TextArea.prototype.textAreaSet = function(text)
{
  this.textArea.value = text; 
}

TextArea.prototype.textAreaClear = function()
{
  this.textArea.value = ""; 
}

TextArea.prototype.updateSelectPolygonBox = function(polylines, id)
{
  // info: http://www.javascriptkit.com/jsref/select.shtml

  // remove all options
  while (this.selectPolygonBox.childNodes.length)
    this.selectPolygonBox.removeChild(this.selectPolygonBox.childNodes[0]);
  
  // re-generate options
  for (var i = 0; i < polylines.length; i++)
  {
    // modern browsers (last parameter null)
    try
      {this.selectPolygonBox.add(new Option(polylines[i].name), null);}
    
    // IE7-8beta2 (last parameter missing)
    catch(ex)
      {this.selectPolygonBox.add(new Option(polylines[i].name));}
  }
  
  // select current polyline option
  if (id != -1)
    this.selectPolygonBox.options[id].selected = true;
  
  // select box doesn't update until DOM change (IE, any version), so force it
  if (JAK.Browser.client == "ie")
  {
    var tempElm = document.createElement("span");
    document.documentElement.appendChild(tempElm);
    window.setTimeout(function(){document.documentElement.removeChild(tempElm);}, 200);
  }
}

/**********************************************************/
/* Polyline - contains one line (or polygon - closed line)*/
/**********************************************************/

function Polyline($super, options)
{
  this.$super = $super;
  this.map = this.$super.map;
  this.points = options.points || [];
  this.name = options.name || "polygon";  /* name in select box */
  this.closed = options.closed || false;   /* allow adding points to the end */
  this.active = false;
  this.Formats = options.Formats || null; /* class that can read/export formats */
  this._setPolygon(options.isPolygon);
  
  for (var i = 0; i < this.points.length; i++)
    this.points[i].custom = this.points[i].custom || {};
  
  // draw line to where the mouse is (if we start from scratch only)  
  this.lineToMousePos = !this.points.length;
  
  // hide points when too close
  this.hidePointsWhenTooClose = true;  
};

Polyline.prototype.init = function()
{
  /* polygon/polyline layer */
  this.layer = new SMap.Layer.Geometry("polylineLayer");
  this.map.addLayer(this.layer);
  this.layer.enable();
  
  /* geometry layer for extra elements (like arrows) */
  this.helperLayer = new SMap.Layer.Geometry("polylineHelperLayer");
  this.map.addLayer(this.helperLayer);
  this.helperLayer.enable();

  /* points (markers) */
  this.markerLayer = new SMap.Layer.Marker();
  this.map.addLayer(this.markerLayer).enable();
  this.markerLayer.enable();

  this.history = [];
  this._saveHistory();
  
  var that = this;
  this.onMarkerRightClick = function(e, target) {that._onMarkerRightClick.call(that, e, target);}
  
  // center map (if there are any points)
  if (this.points.length)
    this.map.setCenterZoom.apply(this.map, this.map.computeCenterZoom(this.points));   
};

Polyline.prototype.exit = function()
{
  this.history = [];
  this.$super.textArea.textAreaClear();

  this.markerLayer.removeAll();
  this.map.removeLayer(this.markerLayer);
  this.map.removeLayer(this.layer);
  this.map.removeLayer(this.helperLayer);
  delete this.markerLayer;
  delete this.layer;
  delete this.helperLayer;
}

/* drag + CTRL - move selected points */
Polyline.prototype.onCtrlDragStart = function(e)
{
  this._saveHistory();
  this.lastSelectedDelta = null;  
}

Polyline.prototype.onCtrlDrag = function(e)
{
  var delta = e._point2.clone().minus(e._point1);
  
  var selected = this.getSelected();
  
  if(this.lastSelectedDelta && selected.length)
  {//log([new Date().getTime(), e.type]);
    this.movePoints(selected, delta.clone().minus(this.lastSelectedDelta));
    this.drawAll(this.points.length > 300);    
  }
  
  this.lastSelectedDelta = delta;
}

Polyline.prototype.onCtrlDragStop = function(e)
{
  // redraw to update text area
  this.drawAll();
}

/* drag + CTRL + SHIFT - rotate selected points */
Polyline.prototype.onCtrlShiftDragStart = function(e)
{
  this._saveHistory();
  this.selectedDragOrigin = null;
  this.lastAzimuth = 0;
}

Polyline.prototype.onCtrlShiftDrag = function(e)
{
  var selected = this.getSelected();
  
  if(this.selectedDragOrigin && selected.length)
  {
    var azimuth = this.selectedDragOrigin.azimuth(e._point2.toCoords(this.map));
    this.rotatePoints(selected, azimuth - (this.lastAzimuth || azimuth), this.selectedDragOrigin);
    this.lastAzimuth = azimuth;
    this.drawAll(this.points.length > 300);    
    
    // create marker at origin
    var markerDiv = JAK.mel("div", {className: "rotateOriginMarker"});
    var marker = new SMap.Marker(this.selectedDragOrigin, null, {url: markerDiv, anchor: {left: 4, top: 4}});
    this.markerLayer.addMarker(marker);
  }
  else
  {
    this.selectedDragOrigin = e._point1.toCoords(this.map);
  }
}

Polyline.prototype.onCtrlShiftDragStop = function(e)
{
  // remove center marker
  this.drawAll();
}

/* drag + CTRL + ALT - scale selected points from origin */
Polyline.prototype.onCtrlAltDragStart = function(e)
{
  this._saveHistory();
  this.selectedDragOrigin = null;
  this.selectedDragLastMouseOn = null;
}

Polyline.prototype.onCtrlAltDrag = function(e)
{
  var selected = this.getSelected();
  
  if(this.selectedDragOrigin && selected.length)
  {
    // get Y difference between last point and this point and enlarge if positive, make smaller if negative 
    var delta = 0.02;
    var difFromLast = e._point2.toCoords(this.map).toPixel(this.map).y - this.selectedDragLastMouseOn.toPixel(this.map).y;
    var scale = 1 + delta * (difFromLast < 0 ? 1 : -1);  
  
    this.scalePoints(selected, scale, this.selectedDragOrigin);
    this.drawAll(this.points.length > 300);    
    
    this.selectedDragLastMouseOn = e._point2.toCoords(this.map);
    
    // create marker at origin
    var markerDiv = JAK.mel("div", {className: "scaleOriginMarker"});
    var marker = new SMap.Marker(this.selectedDragOrigin, null, {url: markerDiv, anchor: {left: 4, top: 4}});
    this.markerLayer.addMarker(marker);       
  }
  else
  {
    this.selectedDragLastMouseOn = this.selectedDragOrigin = e._point1.toCoords(this.map);
  }
}

Polyline.prototype.onCtrlAltDragStop = function(e)
{
  // remove center marker
  this.drawAll();
}

// Polyline.prototype.onFlipStart = function(e, target)
// {
// }
// 
// Polyline.prototype.onFlip = function(e)
// {
// }
// 
// Polyline.prototype.onFlipStop = function(e)
// {
// }

/* drag + CTRL + ALT - draw new points just by dragging the middle button (ALT activates "insert before" mode) */
Polyline.prototype.onDragDrawStart = function(e, target)
{
  this.lastDragDrawCoords = null;
  this.dragDrawAddedPoints = 0;

  // deal with first run
  this.dragDrawFirstRun = 0;
  this.dragDrawFirstE = null;

  this.shiftPending = false;
}

Polyline.prototype.onDragDraw = function(e, options)
{
  // don't add points while map is dragged
  if (this.mapIsLocked)
    return;  

  options = options || {};
  var eventPt = e._point2;

  // use this when run in free mouse move mode (without pressed mouse button) - we get different e
  if (!e._point2)
  {
    // start with SHIFT - new segment on first run
    if (e.shiftKey && !this.dragDrawFirstRun)
      this.shiftPending = true;
      
    // when CTRL goes up and SHIFT is pressed start new segment
    // - to make it reliable, allow SHIFT going up first
    if (options.ctrlUp && e.shiftKey)
      this.shiftPending = true;
      
    if (options.shiftUp && e.ctrlKey)
      this.shiftPending = true;

    // don't create new points when CTRL is pressed (but add point on CTRL-up)
    if (e.ctrlKey && !options.ctrlUp) 
      return;

    eventPt = new SMap.Coords.fromEvent(e, this.map).toPixel(this.map);
    e.customId = (e.altKey) ? "_Alt" : "";
    //e.customId += (e.shiftKey) ? "_Shift" : "";
  }

  // on first run, just save the point and draw it on second run,
  // because this may be just click and not a draw
  if (!this.dragDrawFirstRun++)
  {
    this.dragDrawFirstE = e;
    
    if (e.shiftKey)
      this.shiftPending = true;

    return;
  }
  
  if (this.dragDrawFirstRun == 2)
  {
    this._saveHistory();
    this.onDragDraw(this.dragDrawFirstE);
  }

  //**********
  // parameters
  var minimumMouseMove = 15;
  var saveHistoryInterval = 10;

  // get coordinates
  var point = eventPt.toCoords(this.map);
  point.custom = point.custom || {};

  // draw points after moving mouse more than minimum amount of pixels
  var minimumMouseMove = 15;
  
  if (this.lastDragDrawCoords)
  {
    var distance = this.lastDragDrawCoords.distance(point.toPixel(this.map));
  
    if (distance < minimumMouseMove)
      return;
  }

  this.lastDragDrawCoords = point.toPixel(this.map);
  
  // save history in regular intervals (after adding certain number of points)
  if (this.dragDrawAddedPoints++ >= saveHistoryInterval)
  {
    this._saveHistory();
    this.dragDrawAddedPoints = 0;
  }
  
  // make the point start of a segment (if shift is pushed)
  if ((e.customId.match(/_Shift/) && this.dragDrawFirstRun == 3) || this.shiftPending)
  {
    point.custom.segmentStart = true;
    this.shiftPending = false;
  }
  
  // insert point
  this._insertPoint(point, undefined, e.customId.match(/_Alt/));
  
  // redraw
  this.drawAll(this.points.length > 300);
}

Polyline.prototype.onDragDrawStop = function(e)
{
}

Polyline.prototype.startFreeDrawMode = function(event)
{
  this.freeDrawMode = true;
  this.map.disableMarkerDragging();
  this.onDragDrawStart(event);
}  

Polyline.prototype.cancelFreeDrawMode = function()
{
  // create some delay to prevent immediate click on marker after finishing drawing
  this.map.enableMarkerDragging();
  
  if (this.freeDrawMode)
  {
    this.freeDrawMode = false;
    return true;  
  }

  return false;
}

/* click on a map - add point */
Polyline.prototype.onMapClick = function(e, point /* slouží pro volání z onMarkerLeftClick */)
{
  var point = point || SMap.Coords.fromEvent(e, this.map);

  if (this.cancelFreeDrawMode())
    return;

  // don't create new points if end point is within hidden region
  if(this._isPointsOriginInHiddenSegment(null, e.altKey))
    return;

  // standard - add point to the end, whether standard or alternative (ALT - before start)
  if (!e.shiftKey/* && !e.ctrlKey*/)
  {
    this._saveHistory();
    this._insertPoint(point, undefined, e.altKey);
    this.drawAll();
  }
  
  // SHIFT - create new segment
  else if (e.shiftKey/* && !e.ctrlKey*//* && !e.altKey*/)
  {
    this._saveHistory();
    point.custom = point.custom || {};
    point.custom.segmentStart = true;
    this._insertPoint(point, undefined, e.altKey);
    this.drawAll();
  }  
}

/* middle click on a map */
Polyline.prototype.onMapMiddleClick = function(e)
{
  if (this.cancelFreeDrawMode())
    return;

  // free draw mode (SHIFT starts it with new segment, ALT switches appending mode to before, CTRL pauses drawing)
  if (!e.ctrlKey /*&& !e.shiftKey && !e.altKey*/ && JAK.Browser.client != "opera")
  {
    this.startFreeDrawMode();
    //this.onMapClick(e);
  }

  if (e.ctrlKey && e.shiftKey && !e.altKey)
  {
    this._saveHistory();
    
    // get origin (axis center)
    var origin = new SMap.Coords.fromEvent(e, this.map);
    
    // reset flip settings on first run or moving mouse away from last origin
    if (typeof this.flipSettings === 'undefined' 
      || !this.lastFlipOrigin || !this.lastFlipOrigin.equals(origin))
      this.flipSettings = 0;
    
    this.flipSettings %= 4;
    
    this.lastFlipOrigin = origin;
  
    // each time function is called we advance to the next possibility (out of 4 possible combinations)
    var x = false, y = false;;
    
    // horizontal flip
    if (this.flipSettings == 0) 
      x = true, y = false;
    // reset horizontal flip, set vertical flip
    else if (this.flipSettings == 1) 
      x = true, y = true;
    // horizontal + vertical flip
    else if (this.flipSettings == 2)
      x = true, y = false;
    // original    
    else if (this.flipSettings == 3)
      x = true, y = true;      
    
    this.flipSettings++;
  
    this.flipPoints(this.getSelected(), origin, x, y);
  
    // redraw
    this.drawAll();  
  }
}

/* map redraw - redraw our polyline */
Polyline.prototype.onMapRedraw = function(e)
{
  this.drawAll(); 
}

/* click on geometry - possibly activate/deactivate segment */
Polyline.prototype.onGeometryClick = function(e, segmentId)
{
  if (!e.ctrlKey && (e.button != JAK.Browser.mouse.right))
    return;

  this._saveHistory();
  
  var changed = false;
  
  // SHIFT + CTRL + ALT - display only one selected segment
  if (e.shiftKey && e.ctrlKey && e.altKey)
  {
    changed = this.showOnlyOneSegment(segmentId);
  }
  
  // SHIFT + CTRL - switch state of segment (on/off)
  // alternative version for Opera (which zooms out SVG element on ctrl+left/right click
  // and zooms in SVG elements on ctrl+left/right click): right click + CTRL + ALT
  else if ((e.button == JAK.Browser.mouse.left && e.shiftKey && e.ctrlKey && !e.altKey)
    || (e.button == JAK.Browser.mouse.right && !e.shiftKey && e.ctrlKey && e.altKey))
  {
    var clickedSegment = this.getSegments()[segmentId];
    changed = this.showHideSegments(this.isSegmentHidden(clickedSegment), clickedSegment);
  }

  // if no change, revert history
  changed ? this.drawAll() : this._restoreHistory();
}

// najde id počátečního resp. koncového bodu segmentu (ze zadaného id bodu)
Polyline.prototype.segmentStartId = function(id, segments)
{
  segments = segments || this.getSegments();
  
  var segmentId = this.getSegmentIDFromPointID(id, segments);

  var startId = this.getIndexFromPoint(segments[segmentId][0]);
  return startId;
}

Polyline.prototype.segmentEndId = function(id, segments)
{
  segments = segments || this.getSegments();
  
  var segmentId = this.getSegmentIDFromPointID(id, segments);
  var segmentLen = segments[segmentId].length;

  var endId = this.getIndexFromPoint(segments[segmentId][segmentLen - 1]);
  return endId;
}

Polyline.prototype._isPointInHiddenSegment = function(id, segments)
{
  segments = segments || this.getSegments();
  
  if (!segments.length)
    return false;
    
  var id = this.getSegmentIDFromPointID(id, segments);
  return this.isSegmentHidden(segments[id]);
}

Polyline.prototype.isSegmentHidden = function(segment)
{
  return(!!segment[0].custom.hideSegment);
}

Polyline.prototype.showOnlyOneSegment = function(id)
{
  var segments = this.getSegments();
  var changed = false;
  
  for (var i = 0; i < segments.length; i++)
  {
    changed = this.showHideSegments(id === i, segments[i]) || changed;  
  }

  return changed;
}

/* display or hides segments */
Polyline.prototype.showHideSegments = function(showHide, segments)
{
  segments = segments ? [segments] : this.getSegments(); 
  var hide = !showHide;
  var changed = false;
  
  for (var i = 0; i < segments.length; i++)
  {
    var old = !!segments[i][0].custom.hideSegment;
    changed = changed || (old != hide);
    segments[i][0].custom.hideSegment = hide;
    
    // deselect points on hiding segment
    if (hide && hide != old)
      this.deSelectPoints(segments[i]);
  }

  return changed;
}

/* get all track segments */
Polyline.prototype.getSegments = function(points)
{
  points = points || this.points;
  
  if (!points.length)
    return [];

  var segments = [[]];
  
  for (var i = 0; i < points.length; i++)
  {
    var p = points[i];
    
    // segment start? then add new segment
    if (p.custom.segmentStart && segments[segments.length - 1].length)
      segments.push([]);
    
    // append point to the last segment
    segments[segments.length - 1].push(p);
  }
  
  return segments;
}

Polyline.prototype._getVisibleHiddenSegments = function(visible)
{
  var segments = this.getSegments();
  
  if (!segments.length)
    return [];

  var visible = [];
  
  for (var i = 0; i < segments.length; i++)
  {
    if (!!segments[i][0].custom.hideSegment == !visible)
      visible.push(segments[i]);
  }

  return visible;
}

Polyline.prototype.getVisibleSegments = function()
{
  return(this._getVisibleHiddenSegments(true));
}

Polyline.prototype.getHiddenSegments = function()
{
  return(this._getVisibleHiddenSegments(false));
}

/* converts segments to points */
Polyline.prototype.flattenSegments = function(segments)
{
  var points = [];
  
  for (var i = 0; i < segments.length; i++)
    points = points.concat(segments[i]);

  return points;
}

/* get segment ID from point ID */
Polyline.prototype.getSegmentIDFromPointID = function(id, segments)
{
  var segments = segments || this.getSegments();
  var total = 0;
  
  for (var i = 0; i < segments.length; i++)
  {
    total += segments[i].length;

    if (id < total)
      return i;
  }
  
  return -1;
}

Polyline.prototype._getDataFromMarker = function(e, target)
{
  // ** get marker **
  
  // from marker event
  if (!target)
  {
    var marker = e.target;
    target = marker.getContainer()[SMap.LAYER_MARKER];
  }
  
  // from marker div
  // - check is there to fix occassional bug where marker id is invalid
  // (activate this bug by redrawing minimap on layer-enable and intense point drawing using T)
  else if (this.markerLayer._markers[target.markerId])
  {
    var marker = this.markerLayer._markers[target.markerId].marker;
  }
  
  else
    return null;

  // get point index
  var index = (typeof target["data-index"] === 'undefined') ? -1 : target["data-index"];
  var indexHalf = (typeof target["data-index-half"] === 'undefined') ? -1 : target["data-index-half"];

  // return null if invalid
  if (index < 0 && indexHalf < 0)
    return null;

  // transparent point
  if (index < 0)
    return {type: "half", indexHalf: indexHalf, point: marker.getCoords()};
  // full point
  else
    return {type: "full", index: index, point: marker.getCoords()};
}

/* marker drag move - draw auxiliary lines to position */
Polyline.prototype.onMarkerDragMove = function(e)
{
  var marker = this._getDataFromMarker(e);
  
  if (!marker)
    return;
  
  this.markerIsBeingDragged = true;

  this.lineToMousePosDrag(marker);
}

/* marker drag stop - move it */
Polyline.prototype.onMarkerDragStop = function(e)
{
  var marker = this._getDataFromMarker(e);
  
  if (!marker)
    return;

  this.markerIsBeingDragged = false;

  // half marker - add point
  if (marker.type == "half")
  {
    this._saveHistory();
    this._insertPoint(marker.point, marker.indexHalf + 1);
  }
          
  // normal marker - move it
  // non-visible points will either be straightened to a straight line or (SHIFT) rotated/scaled to fit the move
  else if (marker.type == "full")
    this._replacePoint(marker.index, marker.point, contains_class(this.map.getContainer(), "key_down_shift") ? "rotateScale" : ""); 

  this.drawAll();
}

/* react on right clicking a marker (delete point) */
Polyline.prototype._onMarkerRightClick = function(e, target)
{
  var marker = this._getDataFromMarker(e, target);

  if (!marker || e.ctrlKey || e.altKey)
    return;

  // right click - remove point
  if (marker.type == "full" && !e.shiftKey)
  {
    EventUtil.stopPropagation(e);
    EventUtil.preventDefault(e);  
  
    this._saveHistory();
    this._removePoint(marker.index);
    this.drawAll();
  }
  
  // SHIFT + right click - remove segment
  else if (marker.type == "full" && e.shiftKey)
  {
    EventUtil.stopPropagation(e);
    EventUtil.preventDefault(e);
  
    this._saveHistory();
    var id = this.getSegmentIDFromPointID(marker.index);
    this.removePoints(this.getSegments()[id]);
    this.drawAll();
  }
}

/* react on left clicking a marker (close polygon - if clicked on the first element) */
Polyline.prototype.onMarkerLeftClick = function(e, target)
{
  var marker = this._getDataFromMarker(e, target);
  e = e.data.event;

  if(!marker || this.freeDrawMode)
    return;

  // half marker - add point (left click)
  if (marker.type == "half" && !e.shiftKey && !e.ctrlKey && !e.altKey)
  {
    this._saveHistory();
    this._insertPoint(marker.point, marker.indexHalf + 1);
    this.drawAll();
  }

  // close / open polygon on clicking on the first point (left click)
  else if (marker.type == "full" && marker.index == 0 && this.points.length >= 3 && !e.shiftKey && !e.ctrlKey && !e.altKey)
  {
    this._saveHistory();
    this._setPolygon(!this._isPolygon());
    this.drawAll();
  }
  
  // init / remove closed state (can't add new points to the end) on clicking the last point (SHIFT + left-click)
  else if (marker.type == "full" && (marker.index == this.points.length - 1) && !e.shiftKey && !e.ctrlKey && !e.altKey)
  {
    this._saveHistory();
    this._removeNewPointsOrigin();
    this.closed = !this.closed;
    this.drawAll();
  }
  
  // mark marker as origin - new clicks on map will add points after it, not after track end (ALT + left-click)
  else if (marker.type == "full" && !e.shiftKey && !e.ctrlKey && e.altKey)
  {
    this._saveHistory();
    
    // if clicked on is identical to original, then remove it
    if (this.points[marker.index] === this.addNewPointsOrigin)
    {
      this._removeNewPointsOrigin();
      this.drawAll();
    }
    // standard - add
    // - restore history if no change
    else if (!this._addNewPointsOrigin(this.points[marker.index]))
      this._restoreHistory();
      
    this.drawAll();
  }  
  
  // disconnect segments (SHIFT + left click) 
  else if (marker.type == "full" && e.shiftKey && !e.ctrlKey && !e.altKey && (marker.index < this.points.length - 1))
  {debugger;
    var isSegmentEnd = this.points[marker.index + 1].custom.segmentStart;
    this._saveHistory();
    this.points[marker.index + 1].custom.segmentStart = !isSegmentEnd;
    this.drawAll();
  }
  
  // pokud držíme CTRL, tak snapujeme na body
  // -- omezení: není možno toto u začátku segmentu (ale začátku tracku ano)
  // - také není možno v takovém bodě začít segment
  else if (e.ctrlKey)
  {
    // clone je zde třeba, jinak SHIFT povede k přerušení segmentu u překrytého bodu
    this.onMapClick(e, marker.point.clone());
  }
}

Polyline.prototype.onMarkerLeftDblClick = function(e, target)
{
  // note: dblclick on first point doesn't fire - is cancelled by function above

  var marker = this._getDataFromMarker(e, target);
  e = e.data.event;

  // dbl click without any modifier keys - same as SHIFT+click (join/disconnect segments)
  if (marker.type == "full" && (marker.index < this.points.length - 1) 
    && !e.ctrlKey && !e.altKey && !e.shiftKey)
  {
    this._saveHistory();
    var isSegmentEnd = this.points[marker.index + 1].custom.segmentStart;
    this.points[marker.index + 1].custom.segmentStart = !isSegmentEnd;
    this.drawAll();
    
    EventUtil.preventDefault(e);
    EventUtil.stopPropagation(e);  
  }
}

Polyline.prototype.onMarkerMouseEnter = function(e, target)
{
  // exit if already generated
  if (target.alreadyCreated)
    return;
  
  var that = target.that;

  // get marker div
  var markerDiv = that._getDataFromMarker(e, target);
  
  // invalid marker id (may happen - rarely)?
  if (!markerDiv)
    return;

  var point = markerDiv.point;
  
  marker = that.markerLayer._markers[target.markerId].marker; /* marker object */
    
  // create bubble
  if (!marker.getBubble())
  {
    var bubble = new SMap.Bubble({className: "polygonMarkerBubble"}, {});
    marker.decorate(SMap.Marker.Feature.Bubble, bubble);
  }

  target.alreadyCreated = true;

  // *** create bubble info ***
  var text = "";
  
  // get name (if any)
  if (point.custom.name)
    text += "<span class=track_point_name>" + point.custom.name + "</span><br>";
    
  // get description (if any)
  if (point.custom.desc)
    text += "<span class=track_point_desc>" + point.custom.desc + "</span><br>";

  // get coordinates
  text += "<span class=track_coords>" + point.format("WGS84", "min") + "</span><br>";

  // get indexes
  var segments = that.getSegments();
  var id = markerDiv.index;
  var segId = that.getSegmentIDFromPointID(markerDiv.index, segments);
  var idFromSegmentStart = id - that.getIndexFromPoint(segments[segId][0])

  text += "<span class=track_point_indexes>";
  
  if (point.custom.isPoint || segments.length <= 1)
    text += tr("Index") + ": " + (id + 1);
  else
    text += tr("Index") + ": " + (idFromSegmentStart + 1) + " (" + (id + 1) + ")<br>" + tr("Segment") + ": " + (segId + 1);
  
  text += "</span><br>";

  // get time
  if (!isNaN(point.custom.timestamp))
  {
    var time = new Date(point.custom.timestamp);
    text += "<span class=track_date>" + time.toLocaleDateString() + "</span>";
    text += "  <span class=track_time>" + time.toLocaleTimeString() + "</span><br>";
  }
  
  // get elevation
  if (!isNaN(point.custom.elevation))
  {
    text += "<span class=track_elevation>" + tr("_elevation") + ": " + point.custom.elevation.toFixed(1) + " m</span><br>";
  }  

  marker.getBubble().setHTML(text);
}

Polyline.prototype.getSelected = function(points)
{
  points = points || this.points;
  var selected = [];
  
  for (var i = 0; i < points.length; i++)
  {
    if (points[i].custom.isSelected)
    {
      var p = points[i];
      p.custom.curIndex = i;
      selected.push(p);
    }
  }

  return selected;
}

Polyline.prototype.getIndexFromPoint = function(point)
{
  for (var i = 0; i < this.points.length; i++)
  {
    if (this.points[i] === point)
      return i;
  }
  
  return null;
}

Polyline.prototype.movePoints = function(points, delta /* SMap.pixel */)
{
  // get distance and azimuth of the move
  var center = new SMap.Pixel(0, 0).toCoords(this.map); /* center of map */
  delta = delta.toCoords(this.map);                     /* moved center */
  var distance = center.distance(delta);
  var azimuth =  center.azimuth(delta);
  //log(distance);

  for (var i = 0; i < points.length; i++)
  {
    var p = points[i];
    var newP = p.projectPoint(azimuth, distance);
    this.updateCoords(p, newP.lat(), newP.lon());
  }
}

Polyline.prototype.rotatePoints = function(points, angle, origin)
{
  // get origin point (center of rotation)
  var origin = origin || this.map.computeCenterZoom(points)[0];

  for (var i = 0; i < points.length; i++)
  {
    var p = points[i];
    var newP = origin.projectPoint(origin.azimuth(p) + angle, origin.distance(p));
    this.updateCoords(p, newP.lat(), newP.lon());
  }
}

Polyline.prototype.scalePoints = function(points, scale, origin)
{
  // get origin point (center of rotation)
  origin = origin || this.map.computeCenterZoom(points)[0];

  for (var i = 0; i < points.length; i++)
  {
    var p = points[i];
    var newP = origin.projectPoint(origin.azimuth(p), origin.distance(p) * scale);
    this.updateCoords(p, newP.lat(), newP.lon());
  }
}

Polyline.prototype.flipPoints = function(points, origin, horizontal, vertical)
{
  horizontal = horizontal ? 1 : - 1;
  vertical = vertical ? 1 : - 1;
  
  origin = origin || this.map.computeCenterZoom(points)[0];
  var originLat = origin.lat();
  var originLon = origin.lon();
  
  for (var i = 0; i < points.length; i++)
  {
    var p = points[i];

    var pLat = p.lat();
    var pLon = p.lon();      
    pLat = originLat + (originLat - pLat) * vertical;
    pLon = originLon + (originLon - pLon) * horizontal;
    this.updateCoords(p, pLat, pLon);
  }
}

Polyline.prototype._selectDeselectPoints = function(points, showHide)
{
  if (!points.length)
    return false;

  var changed = false;
  showHide = !!showHide;
  
  for (var i = 0; i < points.length; i++)
  {
    changed = changed || (!!points[i].custom.isSelected != showHide);
    points[i].custom.isSelected = showHide;
  }
  
  return changed;
}

Polyline.prototype.selectPoints = function(points)
{
  return(this._selectDeselectPoints(points, true));
}

Polyline.prototype.deSelectPoints = function(points)
{
  return(this._selectDeselectPoints(points, false));
}

// get polyline total length
Polyline.prototype.getLength = function(segments)
{
  var segments = segments || this.getSegments();
  var distance = 0;

  for (var i = 0; i < segments.length; i++)
  {
    var seg = segments[i];
    
    if (this.isSegmentHidden(seg))
      continue;    
  
    for (var j = 0, len = seg.length; j < len - 1; j++)
      distance += seg[j].distance(seg[j + 1], {quick: (this.points.length > 300)}); /* lot of points --> use quicker formula */
      
    // if polygon, get distance to first segment point as well
    if (this._isPolygon() && seg.length)
      distance += seg[0].distance(seg[seg.length - 1], {quick: (this.points.length >300)});
  }

  return distance;
}

// get polyline (polygon) total area
// info: http://gis.stackexchange.com/questions/711/how-can-i-measure-area-from-geographic-coordinates
// info: http://trac.osgeo.org/openlayers/browser/trunk/openlayers/lib/OpenLayers/Geometry/LinearRing.js?rev=10116#L233
// - doesn't work correctly on polygons, where lines are overlapping
Polyline.prototype.getArea = function()
{
  var area = 0.0;

  // iterate over segments
  var segments = this.getSegments();
  
  for (var i = 0; i < segments.length; i++)
  {
    var seg = segments[i];
    
    if (this.isSegmentHidden(seg))
      continue;
    
    // get pairs of WGS coordinates
    var coords = [];
    
    for (var j = 0; j < seg.length; j++)
      coords.push(seg[j]);
    
    // add starting point to the end  
    if (seg.length)
      coords.push(seg[0]);
    
    // no action if 2 points or less  
    var len = coords.length;
    
    if (len <= 2)
      continue;
    
    // compute area  
    var subarea = 0.0;
    var rad = Math.PI / 180.0;

    for(var k = 0; k < len - 1; k++)
    { 
      p1 = coords[k]; 
      p2 = coords[k + 1];
      subarea += (p2.lon() - p1.lon()) * rad * (2 + Math.sin(p1.lat() * rad) + Math.sin(p2.lat() * rad)); 
    } 
    
    // append to total area
    area += Math.abs(subarea * 6378137.0 * 6378137.0 / 2.0);  
  }

  return area;
}

/* handle keys (moving selection, history) */
Polyline.prototype.onKeyDown = function(e, target)
{
  /* pokud táhneme bod, tak ignorujeme klávesové zkratky */
  if (this.markerIsBeingDragged)
    return;
    
  /* překreslíme - pro případ, že jsme zmačkli alt a čára má tedy vést jinak */
  this.lineToMousePosDraw(null, e);
    
  var key = String.fromCharCode(e.keyCode).toUpperCase();
  
  // CTRL+Z: restore history
  if (key == 'Z' && e.ctrlKey && !e.altKey)
  {
    this._restoreHistory(e.shiftKey ? true : false);
    EventUtil.preventDefault(e);
  }
    
  // TAB: iterate over track segments or points
  if (e.keyCode == 9 && (e.ctrlKey || e.shiftKey) && !e.altKey)
  {
    // SHIFT + TAB - iterate over track segments
    if (e.shiftKey && !e.ctrlKey)
    {
      this.lastTabIndex = this.lastTabIndex || 0;
      var startIndex = this.lastTabIndex;
      var segments = this.getSegments();
  
      this.lastTabIndex = (this.lastTabIndex + 1 >= segments.length) ? 0 : ++this.lastTabIndex;

      if (segments.length && segments[this.lastTabIndex].length)
      {
        this._saveHistory();
        var changed = this.showOnlyOneSegment(this.lastTabIndex);
        changed ? (this.map.setCenterZoom.apply(this.map, this.map.computeCenterZoom(segments[this.lastTabIndex]))) : this._restoreHistory();
      }
    }
    
    // CTRL (+ SHIFT - increases speed) + TAB - iterate over points
    else if (e.ctrlKey && e.shiftKey)
    {
      this.lastPointTabIndex = this.lastPointTabIndex || 0;
      var startIndex = this.lastPointTabIndex;
      var delta = e.ctrlKey ? 25 : 1;

      this.lastPointTabIndex = (this.lastPointTabIndex + delta >= this.points.length) ? 0 : (this.lastPointTabIndex + delta);
      
      if (this.points.length)
        this.map.setCenter(this.points[this.lastPointTabIndex]);
    }    

    EventUtil.preventDefault(e);
    EventUtil.stopPropagation(e);
  }
  
  // escape (deselect all selected + show all hidden segments 
  //   + remove alternative origin + remove free draw mode)
  else if (e.keyCode == 27 /* escape */)
  {
    this._saveHistory();
    var changed = this.deSelectPoints(this.getSelected());
    changed = this.showHideSegments(true) || changed;
    changed = this._removeNewPointsOrigin() || changed;
    changed = this.cancelFreeDrawMode() || changed;
    changed ? this.drawAll() : this._restoreHistory();
  } 
  
  else if (key == "E" || key == "S" || key == "D" || key == "F" || (e.keyCode >= 37 && e.keyCode <= 40) /* arrows */
    || e.keyCode == 46 /* delete */)
  {
    var x = 0, y = 0, angle = 0, scale = 0;

    // info: arrows don't work inside of GSAK, so use alternative keys as well
    //    38
    // 37 40 39
    //
    // keyup - 1x
    // keydown - repeatedly
    // 
    // delete: 46
    // escape: 27    

    // up arrow (no: move, CTRL: scale up) 
    if (key == "E" || e.keyCode == 38)
      e.ctrlKey ? scale = 0.01 : y = -1;
      
    // down arrow (no: move, CTRL: scale down) 
    else if (key == "D" || e.keyCode == 40)
      e.ctrlKey ? scale = -0.01 : y = 1;
      
    // left arrow (no: move, CTRL: rotate left) 
    else if (key == "S" || e.keyCode == 37)
      e.ctrlKey ? angle = -1 : x = -1;
      
    // right arrow (no: move, CTRL: rotate right) 
    else if (key == "F" || e.keyCode == 39)
      e.ctrlKey ? angle = 1 : x = 1;

    // move quicker with SHIFT pressed
    if (e.shiftKey)
    {
      var delta = 10;
      x *= delta;
      y *= delta;
      angle *= delta;
      scale *= delta;      
    }

    // get selected points
    var selected = this.getSelected();

    if (!selected.length)
      return;

    // save state
    this._saveHistory();

    // prevent default action
    EventUtil.stopPropagation(e);
    EventUtil.preventDefault(e);

    // -- modify points --
    // move
    if (x || y)
      this.movePoints(selected, new SMap.Pixel(x, y));
      
    // rotate
    else if (angle)
      this.rotatePoints(selected, angle);

    // scale
    else if (scale)
      this.scalePoints(selected, 1 + scale);
      
    // delete (delete all selected)
    else if (e.keyCode == 46 && !e.shiftKey && !e.ctrlKey && !e.altKey)
      this.removePoints(selected);

    // remove polygon mode, if too few points
    if (this.points.length <= 2)
      this._setPolygon(false);

    this.drawAll();
  }
  
  // space - temporarily hide all markers
  else if (e.keyCode == 32)
  {
    add_class(document.body, "noShowTrackMarkers");
    
    // Firefox requires cancelling here as well
    EventUtil.preventDefault(e);
    EventUtil.stopPropagation(e);    
  }
  
  /*** saving/cutting/pasting ponts ***/
  // - use keydown instead of keypress (always reliable)
  // for reference, working keypress events are this:
  // IE9-10 - base key + SHIFT + CTRL + ALT
  // IE7-8 - base key
  // Chrome -  base key + SHIFT
  // Firefox - base key + SHIFT + CTRL + ALT
  
  // CTRL + A --> select all points
  else if (key == "A" && e.ctrlKey && !e.shiftKey && !e.altKey)
  {
    this._saveHistory();
    var points = this.flattenSegments(this.getVisibleSegments());
    this.selectPoints(points) ? this.drawAll() : this._restoreHistory();
    EventUtil.preventDefault(e);
    EventUtil.stopPropagation(e);  
  }  
  
  // CTRL + C --> copy selected points
  // CTRL + X --> cut selected points
  else if ((key == "C" || key == "X") && e.ctrlKey && /*!e.shiftKey &&*/ !e.altKey)
  {
    EventUtil.preventDefault(e);
    EventUtil.stopPropagation(e);
      
    var selected = this.getSelected();

    if (!selected.length)
      return;
      
    this._saveHistory();

    // unless SHIFT pressed: create segments on beginnings of non-continuous areas
    if (!e.shiftKey)
    {
      var continuousIndexes = this.getContinuousAreas(selected, {returnIndexes: true});
      selected = this._restorePoints(this._backupPoints(selected));
      
      // create segments
      for (var i = 1; i < continuousIndexes.length; i++)
        selected[continuousIndexes[i]].custom.segmentStart = true;
    }

    // remove points on CTRL + X
    if (key == "X")
      this.removePoints(this.getSelected());    
    
    this.$super.copyPastePoints = this._backupPoints(selected);
    this.drawAll();
  }
  
  // CTRL + V --> paste selected points
  // - modifier ALT pastes them before, SHIFT creates new segment    
  else if (key == "V" && e.ctrlKey)
  {
    EventUtil.preventDefault(e);
    EventUtil.stopPropagation(e);

    var points = this.$super.copyPastePoints;

    if (!points || !points.length)
      return;
    
    this._saveHistory();
    points = this._restorePoints(points);

    // push before? then reverse the array first
    if (e.altKey)
      points.reverse();
    
    // SHIFT - creates new segment
    if (e.shiftKey)
    {
      points[0].custom = points.custom || {};
      points[0].custom.segmentStart = true;
    }    

    // insert points
    for (var i = 0; i < points.length; i++)
      this._insertPoint(points[i], undefined, e.altKey);
    
    // make all pasted points selected
    this.selectPoints(points);
    
    this.drawAll();
  }
  
  // R + SHIFT + CTRL -> reverse selected points; R + SHIFT --> reverse entire track
  else if (key == "R" && e.shiftKey && !e.altKey)
  {
    var points = !e.ctrlKey ? this.points : this.getSelected();
    
    if (points.length >= 2)
    { 
      this._saveHistory();
      this.reversePoints(points);
      this.drawAll();
    }
    
    EventUtil.preventDefault(e);
    EventUtil.stopPropagation(e);
  }
  
  // J + SHIFT + CTRL -> join segments with at least one selected point; J + SHIFT --> join all segments
  else if (key == "J" && e.shiftKey && !e.altKey)
  {
    this.joinAllSegments(e.ctrlKey);    
    EventUtil.preventDefault(e);
    EventUtil.stopPropagation(e);
  }
  
  // I -> inverse selection
  else if (key == "I" && !e.shiftKey && !e.altKey && !e.ctrlKey)
  {
    var points = this.flattenSegments(this.getVisibleSegments());
    
    if (points.length)
    {
      this._saveHistory();
      
      for (var i = 0; i < points.length; i++)
        points[i].custom.isSelected = !points[i].custom.isSelected;
    
      this.drawAll();
    }
  }
  
  // T - activate "free draw mode" (ALT + SHIFT are used as modifiers)
  else if (key == "T" && !e.ctrlKey)
  {
    EventUtil.preventDefault(e);
    this.freeDrawMode ? this.cancelFreeDrawMode() : this.startFreeDrawMode();
  }
  
  // M - draw help line from end point to mouse position
  else if (key == "M" && !e.ctrlKey && !e.shiftKey && !e.altKey)
  {
    this.lineToMousePos = !this.lineToMousePos;
    
    if (!this.lineToMousePos)
      this.lineToMousePosDelete();
  }
  
  // SHIFT + M - don't hide points when too close (show them all)
  else if (key == "M" && !e.ctrlKey && e.shiftKey && !e.altKey)
  {
    this.hidePointsWhenTooClose = !this.hidePointsWhenTooClose;
    this.drawAll();
  }  
}

Polyline.prototype.onKeyUp = function(e, target)
{
  var key = String.fromCharCode(e.keyCode).toUpperCase();
  
  /* překreslíme - pro případ, že jsme zmačkli alt a čára má tedy vést jinak */
  this.lineToMousePosDraw(null, e);  
  
  // space unhide markers hidden on keydown
  if (e.keyCode == 32)
  {
    remove_class(document.body, "noShowTrackMarkers");
  }
  
  // CTRL in free draw mode creates point
  else if (e.keyCode == 17 && this.freeDrawMode && this.lastMouseMoveEvent)
  {
    this.onDragDraw(this.lastMouseMoveEvent, {ctrlUp: true});
  }
  
  else if (e.keyCode == 16 && this.freeDrawMode)
  {
    this.onDragDraw(this.lastMouseMoveEvent, {shiftUp: true});
  }  
}

Polyline.prototype.onKeyPress = function(e, target)
{
  var keyCode = EventUtil.getCharCode(e);
  var key = String.fromCharCode(keyCode).toUpperCase();

  // space - temporarily hide all markers (prevent default action)
  if (e.keyCode == 32)
  {
    EventUtil.preventDefault(e);
    EventUtil.stopPropagation(e);
  }
}

Polyline.prototype.onBlur = function(e, target)
{
  remove_class(document.body, "noShowTrackMarkers");
}

Polyline.prototype.onFocus = function(e, target)
{
  remove_class(document.body, "noShowTrackMarkers");
}

Polyline.prototype.onMouseMove = function(e, target)
{
  this.lastMouseMoveEvent = e;
  
  if (this.freeDrawMode)
  {
    this.onDragDraw(e);
  }
  
  else
  {
    this.lineToMousePosDraw(SMap.Coords.fromEvent(e, this.map), e);
  }
}

Polyline.prototype.lineToMousePosDelete = function()
{
  if (this.lineToMousePosGeomLayer)
    this.lineToMousePosGeomLayer.removeAll();
}

// tato funkce je společný konec pro další 2 funkce (přímo vykreslí čáru)
Polyline.prototype.lineToMousePosCreate = function(points)
{
  // clear it
  this.lineToMousePosDelete();
 
  var options = 
  {
    color: "#FFA200" /* orange: IE7-8 fix */,
    width: 3,
    outlineWidth: 0,
    opacity: 0.85
  };
  
  // create geom layer on first run
  if (!this.lineToMousePosGeomLayer)
  {
    this.lineToMousePosGeomLayer = new SMap.Layer.Geometry();
    this.map.addLayer(this.lineToMousePosGeomLayer).enable();
  }  
  
  this.lineToMousePosGeom = new SMap.Geometry(SMap.GEOMETRY_POLYLINE, null, points, options);
  this.lineToMousePosGeomLayer.addGeometry(this.lineToMousePosGeom);    
}

Polyline.prototype.lineToMousePosDrag = function(marker)
{
  var points = [];
  
  // získáme segmenty, abychom je nemuseli získávát zbytečně opakovaně
  var segments = this.getSegments()  
  
  if (marker.type == "full")
  {
    // bod před
    if (marker.index > 0                                   /* bod nesmí být úplně první */
      && !this.points[marker.index].custom.segmentStart)   /* ani nesmí být na začátku segmentu */
    {
      var i = 0;                                           /* hledáme takový předchozí bod, který má marker */
      while (!this.points[marker.index - ++i].hasMarker);  /* -- protože při malém zoomu měníme čáru mezi 2 okolními markery */

      points.push(this.points[marker.index - i]);          /* přidáme předchozí bod s markerem */
    }                                   
    else if (this._isPolygon())                            /* pokud je polygon a jsme na začátku, tak přidáme čáru ke konci segmentu */
    {
      points.push(this.points[this.segmentEndId(marker.index, segments)]);
    }
               
    // aktuální tažený bod     
    points.push(marker.point);

    // bod za
    if (marker.index < this.points.length - 1                  /* bod nesmí být poslední */
      && !this.points[marker.index + 1].custom.segmentStart)   /* ani obecně na konci segmentu */
    {
      var i = 0;
      while (!this.points[marker.index + ++i].hasMarker);    
    
      points.push(this.points[marker.index + i]);
    }        
    else if (this._isPolygon())                                /* pokud máme polygon a jsme na konci segmentu, tak přidáme začátek segmentu */
    {
      points.push(this.points[this.segmentStartId(marker.index - 1, segments)]);
    }
  }
  
  else if (marker.type == "half")
  {
    points = [this.points[marker.indexHalf], marker.point];
    
    if (marker.indexHalf < this.points.length - 1                       /* bod nesmí být poslední */
      && !this.points[marker.indexHalf + 1].custom.segmentStart)        /* ani obecně na konci segmentu (obojí nebude nikdy, pokud není polygon) */
    {    
      points.push(this.points[marker.indexHalf + 1]);                   /* přidáme následující bod */
    }
    else if (this._isPolygon()) /* pokud máme polygon, tak tento bod může být "za posledním", proto přidáme první bod segmentu */
    {
      points.push(this.points[this.segmentStartId(marker.indexHalf, segments)]);
    }
  }

  // draw the line
  this.lineToMousePosCreate(points);
}

Polyline.prototype.lineToMousePosDraw = function(p2, e)
{
  if (!this.lineToMousePos || !this.points.length || this.closed || this.mapIsLocked)
    return false;
  
  if (this.markerIsBeingDragged)    /* nevykreslujeme, pokud je tažen nějaký bod */
    return false;
    
  if (e.shiftKey)                   /* SHIFT značí začátek segmentu, k začátku segmentu nikdy nevykreslujeme */
  {
    this.lineToMousePosDelete();
    return false;
  }
  
  // pokud reagujeme na nějakou jinou klávesovou kombinaci, kde nejsou souřadnice, a tedy bod nemáme, tak ukončíme
  if (!p2 && !this._p2Backup)
    return false;  

  // uložíme si pro případ, že pomocí ALT přehodíme směr kreslení (tam nedostáváme aktuální souřadnice)
  if (!p2)
    p2 = this._p2Backup;
  
  this._p2Backup = p2;

  // získáme segmenty, abychom je nemuseli získávát zbytečně opakovaně
  var segments = this.getSegments()

  // create line from last point (or new points origin) to mouse position
  var p1;
  var p3 = null;     /* koncový bod druhé čáry, pokud máme addNewPointsOrigin */
  
  if (this.addNewPointsOrigin) /* kreslíme za nebo před addNewPointsOrigin */
  {
    // pokud addNewPointsOrigin leží ve skrytém segmentu, tak ukončíme
    if (this._isPointsOriginInHiddenSegment(segments, e.altKey))
    {
      this.lineToMousePosDelete();
      return false;
    }
  
    // jedním z bodů bude addNewPointsOrigin
    p1 = this.addNewPointsOrigin;
    
    // zjistíme jeho id
    var id = this.getIndexFromPoint(this.addNewPointsOrigin);

    // přidáme čáru ještě k dalšímu bodu
    if (!e.altKey && id != this.segmentEndId(id, segments))         /* ALT off - přidáme jednu čáru navíc k dalšímu bodu, pokud existuje */
      p3 = this.points[id + 1];
    else if (e.altKey && id != this.segmentStartId(id, segments))   /* ALT on - přidáme jednu čáru navíc k předchozímu bodu, pokud existuje */
      p3 = this.points[id - 1];
    
    // pokud máme polygon, tak ošetříme případ, že máme označen první nebo poslední bod  
    if (this._isPolygon())
    {
      if (e.altKey && id == this.segmentStartId(id, segments))
        p3 = this.points[this.segmentEndId(id, segments)];
        
      else if (!e.altKey && id == this.segmentEndId(id, segments))
        p3 = this.points[this.segmentStartId(id, segments)];
    }
  }
  else                                /* kreslíme za konec nebo před začátek */
  {
    p1 = !e.altKey ? this.points[this.points.length - 1] : this.points[0];   /* podle stavu ALTu přidáme poslední nebo první bod */
    
    if (this._isPolygon())                             /* pokud máme ale polygon, tak přidáme podle stavu ALTu ještě začátek nebo konec segmentu */
      p3 = !e.altKey ? this.points[this.segmentStartId(this.points.length - 1, segments)] : this.points[this.segmentEndId(0, segments)];
    
    // pokud bod leží ve skrytém segmentu, čáru nezobrazíme  
    if (this._isPointInHiddenSegment(!e.altKey ? (this.points.length - 1) : 0, segments))
    {
      this.lineToMousePosDelete();
      return false;
    }      
  }

  // the following fixes problem, when we were unable to click on lines, because the lineToMouse was on top
  // -- so we make the line somewhat shorter (so that there is nothing on top)
  var subtractDist = 4;
  var newDist = Math.max(p1.pixelDistance(this.map, p2) - subtractDist, 0);

  // draw the line
  var points = [p1, p1.pixelProject(this.map, p1.pixelAzimuth(this.map, p2), newDist)];
  if (p3) points.push(p3);
  
  this.lineToMousePosCreate(points);
  
  return true;
}

/* selection function - we draw polygon and select points based on it */
Polyline.prototype.onPolygonSelectionStart = function(e, target, selection)
{
  if (this.selectionId)
    return;

  this.selectionId = e.customId;
  
  this.selectionPolygon = [];
  
  this.selectionPolygonLayer = new SMap.Layer.Geometry("polylineHelperLayer");
  this.map.addLayer(this.selectionPolygonLayer);
  this.selectionPolygonLayer.enable();
  this.selectionType = e.customId.match(/_(.*?)_/)[1]; /* Rectangle, Polygon */
  this.selectionMode = e.customId.replace(/^.*_/, "");
  //log("start");
  var colors = {"AddToSelection": "green", "Select": "yellow", "DeSelect": "red", "SelectXOR": "#FFA500" /* orange: IE7-8 fix */}
  this.selectionColor = colors[this.selectionMode];
}

Polyline.prototype.onPolygonSelection = function(e)
{
  if (this.selectionId != e.customId)
    return;
    
  // draw polygon
  if (this.selectionType == "Polygon")
  {
    this.selectionPolygon.push(e._point2.clone().toCoords(this.map));
  }
  
  // draw rectangle
  else if (this.selectionType == "Rectangle")
  {
    e._backupRedraw();
    e._dom.container.style.background = this.selectionColor;
    
    this.selectionPolygon = [];
    var p1 = e._point1.clone();
    var p2 = e._point2.clone();
    var p3 = new SMap.Pixel(p2.x, p1.y);
    var p4 = new SMap.Pixel(p1.x, p2.y);
    this.selectionPolygon.push(p1.toCoords(this.map));
    this.selectionPolygon.push(p4.toCoords(this.map));
    this.selectionPolygon.push(p2.toCoords(this.map));
    this.selectionPolygon.push(p3.toCoords(this.map));
  }

  if (this.selectionType != "Rectangle")
  {
    this.selectionPolygonLayer.removeAll();  
    var options = {color: this.selectionColor, outlineWidth: 4, outlineColor: "blue", outlineOpacity: 0.3, opacity: 0.3}; 
    var selPol = new SMap.Geometry(SMap.GEOMETRY_POLYGON, null, this.selectionPolygon, options);
    this.selectionPolygonLayer.addGeometry(selPol);
  }
}

// we have 2 functions here - one is called on drag success and one on drag fail,
// calling _decideSelection on drag fail makes CTRL+click combinations fail
Polyline.prototype.onPolygonSelectionStop = function(e)
{
  if (this.selectionId != e.customId)
    return;
  
  this._decideSelection(this.selectionMode, this.selectionPolygon);
  this.onPolygonSelectionStopFail(e);
}

Polyline.prototype.onPolygonSelectionStopFail = function(e)
{
  if (this.selectionId != e.customId)
    return;

  this.selectionId = null;
  this.selectionPolygonLayer.removeAll();  
  this.map.removeLayer(this.selectionPolygonLayer);
  delete this.selectionPolygonLayer;
}

Polyline.prototype._decideSelection = function(mode, points)
{
  mode = mode || "AddToSelection";
  this._saveHistory();
  var segments = this.getSegments();

  // AddToSelection: add all points in range to existing selection
  // Deselect: remove all points in range from existing selection
  // Select: replace any previous selection with points withing range
  // SelectXOR: intersection of previous selection and points within range
  for (var i = 0, seglen = segments.length; i < seglen; i++)
  {
    var seg = segments[i];
    
    for (var j = 0, len = seg.length; j < len; j++)
    {
      var point = seg[j];
      
      // ignore hidden segments
      if (!j && point.custom.hideSegment)
        break;      
      
      var isIn = false;
      
      if (point.inMap(this.map))
        isIn = point.inPolygon(this.map, points);

      // select or unselect point within range
      if (isIn)
      {
        point.custom.isSelected = (mode == "DeSelect" || (mode == "SelectXOR" && !point.custom.isSelected)) ? false : true;
      }
  
      // if add to selection or selection XOR, then deselect all points out of range
      else if (mode == "Select" || mode == "SelectXOR")
        point.custom.isSelected = false;        
    }
  
  }

  this.drawAll();
}

/* functions for dealing with polygon mode (filled shape) */
Polyline.prototype._isPolygon = function()
{
  return(this.isPolygon);
}

Polyline.prototype._setPolygon = function(bool)
{
  this.isPolygon = !!bool; 
}

// remove origin for newly added points (now it will add to end again)
Polyline.prototype._removeNewPointsOrigin = function()
{
  if (this.addNewPointsOrigin)
  {
    this.addNewPointsOrigin = null;
    return true;
  }
  
  return false;
}

Polyline.prototype._addNewPointsOrigin = function(point)
{
  if (!point || this.addNewPointsOrigin === point)
    return false;
    
  this._removeNewPointsOrigin()
  this.addNewPointsOrigin = point;
  return true;
}

// kontrola (po smazání bodů), zda newPointsOrigin existuje - pokud ne, tak ho odstraníme
Polyline.prototype._syncNewPointsOrigin = function()
{
  if (this.addNewPointsOrigin && !this.getIndexFromPoint(this.addNewPointsOrigin))
    this._removeNewPointsOrigin();
}


Polyline.prototype._isPointsOriginInHiddenSegment = function(segments, altKey)
{
  segments = segments || this.getSegments();
  
  if (!segments.length)
    return false;
  
  // get segment id
  var id = altKey ? 0 : segments.length - 1;
  
  if (this.addNewPointsOrigin)
  {
    var idAlt = this.getSegmentIDFromPointID(this.getIndexFromPoint(this.addNewPointsOrigin));
    id = (idAlt == -1) ? id : idAlt;
  }

  return (!!segments[id][0].custom.hideSegment);
}

// replaces latitude and longitude with new values (using new SMap.Coords would break the functions)
Polyline.prototype.updateCoords = function(p, newLat, newLon)
{
  p.setLatLon(newLat, newLon);
}

/* functions for exporting/importing point (without having to save any methods etc.) */
Polyline.prototype._backupPoints = function(points)
{
  var isArray = Array.isArray(points);
  points = isArray ? points : [points];
  var backup = [];
  
  for (var i = 0; i < points.length; i++)
  {
    var p = points[i];
    var newP = {lat: p.lat(), lon: p.lon(), custom: {}};
    
    for (var prop in p.custom)
      newP.custom[prop] = p.custom[prop];
      
    // p.custom.isSelected;
    // p.custom.segmentStart;
    // p.custom.hideSegment;
    // p.custom.elevation;
    // p.custom.timestamp;
    
    backup.push(newP);
  }
  
  return (isArray ? backup : backup[0]);
}

Polyline.prototype._restorePoints = function(backup)
{
  var isArray = Array.isArray(backup);
  backup = isArray ? backup : [backup];
  var points = [];

  for (var i = 0; i < backup.length; i++)
  {
    var oldP = backup[i];

    var p = new SMap.Coords(oldP.lat, oldP.lon);
    p.custom = {};
    
    for (var prop in oldP.custom)
      p.custom[prop] = oldP.custom[prop];    

    points.push(p);
  }
  
  return (isArray ? points : points[0]);  
}

/* history - save polyline state and allow reverting to previous state */
Polyline.prototype._saveHistory = function()
{
  //log("saved: " + new Date().getTime()  + " - " + arguments.callee.caller);
  var points = this._backupPoints(this.points);
  var addNewPointsOrigin = this.addNewPointsOrigin ? this.getIndexFromPoint(this.addNewPointsOrigin) : null;
  
  this.history.push({points: points, isPolygon: this._isPolygon(), closed: this.closed, 
    addNewPointsOrigin: addNewPointsOrigin, lineToMousePos: this.lineToMousePos});
  //log("saved: " + this.history.length);
}

Polyline.prototype._restoreHistory = function(restoreForward /* remains to be implemented */)
{
  if (!this.history.length)
    return;
  
  var newState = this.history.pop();

  // restore
  this.points = this._restorePoints(newState.points);
  this._setPolygon(newState.isPolygon);
  this.closed = newState.closed;
  this._addNewPointsOrigin(this.points[newState.addNewPointsOrigin]);
  this.lineToMousePos = newState.lineToMousePos;
  
  // redraw
  this.drawAll();
  //log("loaded: " + this.history.length);
}

Polyline.prototype.joinAllSegments = function(selectedOnly)
{
  var segments = this.getSegments();
  
  if (segments.length <= 1)
    return;

  this._saveHistory();

  // join all segments
  if (!selectedOnly)
  {
    for (var i = 1; i < segments.length; i++)
      segments[i][0].custom.segmentStart = false;
  }
  
  // join segments that have at least one point selected
  else
  {
    var changed = false;
  
    for (var i = 1; i < segments.length; i++)
    {
      var points = segments[i];
      
      for (var j = 0; j < points.length; j++)
      {
        if (points[j].custom.isSelected)
        {
          points[0].custom.segmentStart = false;
          changed = true;
          break;
        }
      }
    }
    
    if (!changed)
      this._restoreHistory();
  }
  
  this.drawAll();
}

Polyline.prototype.reversePoints = function(points)
{
  if (points.length <= 1)
    return false;

  var areas = this.getContinuousAreas(points, {ignoreSegmentStarts: true, getPointIndexesLengths: true});
  
  for (var i = 0; i < areas.length; i++)
  {
    var index = areas[i].index;
    var len = areas[i].length;

    // get segment ends
    for (var j = 0; j < this.points.length; j++)
    {
      if ((j == this.points.length - 1) /* track end */ 
        || this.points[j+1].custom.segmentStart /* segment start */)
        this.points[j].segmentEnd = true;
    }

    // get parts before/to_reverse/after
    var after = this.points.splice(index + len);
    var points = this.points.splice(index, len);
    var before = this.points; 
    
    // this fixes problem with track end being end and not-segmentStart being start
    // note: there are still issues, that make reversing not reversible
    var skipTrackEnd = (!after.length && !points[0].custom.segmentStart);
    
    // set segmentStarts
    for (var j = 0; j < points.length; j++)
    {
      if (j == points.length - 1 && skipTrackEnd)
        continue;

        points[j].custom.segmentStart = !!points[j].segmentEnd;
    }

    // join
    this.points = before.concat(points.reverse(), after);
    this.points[0].custom.segmentStart = false;

    // cleanup
    for (var j = 0; j < this.points.length; j++)
      this.points[j].segmentEnd = false;
  }
  
  return true;
}

/* extracts groups of points that directly follow each other */
Polyline.prototype.getContinuousAreas = function(points, options)
{
  options = options || {};
  var areas = [];          // segments
  var indexes = [];        // indexes within points
  var indexesLengths = []; // indexes + lengths within this.point
  
  // get indexes
  for (var i = 0; i < points.length; i++)
  {
    var index = this.getIndexFromPoint(points[i]);
    
    if (index === null)
      throw ("getIndexFromPoint error within getcontinuousAreas!");
    
    points[i].continuousAreaIndex = index;
  }

  // get areas' beginnings
  for (var i = 0; i < points.length; i++)
  {
    if (!i /* first point is always a break */ 
      || (points[i].custom.segmentStart && !options.ignoreSegmentStarts) /* segment break counts as new area too */
      || points[i].continuousAreaIndex != points[i - 1].continuousAreaIndex + 1) /* following points with not following index */
      {
        points[i].continuousAreaStart = true;
        indexes.push(i);
      }
  }
  
  // create areas
  for (var i = 0; i < points.length; i++)
  {
    if (!i || points[i].continuousAreaStart)
    {
      areas.push([]);
      
      // set also automatically segment start (if set in options)
      if (options.setSegmentStart)
        points[i].custom.segmentStart = true;
        
      // mark beginning with custom parameter
      if (options.markUsingParam)
        points[i][options.custom.markUsingParam] = true;  
    }

    areas[areas.length - 1].push(points[i]);  
  }
  
  // get indexes and lengths within this.points
  if (options.getPointIndexesLengths)
  {
    for (var i = 0; i < areas.length; i++)
      indexesLengths.push({index: areas[i][0].continuousAreaIndex, length: areas[i].length});  
  }  

  // cleanup
  for (var i = 0; i < points.length; i++)
  {
    delete points[i].continuousAreaIndex;
    points[i].continuousAreaStart ? delete points[i].continuousAreaStart : "";
  }

  if (options.returnIndexes)
    return indexes;
  else if (options.getPointIndexesLengths)
    return indexesLengths;
  else
    return areas;
}

/* add point */
Polyline.prototype._insertPoint = function(point, index, insertBefore)
{
  point.custom = point.custom || {};
  point.custom.elevation = point.custom.elevation || NaN;
  point.custom.timestamp = point.custom.timestamp || NaN;
  
  // if the track is closed and we don't have custom origin set, then don't add any new points and exit
  if (this.closed && !this.addNewPointsOrigin && typeof index === 'undefined')
    return;      
  
  // no index? then add it to the end or (ALT) beginning; or to custom origin
  if (typeof index === 'undefined')
  {
    index = this.getIndexFromPoint(this.addNewPointsOrigin);

    // no alternative origin? then add point to the end
    if (index === null)
      index = insertBefore ? 0 : this.points.length - 1;
      
    // set alternative origin point to the newly entered one
    else
    {
      this._addNewPointsOrigin(point);
    }
    
    // add point before (and adjust segment start)
    if (insertBefore && this.points.length)
    {
      // if next point is segmentStart, then move segmentStart to current point
      // if current point is segmentStart, then convert the next to segmentStart
      if (!!this.points[index].custom.segmentStart != !!point.custom.segmentStart)
      {
        this.points[index].custom.segmentStart = !!point.custom.segmentStart;
        point.custom.segmentStart = !point.custom.segmentStart;
      }

      //log(point.custom.segmentStart)
    }
    
    // add point after
    else
      index++;
  }

  // add point
  this.points.splice(index, 0, point);

  // if previous point is selected, select entered one as well (unless we started a new segment or we are going backwards)
  if (index && index < this.points.length && (this.points[index].custom.segmentStart != !!point.custom.segmentStart))
    this.points[index].custom.isSelected = !!this.points[index-1].custom.isSelected;  
}

/* replace point */
Polyline.prototype._replacePoint = function(index, newPoint, mode)
{
  this._saveHistory();
  newPoint.custom = newPoint.custom || {};
  newPoint.custom.elevation = newPoint.custom.elevation || NaN;
  newPoint.custom.timestamp = newPoint.custom.timestamp || NaN;  
  
  this._removePoint(index, true, newPoint, mode);
  
  // update point
  this.updateCoords(this.points[index], newPoint.lat(), newPoint.lon());  
}

// rotate and scale all points between two points (used when dragging a marker - moving a point)
Polyline.prototype._rotateScalePointsToFit = function(points, newPoint)
{
  if (points.length == 1)
    return;
  
  var azimuthNew = points[0].azimuth(newPoint);
  var azimuthOld = points[0].azimuth(points[points.length - 1]);
  var distanceNew = points[0].distance(newPoint, {quick: true});
  var distanceOld = points[0].distance(points[points.length - 1], {quick: true});
//   log((azimuthNew - azimuthOld) + "; " + azimuthNew + "; " + azimuthOld);
//   log((distanceNew / distanceOld + "; " + distanceNew + "; " + distanceOld));
//   log(points.length);
  
  this.rotatePoints(points, azimuthNew - azimuthOld, points[0]);
  this.scalePoints(points, distanceOld ? distanceNew / distanceOld : 0, points[0]);
}

// straighten all points between two given points (used when dragging a marker - moving a point)
Polyline.prototype._straightenPoints = function(points, newPoint)
{
  if (points.length == 1)
    return;
  
  var distanceTotal = 0;
  
  for (var i = 0; i < points.length - 1; i++)
    distanceTotal += points[i].distance(points[i + 1], {quick: (points.length > 100)});

  var distanceIncremental = 0;
  var azimuth = points[0].azimuth(newPoint);
  var distance = points[0].distance(newPoint, {quick: (points.length > 100)});

  for (var i = 0; i < points.length - 1; i++)
  {
    var projDist = (distanceTotal ? (distanceIncremental / distanceTotal) : 0) * distance;
    var newPoint = points[0].projectPoint(azimuth, projDist);
    distanceIncremental += points[i].distance(points[i+1], {quick: (points.length > 100)});
    this.updateCoords(points[i], newPoint.lat(), newPoint.lon());  
  }
}

/* remove point */
Polyline.prototype._removePoint = function(index, keepSelf, newPoint, mode)
{
  // save point (will be modified by rotate/scale functions
  var p = this.points[index];
  var oldP = {lat: p.lat(), lon: p.lon()};
  
  // points after (without marker displayed)
  var i = index + 1;

  if (index < this.points.length - 1)
  {
    while(!!this.points[i].hasMarker != true) i++;
    
    if (i != index + 1)
    {
      if (!keepSelf)
        this.points.remove(index + 1 /* first without marker */, i - 1 /* last without marker */);
      else if (mode == "rotateScale")
        this._rotateScalePointsToFit(this.points.slice(index /* our point */, i + 1 /* next with marker - is origin */).reverse(), newPoint);
      else
        this._straightenPoints(this.points.slice(index /* our point */, i + 1 /* next with marker - is origin */).reverse(), newPoint);        
    }
  }
  
  // restore point
  this.updateCoords(p, oldP.lat, oldP.lon);
  
  // points before (without marker displayed)
  var i = index - 1;
  
  if (index > 0)
  {
    while(!!this.points[i].hasMarker != true) i--;
    
    if (i != index - 1)
    {
      if (!keepSelf)
        this.points.remove(i + 1 /* first without marker */, index - 1 /* last without marker */);
      else if (mode == "rotateScale")    
        this._rotateScalePointsToFit(this.points.slice(i /* with marker before - origin */, index + 1/* our point */), newPoint);
      else    
        this._straightenPoints(this.points.slice(i /* with marker before - origin */, index + 1/* our point */), newPoint);
    }
  }
  
  // restore point again
  this.updateCoords(p, oldP.lat, oldP.lon);
  
  // keep or delete main point
  if (!keepSelf)
    this.points.remove(this.getIndexFromPoint(p));    
  
  // remove polygon mode, if too few points
  if (this.points.length <= 2)
    this._setPolygon(false);
    
  // remove new points origin, if we just deleted it
  this._syncNewPointsOrigin(); 

  this.drawAll();
}

/* remove multiple points */
Polyline.prototype.removePoints = function(points)
{
  for (var i = 0; i < points.length; i++)
    this.points.remove(this.getIndexFromPoint(points[i]));
  
  if (this.points.length < 3)
    this._setPolygon(false);
    
  // remove new points origin, if we just deleted it
  this._syncNewPointsOrigin();   
}

/* remove markers (use when deactivating layer) */
Polyline.prototype.removeMarkers = function()
{
  this.markerLayer.removeAll();
}

/* remove all points */
Polyline.prototype.removeAll = function()
{
  this.layer.removeAll();
  this.helperLayer.removeAll();
  this.removeMarkers();
}

/* draw all points */
Polyline.prototype.drawAll = function(noUpdateOfTextAreA)
{
  // blurneme textarea, jinak nám při aktivním kurzoru v textarea nebudou fungovat klávesové zkratky
  // -- možná by bylo dobré vyřešit ještě nějak jinak :(
  this.$super.textArea.textArea.blur();

  // don't redraw while map is locked (may fix bug in free draw mode)
  if (this.mapIsLocked)
    return;
    
  // when no points, turn off .closed
  if (!this.points.length)
    this.closed = false;
    
  // cleanup old layers
  this.removeAll();
  var segments = this.getSegments();

  // remove helper line
  this.lineToMousePosDelete();  

  // draw track / polygon
  this.layer.disable();
  
  for (var i = 0; i < segments.length; i++)
  {
    var hidden = !!segments[i][0].custom.hideSegment;
  
    // skip segments with one point only
    if (segments[i].length <= 1)
      continue;
    
    // polygon options
    if (this._isPolygon())
    {
      var options = {color: "#00f", width: 2, outlineWidth: 2};
      var optionsHidden = {color: "#FFA500" /* orange: IE7-8 fix */, width: 3, outlineWidth: 1, opacity: 0.2};
      var optionsDisabled = {color: "#00f", width: 2, outlineWidth: 2, outlineColor: "#FFA500" /* orange: IE7-8 fix */};       
    }
    
    // track options
    else
    {
      var hidden = !!segments[i][0].custom.hideSegment;
      var options = {color: "red", width: 3, outlineWidth: 0, opacity: 0.85};
      var optionsHidden = {color: "#FFA500" /* orange: IE7-8 fix */, width: 3, outlineWidth: 0, opacity: 0.6};
      var optionsDisabled = {color: "blue", width: 3, outlineWidth: 1, opacity: 0.85};      
    }

    options = this.active ? (hidden ? optionsHidden : options) : optionsDisabled;
    var geom = new SMap.Geometry(SMap[this._isPolygon() ? "GEOMETRY_POLYGON" : "GEOMETRY_POLYLINE"], null, segments[i], options);
    geom.segmentId = i;
    geom.that = this;
    this.layer.addGeometry(geom);
  }

  this.layer.enable();
  
  // draw only polygon if layer is inactive
  if (!this.active)
    return;  

  // generate markers
  var pixelDistLimit = 50;
  var totalParsed = 0;
  this.markerLayer.disable();
  this.helperLayer.disable();

  for (var i = 0, seglen = segments.length; i < seglen; i++)
  {
    var seg = segments[i];
    
    // skip segment if the first point has hideSegment attribute
    if (seg.length && !!seg[0].custom.hideSegment)
    {
      totalParsed += seg.length
      continue;
    }
    
    // prepare points
    for (var j = 0, len = seg.length; j < len; j++)
    {
      seg[j].isInMap = seg[j].inMap(this.map);
      seg[j].pixel = seg[j].toPixel(this.map);
      seg[j].hasMarker = true;
    }

    // create arrows and mark lines between selected points
    if (seg.length >= 2)
    {
      this._colorizeSelected(seg, i);
      this._addDirectionArrows(seg, i);
    }

    // go through points and create markers for them
    for (var j = 0, len = seg.length; j < len; j++)
    {
      var point = seg[j];

      // skip points outside of viewport    
      if (j < len - 2 && !point.isInMap)
        continue;
        
      // color markers based on being selected or segment/track start/end  
      var classAdd = point.custom.isSelected ? " selected" : "";
      var segmentEnd = (i != seglen - 1 && j == len - 1);
      classAdd += segmentEnd ? " segmentEnd" : "";
      classAdd += (i && !j && !segmentEnd) ? " segmentStart" : "";
      
      var isTrackEnd = (i == seglen - 1 && j == len - 1);
      classAdd += isTrackEnd ? " trackEnd" : "";
      classAdd += (isTrackEnd && this.closed) ? " trackClosedEnd" : "";
      classAdd += (point === this.addNewPointsOrigin) ? " addNewPointsOrigin" : "";
      classAdd += (!i && !j) ? " trackStart" : "";                
 
      // create halfway point
      var next = (j == len - 1) ? (this._isPolygon() ? seg[0] : null) : seg[j + 1];
  
      if (next && point.pixel.distance(next.pixel) > 2 * pixelDistLimit)
      {
        var pointMiddle = point.projectPoint(point.azimuth(next), point.distance(next) / 2); 
        this._createMarker(pointMiddle, {className: "PolygonMarkerHalfway" + classAdd, "data-index-half": (totalParsed + j)});
      }
  
      this._createMarker(point, {className: "PolygonMarker" + classAdd, "data-index": (totalParsed + j), bubble: true});
      
      // skip following points if they are too close
      if (!this.hidePointsWhenTooClose)
        continue;
      
      var last = point;
  
      // note: check for distance to the next+1 because otherwise we would skip start of a big leap (and its halfway point as well)
      while (j < len - 3 && last.pixel.distance(seg[j+2].pixel) < pixelDistLimit /* pixel distance */
        && seg[j+1] !== this.addNewPointsOrigin)     /* always show alternative marker origin */
      {
        seg[++j].hasMarker = false;
      }
    }
    
    totalParsed += seg.length;  
  }
  
  this.markerLayer.enable();
  this.helperLayer.enable();
  
  // update text area on the right
  if (!noUpdateOfTextAreA)
    this.$super.textArea.writePolyline(this);
}

Polyline.prototype._colorizeSelected = function(seg, segmentId)
{
  // if polygon, then add first point to the end, so our function sees it
  if (this._isPolygon())
    seg.push(seg[0]);

  // add direction arrows
  for (var j = 0, len = seg.length; j < len; j++)
  {
    if (!seg[j].custom.isSelected)
      continue;
      
    // skip segments with one point only
    if (seg[j].length <= 1)
      continue;      
    
    // get continuous area
    var selectedStartId = Math.max(j - 1, 0);
    var selectedEndId = j + 1;
    
    while(j < len - 1 && seg[j+1].custom.isSelected)
      j++;
    
    var points = seg.slice(selectedStartId, j + 2);
    
    var options = {color: "#FFA500" /* orange: IE7-8 fix */, width: 4, outlineWidth: 0, outlineColor: "green", opacity: 1};
    var geom = new SMap.Geometry(SMap.GEOMETRY_POLYLINE, null, points, options);
    geom.segmentId = segmentId;  /* help identify layer on geometry click */
    geom.that = this;
    this.helperLayer.addGeometry(geom);
  }
  
  if (this._isPolygon())
    seg.pop();  
}

Polyline.prototype._addDirectionArrows = function(seg, segmentId)
{
  if (this._isPolygon())
    seg.push(seg[0]);
    
  // add direction arrows
  var totalPixelDist = 0;
  var arrowInterval = 200;
  
  for (var j = 0, len = seg.length; j < len - 1; j++)
  {
    var dist = seg[j].pixelDistance(this.map, seg[j+1]);
    
    for (var k = Math.floor(totalPixelDist / arrowInterval + 1) * arrowInterval; k < totalPixelDist + dist; k += arrowInterval)
    {
      var dist2 = seg[j].pixelDistance(this.map, seg[j+1]);  
      var arrowAzimuth = seg[j].pixelAzimuth(this.map, seg[j+1]);
      var arrowPoint = seg[j].pixelProject(this.map, arrowAzimuth, (dist ? ((k - totalPixelDist) / dist) : 0) * dist2);
  
      var arrowAngle = 23;
      var arrowLength = 15;
      
      var arrowPt2 = arrowPoint.pixelProject(this.map, arrowAzimuth + 180 + arrowAngle, arrowLength);
      var arrowPt3 = arrowPoint.pixelProject(this.map, arrowAzimuth + 180 - arrowAngle, arrowLength);      
      
      
      if (!arrowPoint.inMap(this.map))
        continue;
      
      var isSelected = (seg[j].custom.isSelected || seg[j+1].custom.isSelected);
      var options = {color: isSelected ? "#FFA500" /* orange: IE7-8 fix */ : "red", width: 6, outlineWidth: 0, outlineColor: "red", opacity: 1}; 
      var geom = new SMap.Geometry(SMap.GEOMETRY_POLYGON, null, [arrowPoint, arrowPt2, arrowPt3], options);
      geom.segmentId = segmentId;  /* help identify layer on geometry click */
      geom.that = this;
      this.helperLayer.addGeometry(geom);      
    }
    
    totalPixelDist += dist; 
  }
  
  if (this._isPolygon())
    seg.pop();  
}

/* create marker (point) */
Polyline.prototype._createMarker = function(point, options)
{
  var markerDiv = JAK.mel("div", options);

  EventUtil.addHandler(markerDiv, "contextmenu", this.onMarkerRightClick);

  var marker = new SMap.Marker(point, null, {url: markerDiv, anchor: {left: 6, top: 6}});
  marker.decorate(SMap.Marker.Feature.Draggable);
  
  markerDiv.markerId = marker.getId();
  
  // bubble
  if (options.bubble)
  {
    EventUtil.addMouseHandler(markerDiv, "mouseenter", this.onMarkerMouseEnter);
    markerDiv.that = this;
  }
  
  this.markerLayer.addMarker(marker);
}

/* deactivate layer */
Polyline.prototype.deactivate = function()
{
  this.active = false;
  
  // redraw to change line color to non-active
  this.drawAll();
  
  // remove markers
  this.removeMarkers();
  
  // remove arrows, etc.
  this.helperLayer.removeAll();
  
  // cancel free draw mode (if running)
  this.cancelFreeDrawMode();
  
  // helper line
  this.lineToMousePosDelete();
}

/* activate layer */
Polyline.prototype.activate = function()
{
  this.active = true;
  this.drawAll();
}

/* destroy layer */
Polyline.prototype.destroy = function()
{
  this.deactivate();
  this.exit();
  
  if (JAK.Browser.client != "ie" || JAK.Browser.version >= 9)
    delete this;
}

/*############################################*/

if (!window.polygon)
  window.polygon = new Polygon(m, polygon_button, settings);


// http://cycleseven.org/gps-waypoints-routes-and-tracks-the-difference

  //ShowLMCButton("cliptext", "capt", "", "basePath + "LMCButton/lmcbutton.swf");
  
 // <td><script>ShowLMCButton('http://www.lettersmarket.com/');</script></td>


