//============================================================================
//                            SMap.Control.Selection
//============================================================================

// ************
// SMap.Control.Selection - modify to right click


// set more params
encapsulate(SMap.Control.Selection.prototype, "$constructor", function(thickness)
{
  // call and backup $super()
  this.$super();
  var temp = this.$super;
  this.$super = function(){};
  
  // note: Firefox and SHIFT+right click launches app context menu (can't be overridden)
  // - customStartDrag - called on mousedown
  // - customRedraw - called on mousemove 
  // - customHandler - called on draw finish (mouse up)
  // - customFailDrag - called on finish, but when mouse movement was too small and customHandler wouldn't be triggered
  var options = (typeof thickness === 'object') ? thickness : {thickness: thickness};
  this.customHandler = options.customHandler || null;
  this.fillColor = options.fillColor || null;
  this.cursor = options.cursor || null;
  this.customId = options.customId || null;
  this.shift = options.shift || false;
  this.ctrl = options.ctrl || false;
  this.alt = options.alt || false;
  this.meta = options.meta || false;
  this.mouseButton = (typeof options.mouseButton !== 'undefined') ? options.mouseButton : JAK.Browser.mouse.right;
  this._backupRedraw = this._redraw;   // enable us to call original redraw from our functions
  this._redraw = options.customRedraw || this._redraw;
  this.customStartDrag = options.customStartDrag || null;
  this.customFailDrag = options.customFailDrag || null; // called when dragging fails (no mouse movement)

  // call main function
  this.$constructorOrig(options.thickness);
  
  // restore $super()
  this.$super = temp;
});

// start selection only on dragging content (not any controls)
encapsulate(SMap.Control.Selection.prototype, "setOwner", function(owner)
{
  this.$super(owner);
  
  // version 2014-8-23 made owner sometimes null
  if (!owner)
    return;

  var elm = owner.getContent();
  this._ec.push(JAK.Events.addListener(elm, "mousedown", this, "_mousedown"));
});

// react on right click
encapsulate(SMap.Control.Selection.prototype, "_mousedown", function(e, elm)
{ 
  // handle special keys (allowing for custom handling)
  if (this.shift != !!e.shiftKey || this.ctrl != !!e.ctrlKey || this.alt != !!e.altKey || this.meta != !!e.metaKey
  || this.mouseButton != e.button || window.selectionActive)
    return;

  // Maxthon 2 fires mousedown on mouseup (on right click with SHIFT etc. pressed), prevent it
  if (this.mouseIsDown)
    return false;

  this.mouseIsDown = true;
  
  // prevent multiple selections being drawn at a time
  window.selectionActive = true;

  // Opera
  if (JAK.Browser.client == "opera")
    EventUtil.preventDefault(e);

  // custom function on start
  if (this.customStartDrag)
    this.customStartDrag(e, elm);

  // change cursor
  this.getMap().setCursor(this.cursor || "pointer");
 
  // *** prevent context menu from appearing ***
  var that = this;
  
  this.handleMouseUp = function(e, target)
  {
    if (that._point2 !== that._point1)
    {
      // context menu stops working on repeated righ-click selections;
      // attempt to fix it by turning it off in Opera (this would show context menu in IE, at least without a CTRL/SHIFT/ALT)
      // - still needs fixing (perhaps turn this on only for non-alt+ctrl+shift stuff???
      if (JAK.Browser.client != "opera")
        EventUtil.stopPropagation(e);
      
      EventUtil.preventDefault(e);  
    }

    if (e.type == "contextmenu")
    {
      while (that.contextMenuHandlers.length)
        EventUtil.removeHandler(that.contextMenuHandlers.pop());
    }
    
    else if (e.type == "mouseup")
    {
      while (that.mouseUpMenuHandlers.length)
        EventUtil.removeHandler(that.mouseUpMenuHandlers.pop());

      // remove handler for context menu here as well
      if (that.mouseButton != JAK.Browser.mouse.right)
      {
        while (that.contextMenuHandlers.length)
          EventUtil.removeHandler(that.contextMenuHandlers.pop());
      }
  
      // call mouseup function (only if any move happened)
      if (that.movedMouse)
        that._mouseup.call(that, e, target);
    }
  }
 
  // prevent context menu bubbling
  // IE7-8 --> guess map layer index (should be 0), IE9+ - use capture mode
  this.contextMenuHandlers = [];
  
  for (var i = 0; i < elm.childNodes.length; i++)
    this.contextMenuHandlers.push(EventUtil.addHandler(elm.childNodes[i], "contextmenu", this.handleMouseUp, false));

  this.contextMenuHandlers.push(EventUtil.addHandler(document.documentElement, "contextmenu", this.handleMouseUp, true));
  
  // prevent middle click bubbling (note: don't do this in IE7-8, would cause a bug)
  this.mouseUpMenuHandlers = [];
  this.mouseUpMenuHandlers.push(EventUtil.addHandler(document.documentElement, "mouseup", this.handleMouseUp, true));

  // *** call original function (e can't be modified, so create a new one) ***
  this._mousedownOrig({button: JAK.Browser.mouse.left, ctrlKey: 1, metaKey: 0, clientX: e.clientX, clientY: e.clientY}, elm);
  
  // change fill color (if set)
  if (this.fillColor)
    this._dom.container.style.backgroundColor = this.fillColor;
});

// mouse up
encapsulate(SMap.Control.Selection.prototype, "_mouseup", function(e, elm)
{
  EventUtil.preventDefault(e);
  EventUtil.stopPropagation(e);
  this.movedMouse = false;
  this.mouseIsDown = false;  /* Maxthon 2 fix */
  
  // restore cursor
  this.getMap().setCursor(null);

  // custom handler
  if (this.customHandler)
  {
    // get left-bottom/right-top points and direction where we started
    var direction = ((this._point1.x <= this._point2.x) ? "l" : "r") + ((this._point1.y <= this._point2.y) ? "t" : "b")
    var p1 = new SMap.Pixel(Math.min(this._point1.x, this._point2.x), Math.min(this._point1.y, this._point2.y));
    var p2 = new SMap.Pixel(Math.max(this._point1.x, this._point2.x), Math.max(this._point1.y, this._point2.y));

    // original cleanup
  	this._ecTmp.forEach(JAK.Events.removeListener, JAK.Events);
  	this._ecTmp = [];
   	this._hide();    

    // make sure something is selected
    var selectedSomething = (p1.x != p2.x && p1.y != p2.y);
    
    if (selectedSomething)
      this.customHandler(p1, p2, this.customId, direction);
    else if (this.customFailDrag)
      this.customFailDrag(p1, p2, this.customId, direction);
  }
  
  // default: zoom to selection
  else
  {
    this._mouseupOrig(e, elm);
  }

  window.selectionActive = false;
});

// handle mouse move
encapsulate(SMap.Control.Selection.prototype, "_mousemove", function(e, elm)
{
	var point = SMap.Pixel.fromEvent(e, this.getMap());
	if (this._point2.distance(point) < 7) { return; }   // trochu to ohulme pro polygon, aby mousemove neprobhalo tak asto
  
  //log( this._point2.distance(point));
  

  this.movedMouse = true;
  return(this._mousemoveOrig(e, elm));
})



// jet je teba pro tuto tdu hookovat SMap.Control.Mouse (viz jinde)